@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.c,v 1.85 2009/08/19 13:12:10 phintuka Exp $
+ * $Id: config.c,v 1.103 2011/03/20 21:31:36 phintuka Exp $
  *
  */
 
@@ -22,6 +22,7 @@
 
 #include "logdefs.h"
 #include "config.h"
+#include "xine_frontend.h" // HUD_*
 
 #define STRN0CPY(dst, src) \
   do { \
@@ -175,7 +176,7 @@ const char * const config_t::s_audioEqNames[ AUDIO_EQ_count+1 ] = {
 };
 
 const char * const config_t::s_audioVisualizations[ AUDIO_VIS_count+1 ] = {
-  "none", "goom", "oscope", "fftscope", "fftgraph",
+  "none", "goom", "oscope", "fftscope", "fftgraph", "image",
   NULL
 };
 
@@ -185,6 +186,7 @@ const char * const config_t::s_audioVisualizationNames[ AUDIO_VIS_count+1 ] = {
   trNOOP("Oscilloscope"),
   trNOOP("FFT Scope"),
   trNOOP("FFT Graph"),
+  trNOOP("Image"),
   NULL
 };
 
@@ -236,12 +238,20 @@ const char * const config_t::s_osdScalings[] = {
   NULL
 };
 
+const char * const config_t::s_osdColorDepths[] = {
+  trNOOP("automatic"),
+  trNOOP("LUT8"),
+  trNOOP("TrueColor"),
+  NULL
+};
+
 const char * const config_t::s_osdSizes[] = {
   trNOOP("automatic"),
   "720x576",
   "1280x720",
   "1920x1080",
   trNOOP("custom"),
+  trNOOP("video stream"),
   NULL
 };
 
@@ -414,25 +424,18 @@ bool config_t::IsDvdImage(const char *fname)
 bool config_t::IsDvdFolder(const char *fname)
 {
   struct stat st;
-  cString buf, folder;
 
-  buf = cString::sprintf("%s/VIDEO_TS/", fname);
-  if (stat(buf, &st) == 0) {
-    folder = buf;
-  } else {
-    buf = cString::sprintf("%s/video_ts/", fname);
-    if (stat(buf, &st) == 0)
-      folder = buf;
-    else
+  cString folder = cString::sprintf("%s/VIDEO_TS/", fname);
+  if (stat(folder, &st) != 0) {
+    folder = cString::sprintf("%s/video_ts/", fname);
+    if (stat(folder, &st) != 0)
       return false;
   }
 
-  buf = cString::sprintf("%s/video_ts.ifo", *folder);
-  if (stat(buf, &st) == 0)
+  if (stat(cString::sprintf("%s/video_ts.ifo", *folder), &st) == 0)
     return true;
 
-  buf = cString::sprintf("%s/VIDEO_TS.IFO", *folder);
-  if (stat(buf, &st) == 0)
+  if (stat(cString::sprintf("%s/VIDEO_TS.IFO", *folder), &st) == 0)
     return true;
 
   return false;
@@ -441,29 +444,21 @@ bool config_t::IsDvdFolder(const char *fname)
 bool config_t::IsBluRayFolder(const char *fname)
 {
   struct stat st;
-  cString buf, folder;
 
-  buf = cString::sprintf("%s/BDMV/", fname);
-  if (stat(buf, &st) == 0) {
-    folder = buf;
-  } else {
-    buf = cString::sprintf("%s/bdmv/", fname);
-    if (stat(buf, &st) == 0)
-      folder = buf;
-    else
+  cString folder = cString::sprintf("%s/BDMV/", fname);
+  if (stat(folder, &st) != 0) {
+    folder = cString::sprintf("%s/bdmv/", fname);
+    if (stat(folder, &st) != 0)
       return false;
   }
 
-  buf = cString::sprintf("%s/MovieObject.bdmv", *folder);
-  if (stat(buf, &st) == 0)
+  if (stat(cString::sprintf("%s/MovieObject.bdmv", *folder), &st) == 0)
     return true;
 
-  buf = cString::sprintf("%s/movieobject.bdmv", *folder);
-  if (stat(buf, &st) == 0)
+  if (stat(cString::sprintf("%s/movieobject.bdmv", *folder), &st) == 0)
     return true;
 
-  buf = cString::sprintf("%s/MOVIEOBJECT.BDMV", *folder);
-  if (stat(buf, &st) == 0)
+  if (stat(cString::sprintf("%s/MOVIEOBJECT.BDMV", *folder), &st) == 0)
     return true;
 
   return false;
@@ -474,8 +469,19 @@ cString config_t::AutocropOptions(void)
   if (!autocrop)
     return NULL;
 
-  return cString::sprintf("enable_autodetect=%d,soft_start=%d,stabilize=%d,enable_subs_detect=%d",
-			  autocrop_autodetect, autocrop_soft, autocrop_fixedsize, autocrop_subs);
+  return cString::sprintf(
+                  "enable_autodetect=%d,autodetect_rate=%d,"
+                  "soft_start=%d,soft_start_step=%d,"
+                  "stabilize=%d,stabilize_time=%d,"
+                  "enable_subs_detect=%d,subs_detect_lifetime=%d,subs_detect_stabilize_time=%d,"
+                  "logo_width=%d,overscan_compensate=%d,use_driver_crop=%d,"
+                  "use_avards_analysis=%d,bar_tone_tolerance=%d",
+			  autocrop_autodetect, autocrop_autodetect_rate,
+			  autocrop_soft, autocrop_soft_start_step,
+			  autocrop_fixedsize, autocrop_stabilize_time,
+			  autocrop_subs, autocrop_subs_detect_lifetime, autocrop_subs_detect_stabilize_time,
+			  autocrop_logo_width, autocrop_overscan_compensate, autocrop_use_driver_crop,
+			  autocrop_use_avards_analysis, autocrop_bar_tone_tolerance);
 }
 
 cString config_t::SwScaleOptions(void)
@@ -545,6 +551,7 @@ config_t::config_t() {
   memset(audio_equalizer,0,sizeof(audio_equalizer));
   strn0cpy(audio_visualization, "goom", sizeof(audio_visualization));
   strn0cpy(audio_vis_goom_opts, "fps:25,width:720,height:576", sizeof(audio_vis_goom_opts));
+  strn0cpy(audio_vis_image_mrl, "file:/usr/share/xine/visuals/default.avi", sizeof(audio_vis_image_mrl));
 
   headphone = 0;
   audio_upmix = 0;
@@ -580,9 +587,12 @@ config_t::config_t() {
   osd_height           = 576;
   osd_width_auto       = 0;
   osd_height_auto      = 0;
+  osd_color_depth      = OSD_DEPTH_auto;
   osd_mixer            = OSD_MIXER_FULL;
   osd_scaling          = OSD_SCALING_NEAREST;
+  osd_spu_scaling      = OSD_SCALING_NEAREST;
   hud_osd              = 0;
+  opengl               = 0;
 
   osd_blending             = OSD_BLENDING_SOFTWARE;
   osd_blending_lowresvideo = OSD_BLENDING_HARDWARE;
@@ -599,11 +609,23 @@ config_t::config_t() {
   height         = 576;
   scale_video    = 0;
   field_order    = 0;
+
   autocrop       = 0;
   autocrop_autodetect = 1;
+  autocrop_autodetect_rate = 4;
   autocrop_soft  = 1;
+  autocrop_soft_start_step  = 4;
   autocrop_fixedsize = 1;
+  autocrop_stabilize_time = (5*25);
   autocrop_subs  = 1;
+  autocrop_subs_detect_lifetime = (60*25);
+  autocrop_subs_detect_stabilize_time = 12;
+  autocrop_logo_width = 20;
+  autocrop_use_driver_crop = 0;
+  autocrop_use_avards_analysis = 0;
+  autocrop_overscan_compensate = 0;
+  autocrop_bar_tone_tolerance = 0;
+
 
   swscale               = 0;    // enable/disable
   swscale_change_aspect = 0;    // change video aspect ratio
@@ -640,6 +662,7 @@ config_t::config_t() {
   remote_local_ip[0] = 0;   // bind locally to this IP - undefined -> any/all
 
   use_x_keyboard = 1;
+  window_id      = WINDOW_ID_NONE;
 
   // video settings
   ibp_trickspeed = 1;
@@ -662,12 +685,15 @@ config_t::config_t() {
   ff_h264_speed_over_accurancy = FF_H264_SPEED_OVER_ACCURACY_auto;
   ff_h264_skip_loop_filter     = FF_H264_SKIP_LOOPFILTER_auto;
 
+  strn0cpy(media_root_dir,    "/",            sizeof(media_root_dir));
   strn0cpy(browse_files_dir,  VideoDirectory, sizeof(browse_files_dir));
   strn0cpy(browse_music_dir,  VideoDirectory, sizeof(browse_music_dir));
   strn0cpy(browse_images_dir, VideoDirectory, sizeof(browse_images_dir));
+  show_hidden_files = 0;
   cache_implicit_playlists = 1;
   enable_id3_scanner = 1;
   dvd_arrow_keys_control_playback = 1;
+  media_menu_items = ~0;
 
   main_menu_mode = ShowMenu;
   force_primary_device = 0;
@@ -694,9 +720,11 @@ bool config_t::ProcessArgs(int argc, char *argv[])
 
   static const struct option long_options[] = {
       { "fullscreen",   no_argument,       NULL, 'f' },
-      { "hud",          no_argument,       NULL, 'D' },
+      { "hud",          optional_argument, NULL, 'D' },
+      { "opengl",       no_argument,       NULL, 'O' },
       { "width",        required_argument, NULL, 'w' },
       { "height",       required_argument, NULL, 'h' },
+      { "geometry",     required_argument, NULL, 'g' },
       //{ "xkeyboard",    no_argument,       NULL, 'k' },
       //{ "noxkeyboard",  no_argument,       NULL, 'K' },
       { "local",        required_argument, NULL, 'l' },
@@ -705,11 +733,12 @@ bool config_t::ProcessArgs(int argc, char *argv[])
       { "audio",        required_argument, NULL, 'A' },
       { "video",        required_argument, NULL, 'V' },
       { "display",      required_argument, NULL, 'd' },
+      { "window",       required_argument, NULL, 'W' },
       { "post",         required_argument, NULL, 'P' },
       { "config",       required_argument, NULL, 'C' },
       { "primary",      no_argument,       NULL, 'p' },
-      { "exit-on-close",no_argument,     NULL, 'c' },
-      { NULL }
+      { "exit-on-close",no_argument,       NULL, 'c' },
+      { NULL,           no_argument,       NULL,  0  }
     };
 
   int c;
@@ -717,19 +746,47 @@ bool config_t::ProcessArgs(int argc, char *argv[])
     switch (c) {
     case 'd': ProcessArg("Video.Port", optarg);
               break;
+    case 'W': ProcessArg("X11.WindowId", optarg);
+              break;
     case 'f': ProcessArg("Fullscreen", "1");
               break;
     case 'D': ProcessArg("X11.HUDOSD", "1");
+              if (optarg && strstr(optarg, "xshape")) {
+                ProcessArg("XShapeHUDOSD", "1");
+#ifndef HAVE_XSHAPE
+                LOGMSG("XShape HUD OSD not supported\n");
+#endif
+              }
+              if (optarg && strstr(optarg, "opengl")) {
+                ProcessArg("OpenglHUDOSD", "1");
+#ifndef HAVE_OPENGL
+                LOGMSG("OpenGL HUD OSD not supported\n");
+#endif
+              }
 #ifndef HAVE_XRENDER
               LOGMSG("HUD OSD not supported\n");
 #endif
               break;
-    case 'w': ProcessArg("Fullscreen", "0");
+    case 'O': ProcessArg("OpenglAlways", "1");
+#ifndef HAVE_OPENGL
+              LOGMSG("OpenGL HUD OSD not supported\n");
+#endif
+              break;
+    case 'w': //ProcessArg("Fullscreen", "0");
               ProcessArg("X11.WindowWidth", optarg);
               break;
-    case 'h': ProcessArg("Fullscreen", "0");
+    case 'h': //ProcessArg("Fullscreen", "0");
               ProcessArg("X11.WindowHeight", optarg);
               break;
+    case 'g': {
+                int _width = width, _height = height, _xpos = 0, _ypos = 0;
+                sscanf (optarg, "%dx%d+%d+%d", &_width, &_height, &_xpos, &_ypos);
+                ProcessArg("X11.WindowWidth",  *cString::sprintf("%d", _width));
+                ProcessArg("X11.WindowHeight", *cString::sprintf("%d", _height));
+                ProcessArg("X11.XPos",         *cString::sprintf("%d", _xpos));
+                ProcessArg("X11.YPos",         *cString::sprintf("%d", _ypos));
+              }
+              break;
     //case 'k': ProcessArg("X11.UseKeyboard", "1");
     //          break;
     //case 'K': ProcessArg("X11.UseKeyboard", "0");
@@ -800,10 +857,16 @@ bool config_t::SetupParse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "DisplayAspect"))      display_aspect = strstra(Value, s_aspects, 0);
   else if (!strcasecmp(Name, "ForcePrimaryDevice")) force_primary_device = atoi(Value);
 
+  else if (!strcasecmp(Name, "X11.WindowId"))     window_id = (!strcmp(Value, "root")) ? WINDOW_ID_ROOT : atoi(Value);
   else if (!strcasecmp(Name, "X11.WindowWidth"))  width = atoi(Value);
   else if (!strcasecmp(Name, "X11.WindowHeight")) height = atoi(Value);
+  else if (!strcasecmp(Name, "X11.XPos"))         xpos = atoi(Value);
+  else if (!strcasecmp(Name, "X11.YPos"))         ypos = atoi(Value);
   else if (!strcasecmp(Name, "X11.UseKeyboard"))  use_x_keyboard = atoi(Value);
-  else if (!strcasecmp(Name, "X11.HUDOSD"))       hud_osd = atoi(Value);
+  else if (!strcasecmp(Name, "X11.HUDOSD"))       hud_osd |= (atoi(Value) ? HUD_COMPOSITE : 0);
+  else if (!strcasecmp(Name, "X11.OpenglAlways")) opengl = atoi(Value);
+  else if (!strcasecmp(Name, "X11.OpenglHUDOSD")) hud_osd |= (atoi(Value) ? HUD_OPENGL : 0);
+  else if (!strcasecmp(Name, "X11.XShapeHUDOSD")) hud_osd |= (atoi(Value) ? HUD_XSHAPE : 0);
 
   else if (!strcasecmp(Name, "Audio.Driver")) STRN0CPY(audio_driver, Value);
   else if (!strcasecmp(Name, "Audio.Port"))   STRN0CPY(audio_port, Value);
@@ -812,6 +875,7 @@ bool config_t::SetupParse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "Audio.Delay"))  audio_delay = atoi(Value);
   else if (!strcasecmp(Name, "Audio.Compression")) audio_compression = atoi(Value);
   else if (!strcasecmp(Name, "Audio.Visualization.GoomOpts")) STRN0CPY(audio_vis_goom_opts, Value);
+  else if (!strcasecmp(Name, "Audio.Visualization.ImageMRL")) STRN0CPY(audio_vis_image_mrl, Value);
   else if (!strcasecmp(Name, "Audio.Visualization")) STRN0CPY(audio_visualization, Value);
   else if (!strcasecmp(Name, "Audio.Surround"))  audio_surround = atoi(Value);
   else if (!strcasecmp(Name, "Audio.Upmix"))     audio_upmix = atoi(Value);
@@ -822,8 +886,10 @@ bool config_t::SetupParse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "OSD.Size"))           osd_size = strstra(Value, s_osdSizes, 0);
   else if (!strcasecmp(Name, "OSD.Width"))          osd_width = atoi(Value);
   else if (!strcasecmp(Name, "OSD.Height"))         osd_height = atoi(Value);
+  else if (!strcasecmp(Name, "OSD.ColorDepth"))     osd_color_depth = strstra(Value, s_osdColorDepths, 0);
   else if (!strcasecmp(Name, "OSD.LayersVisible"))  osd_mixer = atoi(Value);
   else if (!strcasecmp(Name, "OSD.Scaling"))        osd_scaling = atoi(Value);
+  else if (!strcasecmp(Name, "OSD.ScalingSPU"))     osd_spu_scaling = atoi(Value);
   else if (!strcasecmp(Name, "OSD.Blending"))       osd_blending = atoi(Value);
   else if (!strcasecmp(Name, "OSD.BlendingLowRes")) osd_blending_lowresvideo = atoi(Value);
 #if 1
@@ -866,6 +932,7 @@ bool config_t::SetupParse(const char *Name, const char *Value)
 
   else if (!strcasecmp(Name, "Video.Driver"))      STRN0CPY(video_driver, Value);
   else if (!strcasecmp(Name, "Video.Port"))        STRN0CPY(video_port, Value);
+
   else if (!strcasecmp(Name, "Video.Scale"))       scale_video = atoi(Value);
   else if (!strcasecmp(Name, "Video.DeinterlaceOptions")) STRN0CPY(deinterlace_opts, Value);
   else if (!strcasecmp(Name, "Video.Deinterlace")) STRN0CPY(deinterlace_method, Value);
@@ -873,9 +940,19 @@ bool config_t::SetupParse(const char *Name, const char *Value)
 
   else if (!strcasecmp(Name, "Video.AutoCrop"))    autocrop = atoi(Value);
   else if (!strcasecmp(Name, "Video.AutoCrop.AutoDetect"))   autocrop_autodetect = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.AutoDetectRate"))   autocrop_autodetect_rate = atoi(Value);
   else if (!strcasecmp(Name, "Video.AutoCrop.SoftStart"))    autocrop_soft = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.SoftStartStep"))    autocrop_soft_start_step = atoi(Value);
   else if (!strcasecmp(Name, "Video.AutoCrop.FixedSize"))    autocrop_fixedsize = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.StabilizeTime"))    autocrop_stabilize_time = atoi(Value);
   else if (!strcasecmp(Name, "Video.AutoCrop.DetectSubs"))   autocrop_subs = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.SubsDetectLifetime"))   autocrop_subs_detect_lifetime = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.SubsDetectStabilizeTime"))   autocrop_subs_detect_stabilize_time = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.LogoWidth"))   autocrop_logo_width = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.UseDriverCrop"))   autocrop_use_driver_crop = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.UseAvardsAnalysis"))   autocrop_use_avards_analysis = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.OverscanCompensate"))   autocrop_overscan_compensate = atoi(Value);
+  else if (!strcasecmp(Name, "Video.AutoCrop.BarToneTolerance"))   autocrop_bar_tone_tolerance = atoi(Value);
 
   else if (!strcasecmp(Name, "Video.SwScale"))           swscale = atoi(Value);
   else if (!strcasecmp(Name, "Video.SwScale.Aspect"))    swscale_change_aspect = atoi(Value);
@@ -917,12 +994,15 @@ bool config_t::SetupParse(const char *Name, const char *Value)
   else if (!strcasecmp(Name, "Post.denoise3d.chroma"))  denoise3d_chroma = atoi(Value);
   else if (!strcasecmp(Name, "Post.denoise3d.time"))    denoise3d_time   = atoi(Value);
 
+  else if (!strcasecmp(Name, "Media.RootDir"))           STRN0CPY(media_root_dir, Value);
   else if (!strcasecmp(Name, "Media.BrowseFilesDir"))    STRN0CPY(browse_files_dir, Value);
   else if (!strcasecmp(Name, "Media.BrowseMusicDir"))    STRN0CPY(browse_music_dir, Value);
   else if (!strcasecmp(Name, "Media.BrowseImagesDir"))   STRN0CPY(browse_images_dir, Value);
+  else if (!strcasecmp(Name, "Media.ShowHiddenFiles"))   show_hidden_files = atoi(Value);
   else if (!strcasecmp(Name, "Media.CacheImplicitPlaylists")) cache_implicit_playlists = atoi(Value);
   else if (!strcasecmp(Name, "Media.EnableID3Scanner"))  enable_id3_scanner = atoi(Value);
   else if (!strcasecmp(Name, "Media.DVD.ArrowKeysControlPlayback")) dvd_arrow_keys_control_playback = atoi(Value);
+  else if (!strcasecmp(Name, "Media.MenuItems"))         media_menu_items = atoi(Value);
 
   else if (!strcasecmp(Name, "Playlist.Tracknumber")) playlist_tracknumber = atoi(Value);
   else if (!strcasecmp(Name, "Playlist.Artist"))      playlist_artist = atoi(Value);
