@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: config.h,v 1.64 2009/08/19 13:12:10 phintuka Exp $
+ * $Id: config.h,v 1.78 2011/03/20 21:31:36 phintuka Exp $
  *
  */
 
@@ -119,7 +119,8 @@
 
 #define AUDIO_VIS_NONE   0
 #define AUDIO_VIS_GOOM   1
-#define AUDIO_VIS_count  5
+#define AUDIO_VIS_IMAGE  5
+#define AUDIO_VIS_count  6
 
 /* speaker arrangements: xine, audio_out_alsa.c */
 #define SPEAKERS_MONO          0
@@ -144,6 +145,12 @@
 #define OSD_BLENDING_HARDWARE   1  // xine-lib "unscaled osd"
 #define OSD_BLENDING_count      2
 
+// OSD depth
+#define OSD_DEPTH_auto          0
+#define OSD_DEPTH_LUT8          1
+#define OSD_DEPTH_TRUECOLOR     2
+#define OSD_DEPTH_count         3
+
 // OSD layers mixing
 #define OSD_MIXER_NONE          0
 #define OSD_MIXER_GRAY          1
@@ -164,7 +171,18 @@
 #define OSD_SIZE_1280x720       2
 #define OSD_SIZE_1920x1080      3
 #define OSD_SIZE_custom         4
-#define OSD_SIZE_count          5
+#define OSD_SIZE_stream         5
+#define OSD_SIZE_count          6
+
+// Media player menu (bitmask)
+#define MEDIA_MENU_FILES        (1<<0)
+#define MEDIA_MENU_MUSIC        (1<<1)
+#define MEDIA_MENU_IMAGES       (1<<2)
+#define MEDIA_MENU_DVD          (1<<3)
+#define MEDIA_MENU_CD           (1<<4)
+#define MEDIA_MENU_BLURAY       (1<<5)
+#define MEDIA_MENU_VIDEO_SETUP  (1<<6)
+#define MEDIA_MENU_AUDIO_SETUP  (1<<7)
 
 // Video decoder
 #define DECODER_MPEG2_auto       0 /* use value from frontend config_xineliboutput */
@@ -234,6 +252,7 @@ class config_t {
     static const char * const s_osdMixers              [OSD_MIXER_count     + 1];
     static const char * const s_osdScalings            [OSD_SCALING_count   + 1];
     static const char * const s_osdSizes               [OSD_SIZE_count      + 1];
+    static const char * const s_osdColorDepths         [OSD_DEPTH_count     + 1];
     static const char * const s_decoders_MPEG2         [DECODER_MPEG2_count + 1];
     static const char * const s_decoders_H264          [DECODER_H264_count  + 1];
     static const char * const s_ff_skip_loop_filters   [FF_H264_SKIP_LOOPFILTER_count + 1];
@@ -261,11 +280,14 @@ class config_t {
     int  modeswitch;
     int  width;
     int  height;
+    int  xpos;
+    int  ypos;
     int  display_aspect;
     int  scale_video;
     int  field_order;
     int  exit_on_close;    // Terminate VDR when local frontend is closed
     int  use_x_keyboard;   // Use X11 keyboard to control VDR (console kbd is handled by VDR)
+    int  window_id;        // use existing X11 window
 
     // Audio settings
     int  speaker_type;
@@ -297,9 +319,12 @@ class config_t {
     int  osd_height;
     int  osd_width_auto;
     int  osd_height_auto;
+    int  osd_color_depth;
     int  osd_mixer;                // show multiple OSD layers
     int  osd_scaling;              // OSD scaling mode: off, nearest, bilinear
+    int  osd_spu_scaling;          // SPU OSD scaling mode: off, nearest, bilinear
     int  hud_osd;                  // head up display OSD
+    int  opengl;                   // use opengl acceleration for video and HUD OSD
     int  osd_blending;             // OSD blending method
     int  osd_blending_lowresvideo; // Use hardware blending for low-resolution video
     int  alpha_correction;
@@ -308,9 +333,11 @@ class config_t {
     int  dvb_subtitles;            // send DVB subtitles in data stream (decode+display using xine-lib or external media player)
 
     // Media player
+    char media_root_dir[4096];     // restrict file browser
     char browse_files_dir[4096];
     char browse_music_dir[4096];
     char browse_images_dir[4096];
+    int  show_hidden_files;
     int  cache_implicit_playlists; // used in playlist.c
     int  enable_id3_scanner;       // used in playlist.c
     int  subtitle_vpos;            // used in media player. Not saved !
@@ -318,10 +345,12 @@ class config_t {
     int  playlist_artist;
     int  playlist_album;
     int  dvd_arrow_keys_control_playback;
+    uint media_menu_items;         // enabled items in media player menu (bitmask)
 
     // Audio visualization
     char audio_visualization[64];
     char audio_vis_goom_opts[256];
+    char audio_vis_image_mrl[4096];
 
     // deinterlacing post plugin
     char deinterlace_method[32];
@@ -335,9 +364,19 @@ class config_t {
     // automatic 4:3 letterbox -> 16:9 cropping post plugin
     int  autocrop;            // enable / disable
     int  autocrop_autodetect;
+    int  autocrop_autodetect_rate;
     int  autocrop_soft;
+    int  autocrop_soft_start_step;
     int  autocrop_fixedsize;
+    int  autocrop_stabilize_time;
     int  autocrop_subs;
+    int  autocrop_subs_detect_lifetime;
+    int  autocrop_subs_detect_stabilize_time;
+    int  autocrop_logo_width;
+    int  autocrop_use_driver_crop;
+    int  autocrop_use_avards_analysis;
+    int  autocrop_overscan_compensate;
+    int  autocrop_bar_tone_tolerance;
 
     // (video) software scaling
     int  swscale;               // enable/disable
@@ -407,13 +446,13 @@ class config_t {
     bool SetupParse(const char *Name, const char *Value);
     bool ProcessArgs(int argc, char *argv[]);
 
-    bool IsImageFile(const char *);
-    bool IsAudioFile(const char *);
-    bool IsVideoFile(const char *);
-    bool IsPlaylistFile(const char *);
-    bool IsDvdFolder(const char *);
-    bool IsBluRayFolder(const char *);
-    bool IsDvdImage(const char *);
+    static bool IsImageFile(const char *);
+    static bool IsAudioFile(const char *);
+    static bool IsVideoFile(const char *);
+    static bool IsPlaylistFile(const char *);
+    static bool IsDvdFolder(const char *);
+    static bool IsBluRayFolder(const char *);
+    static bool IsDvdImage(const char *);
 
     cString AutocropOptions(void);
     cString SwScaleOptions(void);
