@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: device.c,v 1.90 2009/08/19 17:08:08 phintuka Exp $
+ * $Id: device.c,v 1.57.2.11 2009/09/14 13:09:23 phintuka Exp $
  *
  */
 
@@ -195,6 +195,12 @@ cXinelibDevice::cXinelibDevice()
   m_ac3Present  = false;
   m_spuPresent  = false;
 
+#if VDRVERSNUM < 10515
+  m_CurrentDvdSpuTrack = ttXSubtitleNone;
+  m_ForcedDvdSpuTrack = false;
+  ClrAvailableDvdSpuTracks();
+#endif
+
   memset(m_MetaInfo, 0, sizeof(m_MetaInfo));
     
   m_PlayMode = pmNone;
@@ -205,6 +211,7 @@ cXinelibDevice::cXinelibDevice()
   m_TrickSpeedPts = 0;
   m_TrickSpeedMode = 0;
   m_TrickSpeedDelay = 0;
+  m_VDR_TrickSpeedIBP = 0;
   m_SkipAudio   = false;
   m_PlayingFile = pmNone;
   m_StreamStart = true;
@@ -214,8 +221,6 @@ cXinelibDevice::cXinelibDevice()
   m_h264 = false;
 
   m_VideoSize = (video_size_t*)calloc(1, sizeof(video_size_t));
-
-  TsBufferClear();
 }
 
 cXinelibDevice::~cXinelibDevice() 
@@ -568,6 +573,10 @@ void cXinelibDevice::StopOutput(void)
   Clear();
   ForEach(m_clients, &cXinelibThread::QueueBlankDisplay);
   ForEach(m_clients, &cXinelibThread::SetNoVideo, false);
+#if VDRVERSNUM < 10515
+  ClrAvailableDvdSpuTracks();
+  m_ForcedDvdSpuTrack = false;
+#endif
 }
 
 void cXinelibDevice::SetTvMode(cChannel *Channel)
@@ -575,10 +584,6 @@ void cXinelibDevice::SetTvMode(cChannel *Channel)
   TRACEF("cXinelibDevice::SetTvMode");
   TRACK_TIME(250);
 
-#if VDRVERSNUM >= 10701
-  m_PatPmtParser.Reset();
-#endif
-
   m_RadioStream = false;
   if (Channel && !Channel->Vpid() && (Channel->Apid(0) || Channel->Apid(1)))
     m_RadioStream = true;
@@ -641,6 +646,9 @@ bool cXinelibDevice::SetPlayMode(ePlayMode PlayMode)
   m_ac3Present = false;
   m_spuPresent = false;
 
+#if VDRVERSNUM < 10515
+  ClrAvailableDvdSpuTracks();
+#endif
   m_PlayMode = PlayMode;
 
   TrickSpeed(-1);
@@ -671,6 +679,25 @@ bool cXinelibDevice::SetPlayMode(ePlayMode PlayMode)
 #define trs_NoAudio    0x08  // no audio in trick speed mode
 #define trs_PTS_check  0x80  // detect in PlayVideo if PTS must be recalculated
 
+bool cXinelibDevice::HasIBPTrickSpeed(void)
+{
+  TRACEF("cXinelibDevice::HasIBPTrickSpeed");
+
+#ifndef DEVICE_SUPPORTS_IBP_TRICKSPEED
+#warning VDR has not been patched for smooth fast forward. Disabling smooth fast forward.
+  return false;
+#else
+  m_VDR_TrickSpeedIBP = true;
+
+  return xc.ibp_trickspeed;
+#endif
+}
+
+bool cXinelibDevice::UseIBPTrickSpeed(void) 
+{ 
+  return m_VDR_TrickSpeedIBP && xc.ibp_trickspeed; 
+}
+
 void cXinelibDevice::TrickSpeed(int Speed) 
 {
   TRACEF("cXinelibDevice::TrickSpeed");
@@ -776,8 +803,6 @@ void cXinelibDevice::Clear(void)
   TRACEF("cXinelibDevice::Clear");
   TRACK_TIME(100);
 
-  TsBufferClear();
-
   m_StreamStart = true;
   m_h264 = false;
   m_FreeBufs = 0;
@@ -800,7 +825,6 @@ void cXinelibDevice::Freeze(void)
 {
   TRACEF("cXinelibDevice::Freeze");
 
-  TsBufferFlush();
   TrickSpeed(0);
 }
 
@@ -820,8 +844,6 @@ bool cXinelibDevice::Flush(int TimeoutMs)
   TRACEF("cXinelibDevice::Flush");
   TRACK_TIME(500);
 
-  TsBufferFlush();
-
   if(m_TrickSpeed == 0) {
     ForEach(m_clients, &cXinelibThread::SetLiveMode, false);
     TrickSpeed(-1);
@@ -903,9 +925,6 @@ bool cXinelibDevice::PlayFile(const char *FileName, int Position,
 
 int cXinelibDevice::PlayTrickSpeed(const uchar *buf, int length) 
 {
-#if VDRVERSNUM >= 10705
-  return 0;
-#endif
   if(abs(m_TrickSpeed) > 1 && (m_TrickSpeedMode & trs_I_frames)) {
     uint8_t PictureType = pes_get_picture_type(buf, length);
 #ifdef LOG_TRICKSPEED
@@ -974,7 +993,7 @@ int cXinelibDevice::PlayTrickSpeed(const uchar *buf, int length)
 	  m_TrickSpeedMode = trs_I_frames | trs_PTS_recalc | trs_NoAudio;	  
 	} else {
 	  LOGTRICKSPEED("    Detected fast forward mode");
-          if(xc.ibp_trickspeed)
+          if(UseIBPTrickSpeed())
             m_TrickSpeedMode = trs_IPB_frames;
           else
             m_TrickSpeedMode = trs_I_frames;
@@ -1081,112 +1100,6 @@ int cXinelibDevice::PlayAny(const uchar *buf, int length)
   return length;
 }
 
-#if VDRVERSNUM >= 10701
-/*
- * hook to PlayTs() to get PAT and PMT
- */
-int cXinelibDevice::PlayTs(const uchar *Data, int Length, bool VideoOnly)
-{
-  if (Length > TS_SIZE) Length = TS_SIZE;
-
-  if (Length == TS_SIZE && TsHasPayload(Data)) {
-    int PayloadOffset = TsPayloadOffset(Data);
-    if (PayloadOffset < Length) {
-      int Pid = TsPid(Data);
-      if (Pid == 0) {
-#if VDRVERSNUM >= 10708
-        m_PatPmtParser.Reset();
-#endif
-#if VDRVERSNUM >= 10704
-        m_PatPmtParser.ParsePat(Data, Length);
-#else
-        m_PatPmtParser.ParsePat(Data + PayloadOffset, Length - PayloadOffset);
-#endif
-        //LOGDBG("Got PAT: PMT pid = %d", m_PatPmtParser.PmtPid());
-        if (m_server)
-          m_server->SetHeader(Data, Length, true);
-        PlayTsAny(Data, Length);
-      } else if (Pid == m_PatPmtParser.PmtPid()) {
-#if VDRVERSNUM >= 10704
-        m_PatPmtParser.ParsePmt(Data, Length);
-#else
-        m_PatPmtParser.ParsePmt(Data + PayloadOffset, Length - PayloadOffset);
-#endif
-        m_h264 = (m_PatPmtParser.Vtype() == 0x1b); /* ISO_14496_PART10_VIDEO */
-        //LOGDBG("Got PMT packet, h264 = %d", m_h264?1:0);
-        if (m_server)
-          m_server->SetHeader(Data, Length);
-#ifndef NO_HACKS
-        ForEach(m_clients, &cXinelibThread::SetHDMode, m_h264);
-#endif
-        PlayTsAny(Data, Length);
-        TsBufferFlush();
-      }
-    }
-  } else if (!Data) {
-    TsBufferFlush();
-  }
-
-  return cDevice::PlayTs(Data, Length, VideoOnly);
-}
-
-int cXinelibDevice::TsBufferFlush(void)
-{
-  if (m_TsBufSize) {
-    int n;
-    if ((n = PlayAny(m_TsBuf, m_TsBufSize)) == (int)m_TsBufSize) {
-      m_TsBufSize = 0;
-      return n;
-    }
-    if (n)
-      LOGMSG("cXinelibDevice::TsBufferFlush: error: cache not flushed (%d %d)", n, m_TsBufSize);
-    errno = EAGAIN;
-  }
-  return 0;
-}
-
-int cXinelibDevice::PlayTsAny(const uchar *buf, int length)
-{
-  if (!DATA_IS_TS(buf))
-    LOGMSG("PlayTsAny(): TS SYNC byte missing !");
-  if (length != TS_SIZE)
-    LOGMSG("PlayTsAny(): length == %d !", length);
-
-  // cache full ? try to flush it
-  if (m_TsBufSize >= 2048)
-    if (!TsBufferFlush())
-      return 0;
-
-  // add packet to cache
-  memcpy(m_TsBuf + m_TsBufSize, buf, length);
-  m_TsBufSize += length;
-
-  // time to flush ?
-  if (m_TsBufSize >= 2048-TS_SIZE-1)
-    TsBufferFlush();
-
-  return length;
-}
-
-int cXinelibDevice::PlayTsSubtitle(const uchar *Data, int Length)
-{
-  if (!xc.dvb_subtitles)
-    return cDevice::PlayTsSubtitle(Data, Length);
-
-  return PlayTsAny(Data, Length);
-}
-
-int cXinelibDevice::PlayTsAudio(const uchar *Data, int Length)
-{
-  return PlayTsAny(Data, Length);
-}
-
-int cXinelibDevice::PlayTsVideo(const uchar *Data, int Length)
-{
-  return PlayTsAny(Data, Length);
-}
-#endif // VDRVERSNUM >= 10701
-
 int cXinelibDevice::PlayVideo(const uchar *buf, int length) 
 {
   TRACEF("cXinelibDevice::PlayVideo");
@@ -1264,17 +1177,11 @@ void cXinelibDevice::StillPicture(const uchar *Data, int Length)
   bool isPes   = DATA_IS_PES(Data) && ((Data[3] & 0xF0) == 0xE0);
   bool isMpeg1 = isPes && ((Data[6] & 0xC0) != 0x80);
   bool isH264  = isPes && pes_is_frame_h264(Data, Length);
-#if VDRVERSNUM >= 10701
-  bool isTs    = DATA_IS_TS(Data);
-#endif
-
   int i;
 
   if(m_PlayingFile && (m_PlayingFile == pmAudioVideo || m_PlayingFile == pmVideoOnly))
     return;
 
-  TsBufferFlush();
-
   ForEach(m_clients, &cXinelibThread::Clear);
   ForEach(m_clients, &cXinelibThread::SetNoVideo, false);
   ForEach(m_clients, &cXinelibThread::SetLiveMode, false);
@@ -1290,10 +1197,6 @@ void cXinelibDevice::StillPicture(const uchar *Data, int Length)
 	      &mmin<int>, Length);
     } else if(isPes) {
       /*cDevice::*/PlayPes(Data, Length, m_SkipAudio);
-#if VDRVERSNUM >= 10701
-    } else if(isTs) {
-      /*cDevice::*/PlayTs(Data, Length, m_SkipAudio);
-#endif
     } else {
       ForEach(m_clients, &cXinelibThread::Play_Mpeg2_ES, 
 	      Data, Length, VIDEO_STREAM,
@@ -1307,8 +1210,6 @@ void cXinelibDevice::StillPicture(const uchar *Data, int Length)
 	    &mand<bool>, true);
   }
 
-  TsBufferFlush();
-
   ForEach(m_clients, &cXinelibThread::Flush, 60, 
 	  &mand<bool>, true);
 
@@ -1354,12 +1255,44 @@ int cXinelibDevice::PlayAudio(const uchar *buf, int length, uchar Id)
   return PlayAny(buf, length);
 }
 
+#if VDRVERSNUM < 10510
+int cXinelibDevice::PlaySubtitle(const uchar *buf, int length) 
+{
+  TRACEF("cXinelibDevice::PlaySpu");
+
+  if(!buf || length < 6)
+    return length;
+
+  if(((unsigned char *)buf)[3] == PRIVATE_STREAM1) {
+
+    int PayloadOffset = buf[8] + 9;
+    uchar SubStreamId = buf[PayloadOffset];
+    //uchar SubStreamType = SubStreamId & 0xF0;
+    uchar SubStreamIndex = SubStreamId & 0x1F;
+
+    if(!m_spuPresent) {
+      TRACE("cXinelibDevice::PlaySpu first DVD SPU frame");
+      Skins.QueueMessage(mtInfo,"DVD Subtitles");
+      m_spuPresent = true;
+      
+      ForEach(m_clients, &cXinelibThread::SpuStreamChanged, (int)SubStreamIndex);
+    }
+
+    // Strip all but selected SPU track
+    if(SubStreamIndex != m_CurrentDvdSpuTrack)
+      return length;
+  }
+
+  return PlayAny(buf, length);
+}
+#else
 int cXinelibDevice::PlaySubtitle(const uchar *Data, int Length)
 {
   if(!xc.dvb_subtitles)
     return cDevice::PlaySubtitle(Data, Length);
   return PlayAny(Data, Length);
 }
+#endif
 
 bool cXinelibDevice::Poll(cPoller &Poller, int TimeoutMs) 
 {
@@ -1518,19 +1451,6 @@ eVideoSystem cXinelibDevice::GetVideoSystem(void)
   return cDevice::GetVideoSystem();
 }
 
-#if VDRVERSNUM >= 10708
-void cXinelibDevice::GetVideoSize(int &Width, int &Height, double &VideoAspect)
-{
-  Width  = m_VideoSize->width;
-  Height = m_VideoSize->height;
-  VideoAspect = 1.0;
-  if (m_VideoSize->pixel_aspect.den) {
-    VideoAspect = (double)m_VideoSize->pixel_aspect.num / (double)m_VideoSize->pixel_aspect.den;
-    VideoAspect *= (double)Width / (double)Height;
-  }
-}
-#endif
-
 void cXinelibDevice::GetOsdSize(int &Width, int &Height, double &PixelAspect)
 {
   switch (xc.osd_size) {
@@ -1599,6 +1519,9 @@ uchar *cXinelibDevice::GrabImage(int &Size, bool Jpeg,
 {
   TRACEF("cXinelibDevice::GrabImage");
 
+  if (Quality < 0)
+    Quality = 100;
+
   if(m_local)
     return m_local->GrabImage(Size, Jpeg, Quality, SizeX, SizeY);
   if(m_server)
@@ -1608,15 +1531,178 @@ uchar *cXinelibDevice::GrabImage(int &Size, bool Jpeg,
 }
 
 
+//
+// DVD SPU support in VDR recordings
+//
+//   - override cDevice::PlayPesPacket to get DVD SPUs
+//
+
+#if VDRVERSNUM < 10510
+int cXinelibDevice::PlayPesPacket(const uchar *Data, int Length, 
+				  bool VideoOnly)
+{
+  switch (Data[3]) {
+    case 0xBD: { // private stream 1
+      int PayloadOffset = Data[8] + 9;
+      uchar SubStreamId = Data[PayloadOffset];
+      uchar SubStreamType = SubStreamId & 0xF0;
+      uchar SubStreamIndex = SubStreamId & 0x1F;
+      switch (SubStreamType) {
+        case 0x20: // SPU
+        case 0x30: // SPU
+	  SetAvailableDvdSpuTrack(SubStreamIndex);
+	  return PlaySubtitle(Data, Length);
+	  break;
+        default:
+	  ;
+      }
+    }
+    default:
+      ;
+  }
+  return cDevice::PlayPesPacket(Data, Length, VideoOnly);
+}
+#endif
+
 //
 // Available DVD SPU tracks
 //
 
+#if VDRVERSNUM < 10515
+bool cXinelibDevice::SetCurrentDvdSpuTrack(int Type, bool Force)
+{
+  if(Type == ttXSubtitleNone || 
+     ( Type >= 0 && 
+       Type < 64 &&
+       m_DvdSpuTrack[Type].id != 0xffff)) {
+    m_CurrentDvdSpuTrack = Type;
+    ForEach(m_clients, &cXinelibThread::SpuStreamChanged, Type);
+    if(Force)
+      m_ForcedDvdSpuTrack = true;
+    return true;
+  }
+  return false;
+}
+#endif
+
+#if VDRVERSNUM >= 10515
 void cXinelibDevice::SetSubtitleTrackDevice(eTrackType Type)
 {
   if (m_PlayingFile == pmAudioVideo || m_PlayingFile == pmVideoOnly)
     ForEach(m_clients, &cXinelibThread::SetSubtitleTrack, Type);
 }
+#endif
+
+#if VDRVERSNUM < 10515
+void cXinelibDevice::ClrAvailableDvdSpuTracks(bool NotifyFrontend)
+{
+  for(int i=0; i<64; i++)
+    m_DvdSpuTrack[i].id = 0xffff;
+  if(m_CurrentDvdSpuTrack >= 0 ) {
+    m_CurrentDvdSpuTrack = ttXSubtitleNone;
+    if(NotifyFrontend)
+      ForEach(m_clients, &cXinelibThread::SpuStreamChanged, m_CurrentDvdSpuTrack);
+  }
+}
+#endif
+
+#if VDRVERSNUM < 10515
+int cXinelibDevice::NumDvdSpuTracks(void) const
+{ 
+  int DvdSpuTracks = 0;
+  for(int i=0; i<64; i++)
+    if(m_DvdSpuTrack[i].id != 0xffff)
+      DvdSpuTracks++;
+  return DvdSpuTracks; 
+}
+#endif
+
+#if VDRVERSNUM < 10515
+const tTrackId *cXinelibDevice::GetDvdSpuTrack(int Type) const
+{
+  if(Type >= 0 && Type < 64 &&
+     m_DvdSpuTrack[Type].id != 0xffff)
+    return &m_DvdSpuTrack[Type];
+  return NULL;
+}
+#endif
+
+#if VDRVERSNUM < 10515
+const char *cXinelibDevice::GetDvdSpuLang(int Type) const
+{
+  const tTrackId *track = GetDvdSpuTrack(Type);
+  if(track)
+    return track->language[0] ? track->language : NULL;
+  return NULL;
+}
+#endif
+
+#if VDRVERSNUM < 10515
+bool cXinelibDevice::SetAvailableDvdSpuTrack(int Type, const char *lang, bool Current)
+{
+  if(Type >= 0 && Type < 64) {
+
+    m_DvdSpuTrack[Type].id = Type;
+    m_DvdSpuTrack[Type].language[0] = '\0';
+    if(lang) 
+      strn0cpy(m_DvdSpuTrack[Type].language, lang, MAXLANGCODE2);
+    if(Current)
+      m_CurrentDvdSpuTrack = Type;
+
+    return true;
+  }
+  return false;
+}
+#endif
+
+#if VDRVERSNUM < 10515
+void cXinelibDevice::EnsureDvdSpuTrack(void)
+{
+  if(!m_ForcedDvdSpuTrack &&
+     NumDvdSpuTracks() > 0 &&
+     (m_DvdSpuTrack[0].id == 0xffff ||
+      strcmp(m_DvdSpuTrack[0].language, "menu"))) {
+
+    if(xc.spu_autoshow) {
+      int pref, track;
+      size_t len;
+      for(pref = 0; pref < 4; pref++)
+	for(track = 0; track < 64; track++)
+	  if(m_DvdSpuTrack[track].id != 0xffff)    
+	    if((len=strlen(xc.spu_lang[pref])) > 0)
+	      //if(!strncmp(m_DvdSpuTrack[track].language,
+	      //            xc.spu_lang[pref], len)) {
+	      if(!strcmp(m_DvdSpuTrack[track].language,
+			 xc.spu_lang[pref])) {
+		if(m_CurrentDvdSpuTrack != track) {
+		  LOGMSG("Auto-selecting %d. SPU track \'%s\'  (%d. preferred language is \'%s\')", 
+			 track, m_DvdSpuTrack[track].language, pref+1, xc.spu_lang[pref]);
+		  cXinelibDevice::SetCurrentDvdSpuTrack(track);
+		  cString msg = cString::sprintf("Subtitles: %s", m_DvdSpuTrack[track].language);
+		  Skins.QueueMessage(mtInfo, msg);
+		}
+		m_spuPresent = true;
+		track = 64;
+		pref = 99;
+	      }
+      if(pref < 99) {
+	for(track = 0; track < 64; track++)
+	  if(m_DvdSpuTrack[track].id != 0xffff)
+	    if(m_CurrentDvdSpuTrack != track) {
+	      LOGMSG("Auto-selecting (non-matching) %d. SPU track \'%s\'",
+		     track, m_DvdSpuTrack[track].language);
+	      cXinelibDevice::SetCurrentDvdSpuTrack(track);
+	    }
+      }
+    }
+
+    if(!m_spuPresent) {
+      Skins.QueueMessage(mtInfo, "Subtitles present");
+      m_spuPresent = true;
+    }
+  }
+}
+#endif
 
 //
 // Metainfo
