@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: frontend.c,v 1.76 2009/09/05 21:27:25 rofafor Exp $
+ * $Id: frontend.c,v 1.59.2.6 2009/09/14 10:10:20 phintuka Exp $
  *
  */
 
@@ -109,6 +109,9 @@ void cXinelibThread::InfoHandler(const char *info)
   if(!strncmp(info, "TRACKMAP SPU", 12)) {
     int CurrentTrack = ttXSubtitleAuto;
     map += 12;
+#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
+    cXinelibDevice::Instance().ClrAvailableDvdSpuTracks(false);
+#endif
     while(*map) {
       bool Current = false;
       while(*map == ' ') map++;
@@ -128,17 +131,25 @@ void cXinelibThread::InfoHandler(const char *info)
 	char *lang = map;
 	while(*map && *map != ' ') map++;
 	if(*map == ' ') { *map = 0; map++; };
+#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
+	cXinelibDevice::Instance().SetAvailableDvdSpuTrack(id, iso639_2_to_iso639_1(lang), Current);
+#else
 	cXinelibDevice::Instance().SetAvailableTrack(ttSubtitle, id, id+1, iso639_2_to_iso639_1(lang) ?: *cString::sprintf("%03d", id+1));
 	if (Current)
 	  CurrentTrack = id;
+#endif
       }
     }
+#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
+    cXinelibDevice::Instance().EnsureDvdSpuTrack();
+#else
     if (CurrentTrack == ttXSubtitleAuto)
       cXinelibDevice::Instance().EnsureSubtitleTrack();
     else if (CurrentTrack == ttXSubtitleNone)
       cXinelibDevice::Instance().SetCurrentSubtitleTrack(ttNone, true);
     else
       cXinelibDevice::Instance().SetCurrentSubtitleTrack(eTrackType(CurrentTrack+ttSubtitleFirst), true);
+#endif
   }
 
   else if(!strncmp(info, "TRACKMAP AUDIO", 14)) {
@@ -205,8 +216,13 @@ void cXinelibThread::InfoHandler(const char *info)
     map += 9;
     while(*map == ' ') map++;
     cXinelibDevice::Instance().SetMetaInfo(miDvdTitleNo, map);
+#if VDRVERSNUM < 10515 && !defined(VDRSPUPATCH)
+    if (*map == '0')  // DVD Menu, set spu track to 0
+      cXinelibDevice::Instance().SetCurrentDvdSpuTrack(0);
+#else
     if (*map == '0')  // DVD Menu, set spu track to 0
       cXinelibDevice::Instance().SetCurrentSubtitleTrack(ttSubtitleFirst);
+#endif
   }
 
   else if (!strncmp(info, "WINDOW ", 7)) {
@@ -332,6 +348,13 @@ void cXinelibThread::AudioStreamChanged(bool ac3, int StreamId)
     Xine_Control("AUDIOSTREAM", StreamId);
 }
 
+#if VDRVERSNUM < 10515
+void cXinelibThread::SpuStreamChanged(int StreamId)
+{
+  TRACEF("cXinelibThread::SpuStreamChanged");
+  Xine_Control("SPUSTREAM", StreamId);
+}
+#else
 void cXinelibThread::SetSubtitleTrack(eTrackType Track)
 {
   TRACEF("cXinelibThread::SetSubtitleTrack");
@@ -340,6 +363,7 @@ void cXinelibThread::SetSubtitleTrack(eTrackType Track)
 				 m_SpuLangAuto ? " auto" : "");
   Xine_Control(buf);
 }
+#endif
 
 void cXinelibThread::Clear(void)
 {
@@ -704,9 +728,11 @@ bool cXinelibThread::PlayFile(const char *FileName, int Position,
     m_FileName = FileName;
     m_bPlayingFile = true;
     m_SpuLangAuto = true;
+#if VDRVERSNUM >= 10515
     if (m_StatusMonitor)
       DELETENULL(m_StatusMonitor);
     m_StatusMonitor = new cFrontendStatusMonitor(m_SpuLangAuto);
+#endif
     Unlock();
   }
 
@@ -723,6 +749,7 @@ bool cXinelibThread::PlayFile(const char *FileName, int Position,
     if(xc.extsub_size >= 0)
       Xine_Control("EXTSUBSIZE", xc.extsub_size);
 
+#if VDRVERSNUM >= 10515
     // set preferred subtitle language 
     if (Setup.DisplaySubtitles) {
       const char *langs = I18nLanguageCode(Setup.SubtitleLanguages[0]);
@@ -738,6 +765,7 @@ bool cXinelibThread::PlayFile(const char *FileName, int Position,
       LOGMSG("Preferred SPU language: (none)");
       Xine_Control(cString::sprintf("SPUSTREAM %d", ttXSubtitleNone));
     }
+#endif
   }
 
   return Running() && !result;
