@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: media_player.c,v 1.68 2009/06/02 08:59:45 phintuka Exp $
+ * $Id: media_player.c,v 1.86 2010/12/20 13:06:11 phintuka Exp $
  *
  */
 
@@ -24,8 +24,10 @@
 
 #include "logdefs.h"
 
-static void BackToMenu(void)
+static void BackToMenu(eMainMenuMode Menu)
 {
+  xc.main_menu_mode = Menu;
+
   cRemote::CallPlugin("xineliboutput");
 }
 
@@ -45,15 +47,17 @@ class cXinelibPlayer : public cPlayer
 
     bool m_Error;
     bool m_UseResumeFile;
-    int  m_Speed; 
+    int  m_Speed;
 
     void UpdateNumTracks(void);
 
+    cXinelibDevice *m_Dev;
+
   protected:
     virtual void Activate(bool On);
 
   public:
-    cXinelibPlayer(const char *File, bool Queue = false, const char *SubFile = NULL);
+    cXinelibPlayer(cXinelibDevice *Dev, const char *File, bool Queue = false, const char *SubFile = NULL);
     virtual ~cXinelibPlayer();
 
     // cPlayer
@@ -63,16 +67,20 @@ class cXinelibPlayer : public cPlayer
     virtual bool GetReplayMode(bool &Play, bool &Forward, int &Speed);
 
     // cXinelibPlayer
-    void Control(const char *s) { (void)cXinelibDevice::Instance().PlayFileCtrl(s); }
-    void Control(const char *s, int i) { 
+    void Control(const char *s) { (void)m_Dev->PlayFileCtrl(s); }
+    void Control(const char *s, int i) {
       cString cmd = cString::sprintf(s, i);
-      Control(cmd); 
+      Control(cmd);
     }
     void SetSpeed(int Speed);
     int  Speed(void) { return m_Speed; };
 
+    int  GetPos(void)    { return m_Dev->PlayFileCtrl("GETPOS");    }
+    int  GetLength(void) { return m_Dev->PlayFileCtrl("GETLENGTH"); }
+    const char *GetMetaInfo(eMetainfoType mi) { return m_Dev->GetMetaInfo(mi); }
+
     bool NextFile(int step);
-    bool Playing(void) { return !(m_Error || cXinelibDevice::Instance().EndOfStreamReached()); }
+    bool Playing(void) { return !(m_Error || m_Dev->EndOfStreamReached()); }
     bool Error(void)   { return m_Error; }
     void UseResumeFile(bool Val) { m_UseResumeFile = Val; }
 
@@ -81,14 +89,17 @@ class cXinelibPlayer : public cPlayer
     const cString& File(void) { return m_File; }
     int  CurrentFile(void) { return m_Playlist.Current()->Index(); } 
     int  Files(void) { return m_Playlist.Count(); }
+
+    bool UpdateMetaInfo(bool Force = false);
 };
 
-cXinelibPlayer::cXinelibPlayer(const char *File, bool Queue, const char *SubFile) 
+cXinelibPlayer::cXinelibPlayer(cXinelibDevice *Dev, const char *File, bool Queue, const char *SubFile)
 {
   m_ResumeFile = NULL;
   m_UseResumeFile = true;
   m_Error = false;
   m_Speed = 1;
+  m_Dev = Dev;
 
   if(File) {
     size_t len = strlen(File);
@@ -130,15 +141,15 @@ void cXinelibPlayer::SetAudioTrack(eTrackType Type, const tTrackId *TrackId)
 
 void cXinelibPlayer::SetSubtitleTrack(eTrackType Type, const tTrackId *TrackId)
 {
-  cXinelibDevice::Instance().SetSubtitleTrackDevice(Type);
+  m_Dev->SetSubtitleTrackDevice(Type);
 }
 
 bool cXinelibPlayer::GetIndex(int &Current, int &Total, bool SnapToIFrame) 
 { 
   // Returns the current and total frame index, optionally snapped to the
   // nearest I-frame.
-  int msCurrent = cXinelibDevice::Instance().PlayFileCtrl("GETPOS");
-  int msTotal   = cXinelibDevice::Instance().PlayFileCtrl("GETLENGTH");
+  int msCurrent = GetPos();
+  int msTotal   = GetLength();
   if(msCurrent>=0 && msTotal>=0) {
     Current = msCurrent * 25 / 1000;
     Total = msTotal * 25 / 1000;
@@ -178,15 +189,47 @@ void cXinelibPlayer::SetSpeed(int Speed)
   }
 }
 
+bool cXinelibPlayer::UpdateMetaInfo(bool Force)
+{
+  // update playlist metainfo
+  const char *ti = GetMetaInfo(miTitle);
+  bool TitleChanged =  ti && ti[0] && (!*Playlist().Current()->Title ||
+                                       !strstr(Playlist().Current()->Title, ti));
+  if (Force || TitleChanged) {
+    const char *tr = GetMetaInfo(miTracknumber);
+    const char *al = GetMetaInfo(miAlbum);
+    const char *ar = GetMetaInfo(miArtist);
+
+    if (!Force)
+      LOGDBG("metainfo changed: %s->%s %s->%s %s->%s %s->%s",
+             *Playlist().Current()->Artist      ?: "-", ar ?: "-",
+             *Playlist().Current()->Album       ?: "-", al ?: "-",
+             *Playlist().Current()->Tracknumber ?: "-", tr ?: "-",
+             *Playlist().Current()->Title       ?: "-", ti ?: "-");
+
+    m_Playlist.Current()->Title = ti;
+    if (tr && tr[0])
+      m_Playlist.Current()->Tracknumber = tr;
+    if (al && al[0])
+      m_Playlist.Current()->Album = al;
+    if (ar && ar[0])
+      m_Playlist.Current()->Artist = ar;
+
+    return true;
+  }
+
+  return false;
+}
+
 bool cXinelibPlayer::NextFile(int step)
 {
-  if(m_Playlist.Count()>0) {
-    for(;step < 0; step++)
+  if (m_Playlist.Count() > 0) {
+    for (;step < 0; step++)
       m_Playlist.Prev();
-    for(;step > 0; step--) 
+    for (;step > 0; step--)
       m_Playlist.Next();
 
-    if(!m_Playlist.Current())
+    if (!m_Playlist.Current())
       LOGERR("!m_Playlist.Get(m_CurrInd)");
     m_File = *m_Playlist.Current()->Filename;
     m_ResumeFile = NULL;
@@ -195,7 +238,7 @@ bool cXinelibPlayer::NextFile(int step)
     Activate(true);
     return !m_Error;
   }
-  
+
   return false;
 }
 
@@ -203,7 +246,7 @@ void cXinelibPlayer::UpdateNumTracks(void)
 {
   // cdda tracks
   if(m_Playlist.Count() == 1 && !strcmp("cdda:/", m_Playlist.First()->Filename)) {
-    int count = cXinelibDevice::Instance().PlayFileCtrl("GETAUTOPLAYSIZE CD", 10000);
+    int count = m_Dev->PlayFileCtrl("GETAUTOPLAYSIZE CD", 10000);
     if(count>0) {
       for(int i=0; i<count; i++)
         m_Playlist.Read(cString::sprintf("cdda:/%d", i+1));
@@ -223,49 +266,25 @@ void cXinelibPlayer::Activate(bool On)
 	pos = 0;
       close(fd);
     }
-    // escape file name and join subtitle file
-    // Maybe mrls from playlist files should not be escaped ?
-    // (those may contain #subtitle, #volnorm etc. directives)
-    cString mrl;
-    if(*m_SubFile)
-      mrl = cString::sprintf("%s%s#subtitle:%s%s",
-			     m_File[0] == '/' ? "file:" : "",
-			     *cPlaylist::EscapeMrl(m_File), 
-			     m_SubFile[0] == '/' ? "file:" : "",
-			     *cPlaylist::EscapeMrl(m_SubFile));
-    else if((*m_File)[0] == '/')
-      mrl = cString::sprintf("%s%s",
-			     m_File[0] == '/' ? "file:" : "",
-			     *cPlaylist::EscapeMrl(m_File));
-    else
-      mrl = cPlaylist::EscapeMrl(m_File);
+    // escape raw file names and join subtitle file to the mrl
+    cString mrl = m_File[0] == '/' ? cPlaylist::BuildMrl("file", m_File) : m_File;
+    if (*m_SubFile)
+      mrl = cString::sprintf("%s#subtitle:%s", *mrl,
+                             m_SubFile[0] == '/' ? *cPlaylist::BuildMrl("file", m_SubFile) : *m_SubFile);
 
     // Start replay
     UpdateNumTracks();
-    m_Error = !cXinelibDevice::Instance().PlayFile(mrl, pos);
+    m_Error = !m_Dev->PlayFile(mrl, pos);
     LOGDBG("cXinelibPlayer playing %s (%s)", *m_File, m_Error ? "FAIL" : "OK");
 
-    if(!m_Error) {
-      // update playlist metainfo
-      const char *ti = cXinelibDevice::Instance().GetMetaInfo(miTitle);
-      const char *tr = cXinelibDevice::Instance().GetMetaInfo(miTracknumber);
-      const char *al = cXinelibDevice::Instance().GetMetaInfo(miAlbum);
-      const char *ar = cXinelibDevice::Instance().GetMetaInfo(miArtist);
-      if(ti && ti[0] && (!*m_Playlist.Current()->Title || !strstr(m_Playlist.Current()->Title, ti)))
-	m_Playlist.Current()->Title = ti;
-      if(tr && tr[0])
-        m_Playlist.Current()->Tracknumber = tr;
-      if(al && al[0])
-	m_Playlist.Current()->Album = al;
-      if(ar && ar[0])
-	m_Playlist.Current()->Artist = ar;
-
+    if (!m_Error) {
+      UpdateMetaInfo(true);
       UpdateNumTracks();
     }
   } else {
     if(m_UseResumeFile && *m_ResumeFile) {
-      pos = cXinelibDevice::Instance().PlayFileCtrl("GETPOS");
-      len = cXinelibDevice::Instance().PlayFileCtrl("GETLENGTH");
+      pos = GetPos();
+      len = GetLength();
       if(pos>10000 && pos < (len-10000)) {
 	pos = (pos/1000) - 10; // skip back 10 seconds ("VDR style")
 	if(0 <= (fd = open(m_ResumeFile, O_WRONLY | O_CREAT, 
@@ -282,7 +301,7 @@ void cXinelibPlayer::Activate(bool On)
       }
       m_ResumeFile = NULL;
     }
-    cXinelibDevice::Instance().PlayFile(NULL);
+    m_Dev->PlayFile(NULL);
     m_Error = false;
   }
 }
@@ -294,6 +313,9 @@ void cXinelibPlayer::Activate(bool On)
 
 class cPlaylistMenu : public cOsdMenu, cPlaylistChangeNotify
 {
+  private:
+    int        m_Marked;
+
   protected:
 
     cPlaylist& m_Playlist;
@@ -323,6 +345,7 @@ cPlaylistMenu::cPlaylistMenu(cPlaylist &Playlist, bool& RandomPlay) :
      m_RandomPlay(RandomPlay),
      m_IC("UTF-8", cCharSetConv::SystemCharacterTable())
 {
+  m_Marked = -1;
   SetTitle(cString::sprintf("%s: %s", tr("Playlist"), m_IC.Convert(*Playlist.Name())));
   Playlist.Listen(this);
   Set(true);
@@ -347,6 +370,23 @@ eOSState cPlaylistMenu::ProcessKey(eKeys Key)
 
   eOSState state = cOsdMenu::ProcessKey(Key);
 
+  if (m_Marked >= 0) {
+    switch(Key) {
+      case kOk:
+        m_Playlist.Move(m_Marked, Current());
+        Set();
+        m_Marked = -1;
+        return osContinue;
+
+      case kBack:
+        m_Marked = -1;
+        return osEnd;
+
+      default:;
+    }
+    return osContinue;
+  }
+
   if(state == osUnknown) {
     switch(Key) {
       case kBack:   
@@ -374,7 +414,11 @@ eOSState cPlaylistMenu::ProcessKey(eKeys Key)
 		      Set();
 	              return result;
                     }
-      case kBlue:   
+      case kBlue:
+                    Mark();
+                    m_Marked = Current();
+	            return osContinue;
+      case k0:
                     m_Playlist.Sort();
 		    Set();
 	            return osContinue;
@@ -399,7 +443,7 @@ void cPlaylistMenu::SetHelpButtons(void)
   SetHelp(!m_RandomPlay ? tr("Button$Random") : tr("Button$Normal"),
 	  tr("Button$Add files"),
 	  m_Playlist.Count()>1 ? tr("Button$Remove") : NULL,
-	  tr("Button$Sort"));
+	  tr("Button$Mark"));
   Display();
 }
 
@@ -436,6 +480,47 @@ void cPlaylistMenu::Set(bool setCurrentPlaying)
 
 #include <vdr/skins.h>
 
+class cXinelibPlayerControl : public cControl
+{
+  private:
+    static cMutex m_Lock;
+
+    static cXinelibPlayer *OpenPlayer(const char *File, bool Queue = false, const char *SubFile = NULL);
+
+ protected:
+    static cXinelibPlayer *m_Player;
+
+    cSkinDisplayReplay *m_DisplayReplay;
+    cPlaylistMenu      *m_PlaylistMenu;
+
+    eMainMenuMode m_Mode;
+    bool    m_ShowModeOnly;
+    bool    m_RandomPlay;
+    time_t  m_AutoShowStart;
+    int     m_CurrentPos;
+    int     m_CurrentLen;
+    bool    m_BlinkState;
+
+    cTimeMs lastTime;
+    int     number;
+
+    void CloseMenus(void);
+    void MsgReplaying(const char *Title, const char *File);
+
+  public:
+    cXinelibPlayerControl(eMainMenuMode Mode, const char *File, const char *SubFile = NULL);
+    virtual ~cXinelibPlayerControl();
+
+    virtual void Show(void);
+    virtual void Hide(void);
+    virtual eOSState ProcessKey(eKeys Key);
+    virtual cOsdObject *GetInfo(void);
+
+    static void Close(void);
+    static bool IsOpen(void) { return m_Player != NULL; };
+    static void Queue(const char *File);
+};
+
 cXinelibPlayer *cXinelibPlayerControl::m_Player = NULL;
 cMutex cXinelibPlayerControl::m_Lock;
 
@@ -460,19 +545,24 @@ cXinelibPlayerControl::cXinelibPlayerControl(eMainMenuMode Mode, const char *Fil
 
 cXinelibPlayerControl::~cXinelibPlayerControl()
 {
-  if(m_PlaylistMenu) {
-    delete m_PlaylistMenu;
-    m_PlaylistMenu = NULL;
-  }
-  if(m_DisplayReplay) {
-    delete m_DisplayReplay;
-    m_DisplayReplay = NULL;
-  }
+  CloseMenus();
 
   MsgReplaying(NULL, NULL);
   Close();
 }
 
+void cXinelibPlayerControl::CloseMenus(void)
+{
+  if (m_PlaylistMenu) {
+    delete m_PlaylistMenu;
+    m_PlaylistMenu = NULL;
+  }
+  if (m_DisplayReplay) {
+    delete m_DisplayReplay;
+    m_DisplayReplay = NULL;
+  }
+}
+
 void cXinelibPlayerControl::MsgReplaying(const char *Title, const char *File)
 {
   cStatus::MsgReplaying(this, NULL, NULL, false);
@@ -500,7 +590,7 @@ void cXinelibPlayerControl::Queue(const char *File)
       m_Player->Playlist().Read(File);
   }
 
-  Skins.Message(mtInfo, tr("Queued to playlist"));
+  cRemote::Put(Skins.Message(mtInfo, tr("Queued to playlist")), true);
 
   m_Lock.Unlock();
 
@@ -513,7 +603,7 @@ cXinelibPlayer *cXinelibPlayerControl::OpenPlayer(const char *File, bool Queue,
 {
   m_Lock.Lock();
   if(!m_Player)
-    m_Player = new cXinelibPlayer(File, Queue, SubFile);
+    m_Player = new cXinelibPlayer(&(cXinelibDevice::Instance()), File, Queue, SubFile);
   m_Lock.Unlock();
   return m_Player;
 }
@@ -542,8 +632,8 @@ void cXinelibPlayerControl::Show()
 
   if(!m_ShowModeOnly) {
     char t[128] = "";
-    int  Current = cXinelibDevice::Instance().PlayFileCtrl("GETPOS");
-    int  Total   = cXinelibDevice::Instance().PlayFileCtrl("GETLENGTH");
+    int  Current = m_Player->GetPos();
+    int  Total   = m_Player->GetLength();
     if(Current>=0) m_CurrentPos = Current;
     if(Total>=0) m_CurrentLen = Total;
 
@@ -571,14 +661,7 @@ void cXinelibPlayerControl::Show()
 
 void cXinelibPlayerControl::Hide()
 {
-  if(m_PlaylistMenu) {
-    delete m_PlaylistMenu;
-    m_PlaylistMenu = NULL;
-  }
-  if(m_DisplayReplay) {
-    delete m_DisplayReplay;
-    m_DisplayReplay = NULL;
-  }
+  CloseMenus();
 }
 
 cOsdObject *cXinelibPlayerControl::GetInfo(void)
@@ -615,54 +698,37 @@ eOSState cXinelibPlayerControl::ProcessKey(eKeys Key)
   }
 
   else {
-    // metainfo may change during playback (DVD titles, CDDA tracks)
-    const char *ti = cXinelibDevice::Instance().GetMetaInfo(miTitle);
-    if(ti && ti[0] && (!*m_Player->Playlist().Current()->Title ||
-		       !strstr(m_Player->Playlist().Current()->Title, ti))) {
-      const char *tr = cXinelibDevice::Instance().GetMetaInfo(miTracknumber);
-      const char *al = cXinelibDevice::Instance().GetMetaInfo(miAlbum);
-      const char *ar = cXinelibDevice::Instance().GetMetaInfo(miArtist);
-      LOGDBG("metainfo changed: %s->%s %s->%s %s->%s %s->%s",
-	     *m_Player->Playlist().Current()->Artist?:"-", ar?:"-", 
-	     *m_Player->Playlist().Current()->Album ?:"-", al?:"-", 
-	     *m_Player->Playlist().Current()->Tracknumber ?:"-", tr?:"-",
-             *m_Player->Playlist().Current()->Title ?:"-", ti?:"-");
-      m_Player->Playlist().Current()->Title = ti;
-      if(tr && tr[0])
-        m_Player->Playlist().Current()->Tracknumber = tr;
-      if(al && al[0])
-	m_Player->Playlist().Current()->Album = al;
-      if(ar && ar[0])
-	m_Player->Playlist().Current()->Artist = ar;
+    if (m_Player->UpdateMetaInfo())
       MsgReplaying(*m_Player->Playlist().Current()->Title, *m_Player->File());
-    }
   }
 
-  if(m_PlaylistMenu) {
+  // playlist menu
+  if (m_PlaylistMenu) {
     m_AutoShowStart = 0;
 
     eOSState state = osUnknown;
 
-    switch(state=m_PlaylistMenu->ProcessKey(Key)) {
+    switch (state = m_PlaylistMenu->ProcessKey(Key)) {
       case osBack:
       case osEnd:   Hide(); break;
-      default:      if(state >= os_User) {
-	              m_Player->NextFile( (int)state - (int)os_User - m_Player->CurrentFile());
-		      m_PlaylistMenu->SetCurrentExt(m_Player->CurrentFile()); 
-		      MsgReplaying(*m_Player->Playlist().Current()->Title, *m_Player->File());
+      default:      if (state >= os_User) {
+                      m_Player->NextFile( (int)state - (int)os_User - m_Player->CurrentFile());
+                      m_PlaylistMenu->SetCurrentExt(m_Player->CurrentFile());
+                      MsgReplaying(*m_Player->Playlist().Current()->Title, *m_Player->File());
                     }
 	            break;
     }
 
-    if(state != osUnknown)
+    if (state != osUnknown)
       return osContinue;
   }
 
-  if (m_DisplayReplay) 
+  if (m_DisplayReplay)
     Show();
 
   if ( m_Mode == ShowFiles ) {
     switch(Key) {
+      // replay menu
       case kRed:    if(m_Player->Playlist().Count() > 1) {
                       Hide();
                       m_PlaylistMenu = new cPlaylistMenu(m_Player->Playlist(), m_RandomPlay);
@@ -700,8 +766,9 @@ eOSState cXinelibPlayerControl::ProcessKey(eKeys Key)
                       break;
                     }
       default:      break;
-    }   
+    }
   }
+
   if ( m_Mode == ShowMusic ) {
     switch(Key) {
       case kRed:    Hide();
@@ -721,7 +788,7 @@ eOSState cXinelibPlayerControl::ProcessKey(eKeys Key)
                     MsgReplaying(*m_Player->Playlist().Current()->Title, *m_Player->File());
                     break;
       case kPrev:
-      case kLeft:   if(cXinelibDevice::Instance().PlayFileCtrl("GETPOS") < 3000) {
+      case kLeft:   if(m_Player->GetPos() < 3000) {
                       m_Player->NextFile(-1);
                       if(!m_DisplayReplay)
                         m_AutoShowStart = time(NULL);
@@ -754,10 +821,10 @@ eOSState cXinelibPlayerControl::ProcessKey(eKeys Key)
       default:      break;
     }
   }
+
   switch(Key) { // key bindings common for both players
-    case kBack:   xc.main_menu_mode = m_Mode;
-                  Hide(); 
-		  BackToMenu();
+    case kBack:   Hide();
+                  BackToMenu(m_Mode);
                   break;
     case kStop:
     case kBlue:   Hide();
@@ -848,49 +915,93 @@ eOSState cXinelibPlayerControl::ProcessKey(eKeys Key)
   return osContinue;
 }
 
+// --- DVD player -----------------------------------------------------------
+
 //
 // cDvdMenu
 //
 
 class cDvdMenu : public cOsdMenu {
- public:
-  cDvdMenu(void) : cOsdMenu("DVD Menu")
-  {
-    Add(new cOsdItem("Exit DVD menu",  osUser1));
-    Add(new cOsdItem("DVD Root menu",  osUser2));
-    Add(new cOsdItem("DVD Title menu", osUser3));
-    Add(new cOsdItem("DVD SPU menu",   osUser4));
-    Add(new cOsdItem("DVD Audio menu", osUser5));
-    Add(new cOsdItem("Close menu",     osEnd));
-    Display();
-  }
+  private:
+    cXinelibPlayer *m_Player;
+
+  public:
+    cDvdMenu(cXinelibPlayer *Player);
+
+    virtual eOSState ProcessKey(eKeys Key);
 };
 
+cDvdMenu::cDvdMenu(cXinelibPlayer *Player) : cOsdMenu(tr("DVD Menu")), m_Player(Player)
+{
+  Add(new cOsdItem(tr("Exit DVD menu"),  osUser1));
+  Add(new cOsdItem(tr("DVD Root menu"),  osUser2));
+  Add(new cOsdItem(tr("DVD Title menu"), osUser3));
+  Add(new cOsdItem(tr("DVD SPU menu"),   osUser4));
+  Add(new cOsdItem(tr("DVD Audio menu"), osUser5));
+  Add(new cOsdItem(tr("Close menu"),     osEnd));
+  Display();
+}
+
+eOSState cDvdMenu::ProcessKey(eKeys Key)
+{
+  eOSState state = cOsdMenu::ProcessKey(Key);
+  switch (state) {
+    case osUser1: m_Player->Control("EVENT XINE_EVENT_INPUT_MENU1"); return osEnd;
+    case osUser2: m_Player->Control("EVENT XINE_EVENT_INPUT_MENU2"); return osEnd;
+    case osUser3: m_Player->Control("EVENT XINE_EVENT_INPUT_MENU3"); return osEnd;
+    case osUser4: m_Player->Control("EVENT XINE_EVENT_INPUT_MENU4"); return osEnd;
+    case osUser5: m_Player->Control("EVENT XINE_EVENT_INPUT_MENU5"); return osEnd;
+    case osBack:
+    case osEnd:   return osEnd;
+    default:      break;
+  }
+  return state;
+}
 
 //
 // cXinelibDvdPlayerControl
 //
 
+class cXinelibDvdPlayerControl : public cXinelibPlayerControl
+{
+  private:
+    cOsdMenu *m_DvdMenu;
+
+    void      CloseDvdMenu(void);
+
+  public:
+    cXinelibDvdPlayerControl(const char *File) :
+      cXinelibPlayerControl(ShowFiles, File), m_DvdMenu(NULL)
+      {}
+    virtual ~cXinelibDvdPlayerControl();
+
+    virtual void Show(void);
+    virtual void Hide(void);
+    virtual eOSState ProcessKey(eKeys Key);
+};
+
 cXinelibDvdPlayerControl::~cXinelibDvdPlayerControl() 
 {
-  if(Menu) {
-    delete Menu;
-    Menu = NULL;
-  }
+  CloseDvdMenu();
 }
 
 void cXinelibDvdPlayerControl::Hide(void)
 {
-  if(Menu) {
-    delete Menu;
-    Menu = NULL;
-  }
+  CloseDvdMenu();
   cXinelibPlayerControl::Hide();
 }
 
+void cXinelibDvdPlayerControl::CloseDvdMenu(void)
+{
+  if (m_DvdMenu) {
+    delete m_DvdMenu;
+    m_DvdMenu = NULL;
+  }
+}
+
 void cXinelibDvdPlayerControl::Show(void)
 {
-  if(!Menu)
+  if (!m_DvdMenu)
     cXinelibPlayerControl::Show();
   else
     cXinelibPlayerControl::Hide();
@@ -905,56 +1016,39 @@ eOSState cXinelibDvdPlayerControl::ProcessKey(eKeys Key)
     return osEnd;
   }
 
-  // Update DVD title information
-  const char *ti = cXinelibDevice::Instance().GetMetaInfo(miTitle);
-  if (ti && ti[0] && (!m_CurrentDVDTitle || !strstr(m_CurrentDVDTitle, ti))) {
-    memset(m_CurrentDVDTitle, 0, 63);
-    strn0cpy(m_CurrentDVDTitle, ti, 63);
-    m_Player->Playlist().Current()->Title = m_CurrentDVDTitle;
-    MsgReplaying(m_CurrentDVDTitle, NULL);
-  }
+  // Check for changed title
+  if (m_Player->UpdateMetaInfo())
+    MsgReplaying(*m_Player->Playlist().Current()->Title, NULL);
 
   // Handle menu selection
-  if(Menu) {
-    if(Key == kRed)
+  if (m_DvdMenu) {
+    if (Key == kRed || m_DvdMenu->ProcessKey(Key) == osEnd)
       Hide();
-    else switch(Menu->ProcessKey(Key)) {
-      case osUser1: Hide(); m_Player->Control("EVENT XINE_EVENT_INPUT_MENU1"); break;
-      case osUser2: Hide(); m_Player->Control("EVENT XINE_EVENT_INPUT_MENU2"); break;
-      case osUser3: Hide(); m_Player->Control("EVENT XINE_EVENT_INPUT_MENU3"); break;
-      case osUser4: Hide(); m_Player->Control("EVENT XINE_EVENT_INPUT_MENU4"); break;
-      case osUser5: Hide(); m_Player->Control("EVENT XINE_EVENT_INPUT_MENU5"); break;
-      case osBack:
-      case osEnd:   Hide(); break;
-      default:      break;
-    }
     return osContinue;
   }
 
   // Update progress bar display
-  if (m_DisplayReplay) 
+  if (m_DisplayReplay)
     Show();
 
-  // Handle menu navigation
-
+  // Detect DVD menus
   bool MenuDomain = !xc.dvd_arrow_keys_control_playback;
   if(Key != kNone || m_DisplayReplay) {
-    const char *dt = cXinelibDevice::Instance().GetMetaInfo(miDvdTitleNo);
-    if(dt && !strcmp("0", dt)) 
+    const char *dt = m_Player->GetMetaInfo(miDvdTitleNo);
+    if(dt && !strcmp("0", dt))
       MenuDomain = true;
     else {
-      dt = cXinelibDevice::Instance().GetMetaInfo(miDvdButtons);
+      dt = m_Player->GetMetaInfo(miDvdButtons);
       if(dt && *dt && *dt != '0')
 	MenuDomain = true;
     }
   }
 
+  // DVD menu navigation
   if(MenuDomain) {
     if(m_DisplayReplay)
       Hide();
-
     switch(Key) {
-      // DVD navigation
       case kUp:    m_Player->Control("EVENT XINE_EVENT_INPUT_UP");     return osContinue;
       case kDown:  m_Player->Control("EVENT XINE_EVENT_INPUT_DOWN");   return osContinue;
       case kLeft:  m_Player->Control("EVENT XINE_EVENT_INPUT_LEFT");   return osContinue;
@@ -998,9 +1092,13 @@ eOSState cXinelibDvdPlayerControl::ProcessKey(eKeys Key)
 		     Show();
 		   }
 		   break;
-      case kBack:  xc.main_menu_mode = m_Mode;
-	           Hide(); 
-		   Close(); 
+      case kBack:  if (config_t::IsDvdImage(m_Player->File())) {
+                     BackToMenu(m_Mode);
+                   } else {
+                     BackToMenu(ShowMenu);
+                   }
+                   Hide();
+		   Close();
 		   return osEnd;
       default:     break;
     }
@@ -1009,7 +1107,7 @@ eOSState cXinelibDvdPlayerControl::ProcessKey(eKeys Key)
   switch(Key) {
     // DVD menus
     case kRed:    Hide();
-                  Menu = new cDvdMenu();
+                  m_DvdMenu = new cDvdMenu(m_Player);
 		  break;
     // Playback control
     case kGreen:  m_Player->Control("SEEK -60");  break;
@@ -1019,7 +1117,7 @@ eOSState cXinelibDvdPlayerControl::ProcessKey(eKeys Key)
     case kUser9:
     case k3:      m_Player->Control("SEEK +20");  break;
 
-    case kStop: 
+    case kStop:
     case kBlue:   Hide();
                   Close();
                   return osEnd;
@@ -1092,19 +1190,19 @@ class cXinelibImagePlayer : public cPlayer {
     virtual void Activate(bool On);
 
   public:
-    cXinelibImagePlayer(const char *File); 
+    cXinelibImagePlayer(cXinelibDevice *Dev, const char *File);
     virtual ~cXinelibImagePlayer();
 
     bool ShowImage(const char *File);
     bool Error(void)                 { return m_Error; } 
 };
 
-cXinelibImagePlayer::cXinelibImagePlayer(const char *File) 
+cXinelibImagePlayer::cXinelibImagePlayer(cXinelibDevice *Dev, const char *File)
 {
   m_Mrl    = File;
   m_Active = false;
   m_Error  = false;
-  m_Dev    = &(cXinelibDevice::Instance());
+  m_Dev    = Dev;
 }
 
 cXinelibImagePlayer::~cXinelibImagePlayer()
@@ -1115,10 +1213,10 @@ cXinelibImagePlayer::~cXinelibImagePlayer()
 
 bool cXinelibImagePlayer::Play(void)
 {
-  if ((*m_Mrl)[0] == '/')
-    m_Mrl = cString::sprintf("file:%s", *cPlaylist::EscapeMrl(m_Mrl));
+  if (m_Mrl[0] == '/')
+    m_Mrl = cPlaylist::BuildMrl("file", m_Mrl);
 
-  return m_Dev->PlayFile(m_Mrl, 0, true);
+  return ! (m_Error = !m_Dev->PlayFile(m_Mrl, 0, true));
 }
 
 void cXinelibImagePlayer::Activate(bool On)
@@ -1144,17 +1242,47 @@ bool cXinelibImagePlayer::ShowImage(const char *File)
 // cXinelibImagesControl
 //
 
+class cXinelibImagesControl : public cControl
+{
+  private:
+    static cXinelibImagePlayer *m_Player;
+    static cMutex m_Lock;
+
+    cSkinDisplayReplay *m_DisplayReplay;
+
+    cPlaylist *m_Playlist;
+    int m_Speed;
+    int m_LastShowTime;
+    bool m_ShowModeOnly;
+
+    static cXinelibImagePlayer *OpenPlayer(const char *File);
+
+  protected:
+    void Seek(int Rel);
+    void Delete(void);
+
+  public:
+    cXinelibImagesControl(cPlaylist *Playlist);
+    virtual ~cXinelibImagesControl();
+
+    virtual void Show(void);
+    virtual void Hide(void);
+    virtual eOSState ProcessKey(eKeys Key);
+
+    virtual cOsdObject *GetInfo(void);
+
+    static void Close(void);
+    static bool IsOpen(void) { return m_Player != NULL; }
+};
+
 cXinelibImagePlayer *cXinelibImagesControl::m_Player = NULL;
 cMutex cXinelibImagesControl::m_Lock;
 
-cXinelibImagesControl::cXinelibImagesControl(char **Files, int Index, int Count) :
-  cControl(OpenPlayer(Files[Index]))
+cXinelibImagesControl::cXinelibImagesControl(cPlaylist *Playlist) :
+  cControl(OpenPlayer(Playlist->Current()->Filename))
 {
   m_DisplayReplay = NULL;
-  m_Files = Files;
-  m_File = NULL;
-  m_Index = Index;
-  m_Count = Count;
+  m_Playlist = Playlist;
   m_Speed = 0;
   m_ShowModeOnly = false;
 
@@ -1170,23 +1298,14 @@ cXinelibImagesControl::~cXinelibImagesControl()
   cStatus::MsgReplaying(this, NULL, NULL, false);
   Close();
 
-  if(m_Files) {
-    int i=0;
-    while(m_Files[i]) {
-      free(m_Files[i]);
-      m_Files[i] = NULL;
-      i++;
-    }
-    delete [] m_Files;
-    m_Files = NULL;
-  }
+  delete m_Playlist;
 }
 
 cXinelibImagePlayer *cXinelibImagesControl::OpenPlayer(const char *File)
 {
   m_Lock.Lock();
   if(!m_Player)
-    m_Player = new cXinelibImagePlayer(File);
+    m_Player = new cXinelibImagePlayer(&(cXinelibDevice::Instance()), File);
   m_Lock.Unlock();
   return m_Player;
 }
@@ -1203,12 +1322,8 @@ void cXinelibImagesControl::Close(void)
 void cXinelibImagesControl::Delete(void)
 {
   if(Interface->Confirm(tr("Delete image ?"))) {
-    if(!unlink(m_Files[m_Index])) {
-      free(m_Files[m_Index]);
-      for(int i=m_Index; i<m_Count; i++)
-	m_Files[i] = m_Files[i+1];
-      m_Count--;
-      m_Files[m_Count] = NULL;      
+    if(!unlink(m_Playlist->Current()->Filename)) {
+      m_Playlist->Del(m_Playlist->Current());
       Seek(0);
     }
   }
@@ -1216,36 +1331,20 @@ void cXinelibImagesControl::Delete(void)
 
 cOsdObject *cXinelibImagesControl::GetInfo(void)
 {
-  return new cMetainfoMenu(m_Files[m_Index]);
+  return new cMetainfoMenu(m_Playlist->Current()->Filename);
 }
 
 void cXinelibImagesControl::Seek(int Rel)
 {
-  if(m_Index == m_Count-1 && Rel>0)
-    m_Index = 0;
-  else if(m_Index == 0 && Rel<0)
-    m_Index = m_Count-1;
-  else
-    m_Index += Rel;
+  m_Playlist->Seek(Rel);
 
-  if(m_Index < 0) 
-    m_Index = 0; 
-  else if(m_Index >= m_Count) 
-    m_Index = m_Count;
-  
-  char *pt;
-  free(m_File);
-  m_File = strdup(m_Files[m_Index]);
-  if(NULL != (pt=strrchr(m_File, '/')))
-    strcpy(m_File, pt+1); 
-  if(NULL != (pt=strrchr(m_File, '.')))
-    *pt = 0;
+  const char *Filename = m_Playlist->Current()->Filename;
 
-  cStatus::MsgReplaying(this, m_File, m_Files[m_Index], true);
+  cStatus::MsgReplaying(this, m_Playlist->Current()->Title, Filename, true);
 
-  m_Player->ShowImage(m_Files[m_Index]);
+  m_Player->ShowImage(Filename);
   m_LastShowTime = time(NULL);
-  strn0cpy(xc.browse_images_dir, m_Files[m_Index], sizeof(xc.browse_images_dir));
+  strn0cpy(xc.browse_images_dir, Filename, sizeof(xc.browse_images_dir));
 }
 
 void cXinelibImagesControl::Show(void)
@@ -1258,12 +1357,14 @@ void cXinelibImagesControl::Show(void)
   }
 
   if(!m_ShowModeOnly) {
+    int count = m_Playlist->Count();
+    int index = m_Playlist->Current()->Index();
     char t[128] = "";
-    m_DisplayReplay->SetTitle(m_File);
-    m_DisplayReplay->SetProgress(m_Index, m_Count);
-    sprintf(t, "%d", m_Count);
+    m_DisplayReplay->SetTitle(m_Playlist->Current()->Title);
+    m_DisplayReplay->SetProgress(index, count);
+    sprintf(t, "%d", count);
     m_DisplayReplay->SetTotal( t );
-    sprintf(t, "%d", m_Index+1);
+    sprintf(t, "%d", index+1);
     m_DisplayReplay->SetCurrent( t );
   }
 
@@ -1282,13 +1383,12 @@ void cXinelibImagesControl::Hide(void)
 eOSState cXinelibImagesControl::ProcessKey(eKeys Key)
 {
   switch(Key) {
-    case kBack:    xc.main_menu_mode = ShowImages;
-                   Hide(); 
-                   Close(); 
-                   BackToMenu();
-                   //return osPlugin;		   
+    case kBack:    Hide();
+                   Close();
+                   BackToMenu(ShowImages);
+                   //return osPlugin;
 		   return osEnd;
-    case kYellow:  Delete(); 
+    case kYellow:  Delete();
                    break;
     case kStop:
     case kBlue:    Hide();
@@ -1344,3 +1444,144 @@ eOSState cXinelibImagesControl::ProcessKey(eKeys Key)
 
   return osContinue;
 }
+
+// --- Exteral interface ---------------------------------------------------
+
+//
+// cPlayerFactory
+//
+
+static cPlaylist *CreatePlaylist(const char *Mrl)
+{
+  cPlaylist *pl = new cPlaylist();
+  pl->Read(Mrl);
+  return pl;
+}
+
+cControl *CreateControl(cXinelibDevice *Dev,
+                        ePlayMode PlayMode,
+                        cPlaylist *Playlist,
+                        bool BackToMenu)
+{
+
+  if (PlayMode == pmVideoOnly) {
+    return new cXinelibImagesControl(Playlist);
+  }
+
+  LOGMSG("cPlayerFactory::Create(cPlaylist*) not implemented for PlayMode %d !",
+         (int)PlayMode);
+
+  return NULL;
+}
+
+cControl *CreateControl(cXinelibDevice *Dev,
+                        ePlayMode PlayMode,
+                        const char *Mrl,
+                        const char *SubFile,
+                        bool BackToMenu)
+{
+  // Special mrls
+
+  if (!strncmp(Mrl, "dvd:/", 5))
+    return new cXinelibDvdPlayerControl(Mrl);
+  if (xc.IsDvdImage(Mrl))
+    return new cXinelibDvdPlayerControl(Mrl);
+
+  if (!strncmp(Mrl, "bluray:/", 8))
+    return new cXinelibDvdPlayerControl(Mrl);
+
+  if (!strncmp(Mrl, "cdda:/", 6))
+    return new cXinelibPlayerControl(ShowMusic, Mrl);
+
+  // Playmode
+
+  if (PlayMode == pmAudioOnly)
+    return new cXinelibPlayerControl(ShowMusic, Mrl);
+  if (PlayMode == pmAudioVideo)
+    return new cXinelibPlayerControl(ShowFiles, Mrl, SubFile);
+  if (PlayMode == pmVideoOnly) {
+    return new cXinelibImagesControl(CreatePlaylist(Mrl));
+  }
+
+  // guess from playlist content
+
+  if (xc.IsPlaylistFile(Mrl)) {
+    cPlaylist Playlist;
+    Playlist.Read(Mrl);
+    if (Playlist.Count() < 1)
+      return NULL;
+
+    if (xc.IsAudioFile(Playlist.First()->Filename))
+      return new cXinelibPlayerControl(ShowMusic, Mrl);
+
+    if (xc.IsImageFile(Playlist.First()->Filename)) {
+      return new cXinelibImagesControl(CreatePlaylist(Mrl));
+    }
+
+    return new cXinelibPlayerControl(ShowFiles, Mrl);
+  }
+
+  // guess from file type
+
+  if (xc.IsAudioFile(Mrl))
+    return new cXinelibPlayerControl(ShowMusic, Mrl);
+
+  if (xc.IsVideoFile(Mrl))
+    return new cXinelibPlayerControl(ShowFiles, Mrl, SubFile);
+
+  if (xc.IsImageFile(Mrl)) {
+    return new cXinelibImagesControl(CreatePlaylist(Mrl));
+  }
+
+  // default
+
+  return new cXinelibPlayerControl(ShowFiles, Mrl, SubFile);
+}
+
+void cPlayerFactory::Queue(const char *Mrl)
+{
+  cXinelibPlayerControl::Queue(Mrl);
+}
+
+bool cPlayerFactory::IsOpen(void)
+{
+  return cXinelibPlayerControl::IsOpen();
+}
+
+bool cPlayerFactory::Launch(ePlayMode PlayMode,
+                            const char *Mrl,
+                            const char *SubFile,
+                            bool BackToMenu)
+{
+  cXinelibDevice *Dev     = &(cXinelibDevice::Instance());
+  cControl       *Control = CreateControl(Dev, PlayMode, Mrl, SubFile, BackToMenu);
+
+  if (!Control) {
+    LOGMSG("cPlayerFactory::Launch(%s) failed !", Mrl);
+    return false;
+  }
+
+  cControl::Shutdown();
+  cControl::Launch(Control);
+
+  return true;
+}
+
+bool cPlayerFactory::Launch(ePlayMode PlayMode,
+                            cPlaylist *Playlist,
+                            bool BackToMenu)
+{
+  cXinelibDevice *Dev     = &(cXinelibDevice::Instance());
+  cControl       *Control = CreateControl(Dev, PlayMode, Playlist, BackToMenu);
+
+  if (!Control) {
+    LOGMSG("cPlayerFactory::Launch(cPlaylist*) failed !");
+    delete Playlist;
+    return false;
+  }
+
+  cControl::Shutdown();
+  cControl::Launch(Control);
+
+  return true;
+}
