@@ -4,20 +4,18 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: setup_menu.c,v 1.71 2009/08/07 12:29:31 phintuka Exp $
+ * $Id: setup_menu.c,v 1.62.2.3 2009/11/16 11:34:49 phintuka Exp $
  *
  */
 
-#include "features.h"
-
 #include <vdr/config.h>
 #include <vdr/plugin.h>
 #include <vdr/remote.h>
-#include <vdr/i18n.h>
 
 #include "config.h"
 #include "device.h"
 #include "menuitems.h"
+#include "i18n.h"      // trVDR for vdr-1.4.x
 #include "osd.h"       // cXinelibOsdProvider::RefreshOsd()
 #include "setup_menu.h"
 
@@ -694,10 +692,8 @@ void cMenuSetupVideo::Set(void)
   Add(new cMenuEditIntItem(tr("Saturation"), &newconfig.saturation,-1,0xffff));
   Add(new cMenuEditIntItem(tr("Contrast"),   &newconfig.contrast, -1, 0xffff));
   Add(new cMenuEditIntItem(tr("Brightness"), &newconfig.brightness,-1,0xffff));
-#ifdef HAVE_VDPAU
   Add(new cMenuEditIntItem(tr("Sharpness"),  &newconfig.sharpness, -1,0xffff));
   Add(new cMenuEditIntItem(tr("Noise Reduction"), &newconfig.noise_reduction, -1,0xffff));
-#endif
 #else
   Add(ctrl_hue = new cMenuEditStraItem(tr("HUE"), &newconfig.hue, 33, 
 				       controls));
@@ -710,7 +706,6 @@ void cMenuSetupVideo::Set(void)
   Add(ctrl_brightness = 
       new cMenuEditStraItem(tr("Brightness"), &newconfig.brightness, 33, 
 			    controls));
-#ifdef HAVE_VDPAU
   Add(ctrl_sharpness = 
       new cMenuEditStraItem(tr("Sharpness"), &newconfig.sharpness, 33, 
 			    controls));
@@ -718,7 +713,6 @@ void cMenuSetupVideo::Set(void)
       new cMenuEditStraItem(tr("Noise Reduction"), &newconfig.noise_reduction, 33, 
 			    controls));
 #endif
-#endif
 
 #ifdef DEVICE_SUPPORTS_IBP_TRICKSPEED
   Add(new cMenuEditBoolItem(tr("Smooth fast forward"),
@@ -886,18 +880,19 @@ class cMenuSetupOSD : public cMenuSetupPage
     int orig_alpha_correction;
     int orig_alpha_correction_abs;
 
-    cOsdItem *ctrl_size;
-    cOsdItem *ctrl_width;
     cOsdItem *ctrl_scaling;
     cOsdItem *ctrl_alpha;
     cOsdItem *ctrl_alpha_abs;
     cOsdItem *ctrl_blending;
     cOsdItem *ctrl_lowres;
+#if VDRVERSNUM < 10515
+    cOsdItem *ctrl_spulang0;
+#endif
 
   protected:
     virtual void Store(void);
     void Set(void);
-
+  
   public:
     cMenuSetupOSD(void);
     ~cMenuSetupOSD();
@@ -922,34 +917,24 @@ cMenuSetupOSD::~cMenuSetupOSD()
 }
 
 void cMenuSetupOSD::Set(void)
-{
+{ 
   SetPlugin(cPluginManager::GetPlugin(PLUGIN_NAME_I18N));
   int current = Current();
   Clear();
 
-  ctrl_size = NULL;
-  ctrl_width = NULL;
   ctrl_scaling = NULL;
   ctrl_blending = NULL;
   ctrl_lowres = NULL;
   ctrl_alpha = NULL;
   ctrl_alpha_abs = NULL;
+#if VDRVERSNUM < 10515
+  ctrl_spulang0 = NULL;
+#endif
 
   Add(NewTitle(tr("On-Screen Display")));
-  Add(new cMenuEditBoolItem(tr("Hide main menu"),
+  Add(new cMenuEditBoolItem(tr("Hide main menu"), 
 			    &newconfig.hide_main_menu));
 
-  Add(ctrl_size =
-      new cMenuEditStraI18nItem(tr("Resolution"), &newconfig.osd_size,
-	                        OSD_SIZE_count, xc.s_osdSizes));
-  if (newconfig.osd_size == OSD_SIZE_custom) {
-    Add(ctrl_width =
-        new cMenuEditTypedIntItem(tr("  Width"), "px",
-                                  &newconfig.osd_width, 480, 2048));
-    Add(new cMenuEditTypedIntItem(tr("  Height"), "px",
-                                  &newconfig.osd_height, 576, 1200));
-  }
-
   Add(ctrl_blending =
       new cMenuEditBoolItem(tr("Blending method"),
 			    &newconfig.osd_blending,
@@ -965,8 +950,12 @@ void cMenuSetupOSD::Set(void)
       new cMenuEditStraI18nItem(tr("Scaling method"), &newconfig.osd_scaling,
 	                        OSD_SCALING_count, xc.s_osdScalings));
 
-  Add(new cMenuEditStraI18nItem(tr("Show all layers"), &newconfig.osd_mixer,
-                                OSD_MIXER_count, xc.s_osdMixers));
+#if VDRVERSNUM >= 10509
+# if 0
+  Add(new cMenuEditStraI18nItem(tr("Show all layers"), &newconfig.osd_mixer, 
+			OSD_MIXER_count, xc.s_osdMixers));
+# endif
+#endif
 
   Add(ctrl_alpha =
       new cMenuEditTypedIntItem(tr("Dynamic transparency correction"), "%", 
@@ -977,13 +966,32 @@ void cMenuSetupOSD::Set(void)
 				&newconfig.alpha_correction_abs, -0xff, 0xff,
 				tr("Off")));
   
+#if VDRVERSNUM < 10515
+  Add(NewTitle(tr("Subtitles")));
+  Add(new cMenuEditBoolItem(trVDR("Setup.EPG$Preferred languages"), 
+			    &newconfig.spu_autoshow));
+  if(newconfig.spu_autoshow) {
+    Add(ctrl_spulang0 =
+	new cMenuEditStrItem(trVDR("Setup.EPG$Preferred language"),
+			     newconfig.spu_lang[0], 4, LangNameChars));
+    Add(new cMenuEditStrItem(trVDR("Setup.EPG$Preferred language"),
+			     newconfig.spu_lang[1], 4, LangNameChars));
+    Add(new cMenuEditStrItem(trVDR("Setup.EPG$Preferred language"),
+			     newconfig.spu_lang[2], 4, LangNameChars));
+    Add(new cMenuEditStrItem(trVDR("Setup.EPG$Preferred language"),
+			     newconfig.spu_lang[3], 4, LangNameChars));
+  }
+#endif
+
   Add(new cMenuEditStraI18nItem(tr("External subtitle size"),
 				&newconfig.extsub_size, SUBTITLESIZE_count, xc.s_subtitleSizes));
 
+#if VDRVERSNUM >= 10510
   Add(new cMenuEditBoolItem(tr("DVB subtitle decoder"),
 			    &newconfig.dvb_subtitles,
 			    "VDR",
 			    "frontend"));
+#endif
 
   if(current<1) current=1; /* first item is not selectable */
   SetCurrent(Get(current));
@@ -1009,15 +1017,16 @@ eOSState cMenuSetupOSD::ProcessKey(eKeys Key)
   else if(item == ctrl_alpha_abs)
     xc.alpha_correction_abs = newconfig.alpha_correction_abs;
 
-  if (newconfig.osd_size == OSD_SIZE_custom && !ctrl_width)
-    Set();
-  if (newconfig.osd_size != OSD_SIZE_custom && ctrl_width)
-    Set();
-
   if(newconfig.osd_blending==OSD_BLENDING_SOFTWARE && !ctrl_lowres)
     Set();
   if(newconfig.osd_blending!=OSD_BLENDING_SOFTWARE && ctrl_lowres)
     Set();
+#if VDRVERSNUM < 10515
+  if(newconfig.spu_autoshow && !ctrl_spulang0)
+    Set();
+  if(!newconfig.spu_autoshow && ctrl_spulang0)
+    Set();
+#endif
   return state;
 }
 
@@ -1033,9 +1042,6 @@ void cMenuSetupOSD::Store(void)
   orig_alpha_correction = xc.alpha_correction;
   orig_alpha_correction_abs = xc.alpha_correction_abs;
 
-  SetupStore("OSD.Size",            xc.s_osdSizes[xc.osd_size]);
-  SetupStore("OSD.Width",           xc.osd_width);
-  SetupStore("OSD.Height",          xc.osd_height);
   SetupStore("OSD.Scaling",         xc.osd_scaling);
   SetupStore("OSD.HideMainMenu",    xc.hide_main_menu);
   SetupStore("OSD.LayersVisible",   xc.osd_mixer);
@@ -1053,6 +1059,13 @@ void cMenuSetupOSD::Store(void)
   SetupStore("OSD.AlphaCorrectionAbs", xc.alpha_correction_abs);
 
   SetupStore("OSD.ExtSubSize", xc.extsub_size);
+#if VDRVERSNUM < 10515
+  SetupStore("OSD.SpuAutoSelect", xc.spu_autoshow);
+  SetupStore("OSD.SpuLang0", xc.spu_lang[0]);
+  SetupStore("OSD.SpuLang1", xc.spu_lang[1]);
+  SetupStore("OSD.SpuLang2", xc.spu_lang[2]);
+  SetupStore("OSD.SpuLang3", xc.spu_lang[3]);
+#endif
   SetupStore("OSD.DvbSubtitles", xc.dvb_subtitles);
 
   Setup.Save();
@@ -1675,27 +1688,38 @@ void cMenuSetupMediaPlayer::Set(void)
   Add(new cMenuEditBoolItem(tr("Arrow keys control DVD playback"),
                             &newconfig.dvd_arrow_keys_control_playback));
 
+  Add(NewTitle(tr("Media Player")));
+  Add(new cMenuEditBitItem(tr("Play file >>"),        &newconfig.media_menu_items, MEDIA_MENU_FILES));
+  Add(new cMenuEditBitItem(tr("Play music >>"),       &newconfig.media_menu_items, MEDIA_MENU_MUSIC));
+  Add(new cMenuEditBitItem(tr("View images >>"),      &newconfig.media_menu_items, MEDIA_MENU_IMAGES));
+  Add(new cMenuEditBitItem(tr("Play DVD disc >>"),    &newconfig.media_menu_items, MEDIA_MENU_DVD));
+  Add(new cMenuEditBitItem(tr("Play audio CD >>"),    &newconfig.media_menu_items, MEDIA_MENU_CD));
+  Add(new cMenuEditBitItem(tr("Play BluRay disc >>"), &newconfig.media_menu_items, MEDIA_MENU_BLURAY));
+  Add(new cMenuEditBitItem(tr("Video settings"),      &newconfig.media_menu_items, MEDIA_MENU_VIDEO_SETUP));
+  Add(new cMenuEditBitItem(tr("Audio settings"),      &newconfig.media_menu_items, MEDIA_MENU_AUDIO_SETUP));
+
   if(current<1) current=1; /* first item is not selectable */
   SetCurrent(Get(current));
   Display();
 }
 
 eOSState cMenuSetupMediaPlayer::ProcessKey(eKeys Key)
-{ 
+{
   eOSState state = cMenuSetupPage::ProcessKey(Key);
   return state;
 }
 
 void cMenuSetupMediaPlayer::Store(void)
-{ 
+{
   memcpy(&xc, &newconfig, sizeof(config_t));
-  
+
   SetupStore("Playlist.Tracknumber", xc.playlist_tracknumber);
   SetupStore("Playlist.Album", xc.playlist_album);
   SetupStore("Playlist.Artist", xc.playlist_artist);
   SetupStore("Media.CacheImplicitPlaylists", xc.cache_implicit_playlists);
   SetupStore("Media.EnableID3Scanner", xc.enable_id3_scanner);
   SetupStore("Media.DVD.ArrowKeysControlPlayback", xc.dvd_arrow_keys_control_playback);
+  SetupStore("Media.MenuItems", xc.media_menu_items);
 
   Setup.Save();
 }
@@ -1736,7 +1760,11 @@ void cTestGrayscale::Show()
   int i;
 
   if(!m_Osd)
+#if VDRVERSNUM >= 10509
     m_Osd = cOsdProvider::NewOsd(OSD_X, OSD_Y, 0);
+#else
+    m_Osd = cOsdProvider::NewOsd(OSD_X, OSD_Y);
+#endif
 
   if(m_Osd) {
     if (m_Osd->CanHandleAreas(areas, sizeof(areas) / sizeof(tArea) ) == oeOk) {
@@ -1835,7 +1863,11 @@ void cTestBitmap::Show()
   int x, y, bit = 0;
 
   if(!m_Osd) {
+#if VDRVERSNUM >= 10509
     m_Osd = cOsdProvider::NewOsd(OSD_X, OSD_Y, 0);
+#else
+    m_Osd = cOsdProvider::NewOsd(OSD_X, OSD_Y);
+#endif
 
     if(m_Osd) {
       if (m_Osd->CanHandleAreas(areas, sizeof(areas) / sizeof(tArea) ) == oeOk) {
