@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: playlist.c,v 1.23 2009/06/02 08:36:16 phintuka Exp $
+ * $Id: playlist.c,v 1.33 2010/12/19 14:25:38 phintuka Exp $
  *
  */
 
@@ -132,6 +132,39 @@ static const char *shell_escape(char *buf, int buflen, const cString& src, char
 }
 #endif
 
+#if defined(HAVE_LIBEXTRACTOR) && EXTRACTOR_VERSION >= 0x00060000
+static int extractor_callback_id3(void *priv,
+                                  const char *plugin_name,
+                                  enum EXTRACTOR_MetaType type,
+                                  enum EXTRACTOR_MetaFormat format,
+                                  const char *data_mime_type,
+                                  const char *data,
+                                  size_t data_len)
+{
+  if (format == EXTRACTOR_METAFORMAT_UTF8) {
+    cPlaylistItem *Item = (cPlaylistItem *)priv;
+    switch (type) {
+      case EXTRACTOR_METATYPE_TITLE:
+        Item->Title = data;
+        break;
+      case EXTRACTOR_METATYPE_ARTIST:
+        Item->Artist = data;
+        break;
+      case EXTRACTOR_METATYPE_ALBUM:
+        Item->Album = data;
+        break;
+      case EXTRACTOR_METATYPE_TRACK_NUMBER:
+        Item->Tracknumber = strlen(data) == 1 ? *cString::sprintf("0%s", data) : data;
+        break;
+      default:
+        break;
+    }
+  }
+  return 0;
+}
+#endif // defined(HAVE_LIBEXTRACTOR) && EXTRACTOR_VERSION >= 0x00060000
+
+
 class cID3Scanner : public cThread 
 {
  public:
@@ -169,6 +202,12 @@ class cID3Scanner : public cThread
       if(xc.IsAudioFile(Item->Filename)) {
         LOGDBG("Scanning metainfo for file %s", *Item->Filename);
 #ifdef HAVE_LIBEXTRACTOR
+#  if EXTRACTOR_VERSION >= 0x00060000
+	EXTRACTOR_PluginList * plugins;
+	plugins = EXTRACTOR_plugin_add_defaults(EXTRACTOR_OPTION_DEFAULT_POLICY);
+	EXTRACTOR_extract(plugins, *Item->Filename, NULL, 0, (EXTRACTOR_MetaDataProcessor)&extractor_callback_id3, Item);
+	EXTRACTOR_plugin_remove_all(plugins); /* unload plugins */
+#  else // EXTRACTOR_VERSION >= 0x00060000
         EXTRACTOR_ExtractorList * plugins;
         EXTRACTOR_KeywordList   * md_list;
         plugins = EXTRACTOR_loadDefaultLibraries();
@@ -189,7 +228,8 @@ class cID3Scanner : public cThread
          }
         EXTRACTOR_freeKeywords(md_list);
         EXTRACTOR_removeAll(plugins); /* unload plugins */
-#else
+#  endif // EXTRACTOR_VERSION >= 0x00060000
+#else // HAVE_LIBEXTRACTOR
         char buf[4096];
         cString Cmd = "";
         if(!strcasecmp((Item->Filename) + strlen(Item->Filename) - 5, ".flac"))
@@ -224,7 +264,7 @@ class cID3Scanner : public cThread
               Item->Tracknumber = cString::sprintf("%s%s", strlen(pt) == 13 ? "0" : "", (pt+12));
           }
         }
-#endif
+#endif // HAVE_LIBEXTRACTOR
       }
     }
     LOGDBG("ID3Scanner Done.");
@@ -451,6 +491,17 @@ void cPlaylist::Del(cPlaylistItem *it)
   m_Version++;
 }
 
+void cPlaylist::Move(int From, int To)
+{
+  cMutexLock ml(&m_Lock);
+
+  if (Count() < 3)
+    return;
+
+  cListBase::Move(From, To);
+  m_Version++;
+}
+
 void cPlaylist::SetCurrent(cPlaylistItem *current) 
 {
   cMutexLock ml(&m_Lock);
@@ -473,6 +524,24 @@ cPlaylistItem *cPlaylist::Prev(void)
   return NULL;
 }
 
+cPlaylistItem *cPlaylist::Seek(int Rel)
+{
+  cMutexLock ml(&m_Lock);
+  if (!Current())
+    return NULL;
+
+  if (Rel > 0) {
+    while (Rel--)
+      m_Current = (cList<cPlaylistItem>::Next(Current()) ?: Last());
+
+  } else if (Rel < 0) {
+    while (Rel++)
+      m_Current = (cList<cPlaylistItem>::Prev(Current()) ?: First());
+  }
+
+  return Current();
+}
+
 bool cPlaylist::StoreCache(void) 
 {
   if(!xc.cache_implicit_playlists ||
@@ -634,7 +703,7 @@ static FILE *open_http(const char *PlaylistFile)
 
 int cPlaylist::ScanFolder(const char *FolderName, 
 			  bool Recursive,
-			  bool (config_t::*Filter)(const char *))
+			  bool (*Filter)(const char *))
 {
   cMutexLock ml(&m_Lock);  
   static int depth = 0;
@@ -676,7 +745,7 @@ int cPlaylist::ScanFolder(const char *FolderName,
 	    if (stat(Buffer, &st) != 0)
 	      continue;
 	  }
-	  if((xc.*Filter)(Buffer)) {
+	  if((*Filter)(Buffer)) {
 	    /* TODO: Should ScanDir add contents of playlist files ... ? */
 	    if(Filter == &config_t::IsPlaylistFile || !xc.IsPlaylistFile(Buffer)) {
 	      n++;
@@ -917,18 +986,14 @@ bool cPlaylist::Read(const char *PlaylistFile, bool Recursive)
   return Result;
 }
 
-cString cPlaylist::EscapeMrl(const char *mrl)
+static cString EscapeString(const char *s)
 {
   static const uint8_t hex[16] = {'0','1','2','3','4','5','6','7','8','9','a','b','c','d','e','f'};
-  const uint8_t *fn = (const uint8_t*)mrl;
-  int size = strlen(mrl) + 16;
+  const uint8_t *fn = (const uint8_t*)s;
+  int size = strlen(s) + 16;
   char *buf = (char *)malloc(size);
-  int i = 0, found = 0;
-  LOGDBG("cPlaylist::EscapeMrl('%s')", fn);
-
-  // Wait for first '/' (do not escape mrl start dvd:/, http://a@b/, ...)
-  if (*fn == '/')
-    found = 3;
+  int i = 0;
+  LOGVERBOSE("EscapeString('%s')", fn);
 
   while (*fn) {
     if(size-7 < i)
@@ -938,35 +1003,18 @@ cString cPlaylist::EscapeMrl(const char *mrl)
     case 127 ... 255:
     case '#':
     case '%':
+    case '?':
     case ':':
     case ';':
     case '\'':
     case '\"':
     case '(':
     case ')':
-      if (found > 2) {
-	buf[i++] = '%';
-	buf[i++] = hex[(*fn & 0xf0)>>4];
-	buf[i++] = hex[(*fn & 0x0f)];
-	break;
-      }
+      buf[i++] = '%';
+      buf[i++] = hex[(*fn & 0xf0)>>4];
+      buf[i++] = hex[(*fn & 0x0f)];
+      break;
     default:
-      // file:/... -> only one '/' before escaping
-      // http://.../ --> three '/' before escaping
-      if(!found && (fn[0] == ':' && fn[1] == '/')) {
-	if(fn[2] == '/') {
-	  // ex. http://user:pass@host/... --> wait for third '/'
-	  buf[i++] = *fn++;
-	  buf[i++] = *fn++;
-	  found += 2;
-	} else {
-	  //  ex. file:/local_file
-	  buf[i++] = *fn++;
-	  found += 3;
-	}
-      } else if(*fn == '/') {
-	found++;
-      }
       buf[i++] = *fn;
       break;
     }
@@ -974,13 +1022,22 @@ cString cPlaylist::EscapeMrl(const char *mrl)
   }
 
   buf[i] = 0;
-  LOGDBG("    --> '%s'", buf);
+  LOGVERBOSE("    --> '%s'", buf);
   return cString(buf, true);
 }
 
+cString cPlaylist::BuildMrl(const char *proto, const char *s1, const char *s2, const char *s3, const char *s4)
+{
+  return cString::sprintf("%s:%s%s%s%s",
+                          proto,
+                          s1 ? *EscapeString(s1) : "",
+                          s2 ? *EscapeString(s2) : "",
+                          s3 ? *EscapeString(s3) : "",
+                          s4 ? *EscapeString(s4) : "");
+}
+
 cString cPlaylist::GetEntry(cPlaylistItem *i, bool isPlaylist, bool isCurrent)
 {
-
   cString Entry = "";
   if ((*i->Artist && xc.playlist_artist) || (*i->Album && xc.playlist_album)) {
       Entry = cString::sprintf("%s%s%s%s%s%s(%s%s%s)",
