@@ -4,7 +4,7 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: ts.c,v 1.14 2009/09/09 11:58:40 phintuka Exp $
+ * $Id: ts.c,v 1.20 2010/02/03 10:09:35 phintuka Exp $
  *
  */
 
@@ -529,26 +529,35 @@ struct ts_state_s {
 
   uint8_t  inside_pes; /* Scanning ES (PES start code seen and skipped) */
 
-  uint32_t buf_len;    /* bytes queued */
-  uint32_t buf_size;   /* buffer size */
+  size_t   buf_len;    /* bytes queued */
+  size_t   buf_size;   /* buffer size */
   uint8_t  buf[0];     /* payload: partial PES / video stream header etc. */
 };
 
-ts_state_t *ts_state_init(int buffer_size)
+ts_state_t *ts_state_init(size_t buffer_size)
 {
   if (buffer_size < 8 * TS_SIZE)
     buffer_size = 8 * TS_SIZE;
+  if (buffer_size > 4*1024*1024) {
+    LOGMSG("ERROR: ts_state_init(%zd)", buffer_size);
+    buffer_size = 4*1024*1024;
+  }
 
   ts_state_t *ts = (ts_state_t*)calloc(1, sizeof(ts_state_t) + buffer_size);
-  ts->buf_size = buffer_size;
+  if (ts) {
+    ts->buf_size = buffer_size;
+  }
+
   return ts;
 }
 
 void ts_state_reset(ts_state_t *ts)
 {
-  int buf_size = ts->buf_size;
-  memset(ts, 0, sizeof(ts_state_t));
-  ts->buf_size = buf_size;
+  if (ts) {
+    size_t buf_size = ts->buf_size;
+    memset(ts, 0, sizeof(ts_state_t));
+    ts->buf_size = buf_size;
+  }
 }
 
 void ts_state_dispose(ts_state_t *ts)
@@ -563,7 +572,7 @@ void ts_state_dispose(ts_state_t *ts)
  *  - PUSI resets the buffer
  *  - all data before first PUSI is discarded
  */
-static int ts_add_payload(ts_state_t *ts, const uint8_t *data)
+static size_t ts_add_payload(ts_state_t *ts, const uint8_t *data)
 {
   /* start from PUSI */
   if (!ts->pusi_seen) {
@@ -574,12 +583,12 @@ static int ts_add_payload(ts_state_t *ts, const uint8_t *data)
   }
 
   if (ts->buf_len >= ts->buf_size - TS_SIZE) {
-    LOGMSG("ts_add_payload: buffer full");
+    LOGDBG("ts_add_payload: buffer full");
     ts->buf_len -= TS_SIZE;
-    memcpy(ts->buf, ts->buf+TS_SIZE, ts->buf_len);
+    memmove(ts->buf, ts->buf+TS_SIZE, ts->buf_len);
   }
 
-  int len = ts_PAYLOAD_SIZE(data);
+  size_t len = ts_PAYLOAD_SIZE(data);
   if (len > 0) {
     memcpy(ts->buf + ts->buf_len, ts_GET_PAYLOAD(data), len);
     ts->buf_len += len;
@@ -591,11 +600,11 @@ static int ts_add_payload(ts_state_t *ts, const uint8_t *data)
 /*
  * ts_skip_payload()
  */
-static void ts_skip_payload(ts_state_t *ts, unsigned int n)
+static void ts_skip_payload(ts_state_t *ts, size_t n)
 {
   if (n < ts->buf_len) {
     ts->buf_len -= n;
-    memcpy(ts->buf, ts->buf + n, ts->buf_len);
+    memmove(ts->buf, ts->buf + n, ts->buf_len);
   } else {
     ts->buf_len = 0;
   }
@@ -607,17 +616,17 @@ static void ts_skip_payload(ts_state_t *ts, unsigned int n)
  * - discard all data until startcode (00 00 01) is found
  * - returns number of bytes left
  */
-static int ts_scan_startcode(ts_state_t *ts)
+static size_t ts_scan_startcode(ts_state_t *ts)
 {
   if (ts->buf_len > 2) {
     /* scan for PES or MPEG 00 00 01 */
-    unsigned int i = 0, n = ts->buf_len - 2;
+    size_t i = 0, n = ts->buf_len - 2;
     while (i < n) {
-      if (ts->buf[i+2] != 1)
-        i += 3;
-      else if(ts->buf[i+1])
+      if (ts->buf[i+1])
         i += 2;
-      else if(ts->buf[i])
+      else if (ts->buf[i])
+        i++;
+      else if (ts->buf[i+2] != 1)
         i++;
       else
         break;
@@ -650,7 +659,7 @@ static int ts_get_pes(ts_state_t *ts, const uint8_t *data)
 int64_t ts_get_pts(ts_state_t *ts, const uint8_t *data)
 {
   int64_t pts = NO_PTS;
-  int     cnt = ts_get_pes(ts, data);
+  size_t  cnt = ts_get_pes(ts, data);
 
   if (cnt > 14) {
     pts = pes_get_pts(ts->buf, ts->buf_len);
@@ -676,7 +685,55 @@ int ts_get_picture_type(ts_state_t *ts, const uint8_t *data, int h264)
  * ts_get_video_size()
  */
 
+#include "h264.h"
+#include "mpeg.h"
+
 int ts_get_video_size(ts_state_t *ts, const uint8_t *data, video_size_t *size, int h264)
 {
+  /* Accumulate data. Skip all data until start code. */
+  if (ts_get_pes(ts, data) < 9)
+    return 0;
+
+  /* start scanning PES payload */
+  if (!ts->inside_pes) {
+
+    /* Skip PES header */
+    ts_skip_payload(ts, PES_HEADER_LEN(ts->buf));
+    ts->inside_pes = 1;
+
+    /* move to first ES header */
+    ts_scan_startcode(ts);
+  }
+
+  /* scan for start code */
+
+  while (ts->buf_len > 9) {
+    uint8_t *buf = ts->buf;
+
+    /* MPEG2 sequence start code */
+    if (h264 != 1 && IS_SC_SEQUENCE(buf)) {
+      if (mpeg2_get_video_size(ts->buf, ts->buf_len, size)) {
+        ts_state_reset(ts);
+        return 1;
+      }
+      if (ts->buf_len < ts->buf_size - TS_SIZE)
+        return 0;
+    }
+
+    /* H.264 NAL AUD */
+    if (h264 != 0 && IS_NAL_AUD(buf)) {
+      if (h264_get_video_size(ts->buf, ts->buf_len, size)) {
+        ts_state_reset(ts);
+        return 1;
+      }
+      if (ts->buf_len < ts->buf_size - TS_SIZE)
+        return 0;
+    }
+
+    /* find next start code */
+    ts_skip_payload(ts, 4);
+    ts_scan_startcode(ts);
+  }
+
   return 0;
 }
