@@ -45,19 +45,28 @@
 # include <xine/video_overlay.h>
 #endif
 
-#define TRACE(x...) printf(x)
-/*#define TRACE(x...) */
-#define ERROR(x...) fprintf(stderr, x)
-/*#define ERROR(x...) lprintf(x) */
+#define XINE_HDMV_TRACE(x...) printf(x)
+/*#define XINE_HDMV_TRACE(x...) */
+#define XINE_HDMV_ERROR(x...) fprintf(stderr, "spuhdmv: " x)
+/*#define XINE_HDMV_ERROR(x...) lprintf(x) */
 
 #ifndef EXPORTED
 #  define EXPORTED __attribute__((visibility("default")))
 #endif
 
 #ifndef BUF_SPU_HDMV
-#  define BUF_SPU_HDMV            0x04180000
+#  define BUF_SPU_HDMV            0x04080000
 #endif
 
+enum {
+  SEGTYPE_PALETTE              = 0x14,
+  SEGTYPE_OBJECT               = 0x15,
+  SEGTYPE_PRESENTATION_SEGMENT = 0x16,
+  SEGTYPE_WINDOW_DEFINITION    = 0x17,
+  SEGTYPE_INTERACTIVE          = 0x18,
+  SEGTYPE_END_OF_DISPLAY       = 0x80,
+} eSegmentType;
+
 /*
  * cached palette (xine-lib format)
  */
@@ -67,6 +76,8 @@ struct subtitle_clut_s {
   uint32_t         color[256];
   uint8_t          trans[256];
   subtitle_clut_t *next;
+
+  int shown;
 };
 
 /*
@@ -79,14 +90,18 @@ struct subtitle_object_s {
   uint16_t    width, height;
 
   rle_elem_t *rle;
-  uint        num_rle;
+  unsigned int num_rle;
   size_t      data_size;
 
+#if 0
   uint8_t    *raw_data; /* partial RLE data in HDMV format */
   size_t      raw_data_len;
   size_t      raw_data_size;
+#endif
 
   subtitle_object_t *next;
+
+  int shown;
 };
 
 /*
@@ -99,6 +114,8 @@ struct window_def_s {
   uint16_t  width, height;
 
   window_def_t *next;
+
+  int shown;
 };
 
 
@@ -118,8 +135,78 @@ struct composition_object_s {
   uint16_t    crop_width, crop_height;
 
   composition_object_t *next;
+
+  int shown;
 };
 
+typedef struct composition_descriptor_s composition_descriptor_t;
+struct composition_descriptor_s {
+  uint16_t number;
+  uint8_t  state;
+};
+
+typedef struct presentation_segment_s presentation_segment_t;
+struct presentation_segment_s {
+  composition_descriptor_t comp_descr;
+
+  uint8_t palette_update_flag;
+  uint8_t palette_id_ref;
+  uint8_t object_number;
+
+  composition_object_t *comp_objs;
+
+  presentation_segment_t *next;
+
+  int64_t pts;
+  int     shown;
+};
+
+/*
+ * list handling
+ */
+
+#define LIST_REPLACE(list, obj, FREE_FUNC)      \
+  do {						\
+    unsigned int id = obj->id;			\
+						\
+    /* insert to list */			\
+    obj->next = list;				\
+    list = obj;					\
+						\
+    /* remove old */				\
+    while (obj->next && obj->next->id != id)	\
+      obj = obj->next;				\
+    if (obj->next) {				\
+      void *tmp = (void*)obj->next;		\
+      obj->next = obj->next->next;		\
+      FREE_FUNC(tmp);				\
+    }						\
+  } while (0);
+
+#define LIST_DESTROY(list, FREE_FUNC) \
+  while (list) {	       \
+    void *tmp = (void*)list;   \
+    list = list->next;	       \
+    FREE_FUNC(tmp);            \
+  }
+
+static void free_subtitle_object(void *ptr)
+{
+  if (ptr) {
+    free(((subtitle_object_t*)ptr)->rle);
+    free(ptr);
+  }
+}
+static void free_presentation_segment(void *ptr)
+{
+  if (ptr) {
+    presentation_segment_t *seg = (presentation_segment_t*)ptr;
+    LIST_DESTROY(seg->comp_objs, free);
+    free(ptr);
+  }
+}
+
+
 /*
  * segment_buffer_t
  *
@@ -182,7 +269,7 @@ static void segbuf_parse_segment_header(segment_buffer_t *buf)
     if ( buf->segment_type < 0x14 ||
 	 ( buf->segment_type > 0x18 &&
 	   buf->segment_type != 0x80)) {
-      ERROR("unknown segment type, resetting\n");
+      XINE_HDMV_ERROR("unknown segment type, resetting\n");
       segbuf_reset(buf);
     }
   } else {
@@ -221,10 +308,10 @@ static void segbuf_skip_segment(segment_buffer_t *buf)
 
     segbuf_parse_segment_header(buf);
 
-    TRACE("  skip_segment: %d bytes left\n", (uint)buf->len);
+    XINE_HDMV_TRACE("  skip_segment: %zd bytes left\n", buf->len);
   } else {
-    ERROR("  skip_segment: ERROR - %d bytes queued, %d required\n",
-          (uint)buf->len, buf->segment_len);
+    XINE_HDMV_ERROR("  skip_segment: ERROR - %zd bytes queued, %d required\n",
+                    buf->len, buf->segment_len);
     segbuf_reset (buf);
   }
 }
@@ -233,7 +320,7 @@ static void segbuf_skip_segment(segment_buffer_t *buf)
  * access segment data
  */
 
-uint8_t segbuf_segment_type(segment_buffer_t *buf)
+static uint8_t segbuf_segment_type(segment_buffer_t *buf)
 {
   return buf->segment_type;
 }
@@ -249,7 +336,7 @@ static uint8_t segbuf_get_u8(segment_buffer_t *buf)
 {
   if (!(buf->error = ++buf->segment_data > buf->segment_end))
     return buf->segment_data[-1];
-  ERROR("segbuf_get_u8: read failed (end of segment reached) !");
+  XINE_HDMV_ERROR("segbuf_get_u8: read failed (end of segment reached) !");
   return 0;
 }
 
@@ -263,19 +350,6 @@ static uint32_t segbuf_get_u24(segment_buffer_t *buf)
   return (segbuf_get_u8(buf) << 16) | (segbuf_get_u8(buf) << 8) | segbuf_get_u8(buf);
 }
 
-uint8_t *segbuf_get_string(segment_buffer_t *buf, size_t len)
-{
-  if (len > 0) {
-    uint8_t *val = buf->segment_data;
-    buf->segment_data += len;
-    if (buf->segment_data <= buf->segment_end)
-      return val;
-  }
-  ERROR("segbuf_get_string(%d): read failed (end of segment reached) !", (int)len);
-  buf->error = 1;
-  return NULL;
-}
-
 /*
  * decode segments
  */
@@ -293,12 +367,12 @@ static subtitle_clut_t *segbuf_decode_palette(segment_buffer_t *buf)
     return NULL;
 
   if (len % 5) {
-    ERROR("  decode_palette: segment size error (%d ; expected %d for %d entries)\n",
-	  (uint)len, (uint)(5 * entries), (uint)entries);
+    XINE_HDMV_ERROR("  decode_palette: segment size error (%zd ; expected %zd for %zd entries)\n",
+                    len, (5 * entries), entries);
     return NULL;
   }
-  TRACE("decode_palette: %d items (id %d, version %d)\n",
-	(uint)entries, palette_id, palette_version_number);
+  XINE_HDMV_TRACE("decode_palette: %zd items (id %d, version %d)\n",
+                  entries, palette_id, palette_version_number);
 
   /* convert to xine-lib clut */
   subtitle_clut_t *clut = calloc(1, sizeof(subtitle_clut_t));
@@ -387,8 +461,8 @@ static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf)
   uint8_t  version   = segbuf_get_u8 (buf);
   uint8_t  seq_desc  = segbuf_get_u8 (buf);
 
-  TRACE("  decode_object: object_id %d, version %d, seq 0x%x\n",
-        object_id, version, seq_desc);
+  XINE_HDMV_TRACE("  decode_object: object_id %d, version %d, seq 0x%x\n",
+                  object_id, version, seq_desc);
 
   //LIST_FIND();
   subtitle_object_t *obj = calloc(1, sizeof(subtitle_object_t));
@@ -400,19 +474,19 @@ static subtitle_object_t *segbuf_decode_object(segment_buffer_t *buf)
     obj->width    = segbuf_get_u16(buf);
     obj->height   = segbuf_get_u16(buf);
 
-    TRACE("    object length %d bytes, size %dx%d\n", data_len, obj->width, obj->height);
+    XINE_HDMV_TRACE("    object length %d bytes, size %dx%d\n", data_len, obj->width, obj->height);
 
     segbuf_decode_rle (buf, obj);
 
     if (buf->error) {
-      free(obj);
+      free_subtitle_object(obj);
       return NULL;
     }
 
   } else {
-    ERROR("    TODO: APPEND RLE, length %d bytes\n", buf->segment_len - 4);
+    XINE_HDMV_ERROR("    TODO: APPEND RLE, length %d bytes\n", buf->segment_len - 4);
     /* TODO */
-    free(obj);
+    free_subtitle_object(obj);
     return NULL;
   }
 
@@ -430,8 +504,8 @@ static window_def_t *segbuf_decode_window_definition(segment_buffer_t *buf)
   wnd->width  = segbuf_get_u16 (buf);
   wnd->height = segbuf_get_u16 (buf);
 
-  TRACE("  window: [%02x %d]  %d,%d %dx%d\n", a,
-        wnd->id, wnd->xpos, wnd->ypos, wnd->width, wnd->height);
+  XINE_HDMV_TRACE("  window: [%02x %d]  %d,%d %dx%d\n", a,
+                  wnd->id, wnd->xpos, wnd->ypos, wnd->width, wnd->height);
 
   if (buf->error) {
     free(wnd);
@@ -447,22 +521,16 @@ static int segbuf_decode_video_descriptor(segment_buffer_t *buf)
   uint16_t height     = segbuf_get_u16(buf);
   uint8_t  frame_rate = segbuf_get_u8 (buf);
 
-  TRACE("  video_descriptor: %dx%d fps %d\n", width, height, frame_rate);
+  XINE_HDMV_TRACE("  video_descriptor: %dx%d fps %d\n", width, height, frame_rate);
   return buf->error;
 }
 
-typedef struct composition_descriptor_s composition_descriptor_t;
-struct composition_descriptor_s {
-  uint16_t number;
-  uint8_t  state;
-};
-
 static int segbuf_decode_composition_descriptor(segment_buffer_t *buf, composition_descriptor_t *descr)
 {
   descr->number = segbuf_get_u16(buf);
   descr->state  = segbuf_get_u8 (buf);
 
-  TRACE("  composition_descriptor: number %d, state %d\n", descr->number, descr->state);
+  XINE_HDMV_TRACE("  composition_descriptor: number %d, state %d\n", descr->number, descr->state);
   return buf->error;
 }
 
@@ -491,16 +559,45 @@ static composition_object_t *segbuf_decode_composition_object(segment_buffer_t *
     return NULL;
   }
 
-  TRACE("  composition_object: id: %d, win: %d, position %d,%d crop %d forced %d\n",
-        cobj->object_id_ref, cobj->window_id_ref, cobj->xpos, cobj->ypos,
-        cobj->cropped_flag, cobj->forced_flag);
+  XINE_HDMV_TRACE("  composition_object: id: %d, win: %d, position %d,%d crop %d forced %d\n",
+                  cobj->object_id_ref, cobj->window_id_ref, cobj->xpos, cobj->ypos,
+                  cobj->cropped_flag, cobj->forced_flag);
 
   return cobj;
 }
 
+static presentation_segment_t *segbuf_decode_presentation_segment(segment_buffer_t *buf)
+{
+  presentation_segment_t *seg = calloc(1, sizeof(presentation_segment_t));
+  int                     index;
+
+  segbuf_decode_video_descriptor (buf);
+  segbuf_decode_composition_descriptor (buf, &seg->comp_descr);
+
+  seg->palette_update_flag  = !!((segbuf_get_u8(buf)) & 0x80);
+  seg->palette_id_ref       = segbuf_get_u8 (buf);
+  seg->object_number        = segbuf_get_u8 (buf);
+
+  XINE_HDMV_TRACE("  presentation_segment: object_number %d, palette %d\n",
+                  seg->object_number, seg->palette_id_ref);
+
+  for (index = 0; index < seg->object_number; index++) {
+    composition_object_t *cobj = segbuf_decode_composition_object (buf);
+    cobj->next = seg->comp_objs;
+    seg->comp_objs = cobj;
+  }
+
+  if (buf->error) {
+    free_presentation_segment(seg);
+    return NULL;
+  }
+
+  return seg;
+}
+
 static rle_elem_t *copy_crop_rle(subtitle_object_t *obj, composition_object_t *cobj)
 {
-  /* TODO: exec cropping here (w,h sized image from pos x,y) */
+  /* TODO: cropping (w,h sized image from pos x,y) */
 
   rle_elem_t *rle = calloc (obj->num_rle, sizeof(rle_elem_t));
   memcpy (rle, obj->rle, obj->num_rle * sizeof(rle_elem_t));
@@ -524,57 +621,17 @@ typedef struct spuhdmv_decoder_s {
 
   segment_buffer_t *buf;
 
-  subtitle_clut_t      *cluts;
-  subtitle_object_t    *objects;
-  window_def_t         *windows;
+  subtitle_clut_t        *cluts;
+  subtitle_object_t      *objects;
+  window_def_t           *windows;
+  presentation_segment_t *segments;
+
   int overlay_handles[MAX_OBJECTS];
 
   int64_t               pts;
 
 } spuhdmv_decoder_t;
 
-#define LIST_REPLACE_OLD(type, list, obj) \
-  do { \
-    /* insert to list */ \
-    obj->next = list; \
-    list = obj; \
-\
-    /* remove old */ \
-    type *i = list; \
-    while (i->next && i->next->id != obj->id) \
-      i = i->next; \
-    if (i->next) { \
-      void *tmp = (void*)i->next; \
-      i->next = i->next->next; \
-      free(tmp); \
-    } \
-  } while (0);
-
-#define LIST_REPLACE(list, obj)			\
-  do {						\
-    uint id = obj->id;				\
-						\
-    /* insert to list */			\
-    obj->next = list;				\
-    list = obj;					\
-						\
-    /* remove old */				\
-    while (obj->next && obj->next->id != id)	\
-      obj = obj->next;				\
-    if (obj->next) {				\
-      void *tmp = (void*)obj->next;		\
-      obj->next = obj->next->next;		\
-      free(tmp);				\
-    }						\
-  } while (0);
-
-#define LIST_DESTROY(list)     \
-  while (list) {	       \
-    void *tmp = (void*)list;   \
-    list = list->next;	       \
-    free (tmp);		       \
-  }
-
 static int decode_palette(spuhdmv_decoder_t *this)
 {
   /* decode */
@@ -582,7 +639,7 @@ static int decode_palette(spuhdmv_decoder_t *this)
   if (!clut)
     return 1;
 
-  LIST_REPLACE (this->cluts, clut);
+  LIST_REPLACE (this->cluts, clut, free);
 
   return 0;
 }
@@ -594,7 +651,7 @@ static int decode_object(spuhdmv_decoder_t *this)
   if (!obj)
     return 1;
 
-  LIST_REPLACE (this->objects, obj);
+  LIST_REPLACE (this->objects, obj, free_subtitle_object);
 
   return 0;
 }
@@ -606,13 +663,30 @@ static int decode_window_definition(spuhdmv_decoder_t *this)
   if (!wnd)
     return 1;
 
-  LIST_REPLACE (this->windows, wnd);
+  LIST_REPLACE (this->windows, wnd, free);
 
   return 0;
 }
 
-static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, uint palette_id_ref,
-			int overlay_index, int64_t pts)
+static int decode_presentation_segment(spuhdmv_decoder_t *this)
+{
+  /* decode */
+  presentation_segment_t *seg = segbuf_decode_presentation_segment(this->buf);
+  if (!seg)
+    return 1;
+
+  seg->pts = this->pts;
+
+  /* replace */
+  if (this->segments)
+    LIST_DESTROY(this->segments, free_presentation_segment);
+  this->segments = seg;
+
+  return 0;
+}
+
+static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, unsigned int palette_id_ref,
+			int overlay_index, int64_t pts, int force_update)
 {
   video_overlay_manager_t *ovl_manager = this->stream->video_out->get_overlay_manager(this->stream->video_out);
   metronom_t              *metronom    = this->stream->metronom;
@@ -624,21 +698,16 @@ static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, uin
   while (clut && clut->id != palette_id_ref)
     clut = clut->next;
   if (!clut) {
-    ERROR("  fill_overlay: clut %d not found !\n", palette_id_ref);
+    XINE_HDMV_TRACE("  show_overlay: clut %d not found !\n", palette_id_ref);
     return -1;
   }
 
-  /* copy palette to xine overlay */
-  overlay.rgb_clut = 0;
-  memcpy(overlay.color, clut->color, sizeof(uint32_t) * 256);
-  memcpy(overlay.trans, clut->trans, sizeof(uint8_t)  * 256);
-
   /* find RLE image */
   subtitle_object_t *obj = this->objects;
   while (obj && obj->id != cobj->object_id_ref)
     obj = obj->next;
   if (!obj) {
-    ERROR("  fill_overlay: object %d not found !\n", cobj->object_id_ref);
+    XINE_HDMV_TRACE("  show_overlay: object %d not found !\n", cobj->object_id_ref);
     return -1;
   }
 
@@ -647,10 +716,20 @@ static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, uin
   while (wnd && wnd->id != cobj->window_id_ref)
     wnd = wnd->next;
   if (!wnd) {
-    ERROR("  fill_overlay: window %d not found !\n", cobj->window_id_ref);
+    XINE_HDMV_TRACE("  show_overlay: window %d not found !\n", cobj->window_id_ref);
     return -1;
   }
 
+  /* do not show again if all elements are unchanged */
+  if (!force_update && clut->shown && obj->shown && wnd->shown && cobj->shown)
+    return 0;
+  clut->shown = obj->shown = wnd->shown = cobj->shown = 1;
+
+  /* copy palette to xine overlay */
+  overlay.rgb_clut = 0;
+  memcpy(overlay.color, clut->color, sizeof(uint32_t) * 256);
+  memcpy(overlay.trans, clut->trans, sizeof(uint8_t)  * 256);
+
   /* copy and crop RLE image to xine overlay */
   overlay.width     = obj->width;
   overlay.height    = obj->height;
@@ -670,8 +749,8 @@ static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, uin
   overlay.hili_left   = -1;
   overlay.hili_right  = -1;
 
-  TRACE("    -> overlay: %d,%d %dx%d\n",
-        overlay.x, overlay.y, overlay.width, overlay.height);
+  XINE_HDMV_TRACE("    -> overlay: %d,%d %dx%d\n",
+                  overlay.x, overlay.y, overlay.width, overlay.height);
 
 
   /* set timings */
@@ -696,48 +775,16 @@ static int show_overlay(spuhdmv_decoder_t *this, composition_object_t *cobj, uin
 
   ovl_manager->add_event (ovl_manager, (void *)&event);
 
-  obj->rle = NULL;
-
   return 0;
 }
 
-typedef struct presentation_segment_s presentation_segment_t;
-struct presentation_segment_s {
-  composition_descriptor_t comp_descr;
-
-  uint8_t palette_update_flag;
-  uint8_t palette_id_ref;
-  uint8_t object_number;
-
-  composition_object_t *comp_objs;
-
-  presentation_segment_t *next;
-
-  int64_t pts;
-};
-
-static void show_overlays(spuhdmv_decoder_t *this, presentation_segment_t *pseg)
-{
-  composition_object_t *cobj = pseg->comp_objs;
-  int i;
-
-  for (i = 0; i < pseg->object_number; i++) {
-    if (!cobj) {
-      ERROR("show_overlays: composition object %d missing !\n", i);
-    } else {
-      show_overlay(this, cobj, pseg->palette_id_ref, i, pseg->pts);
-      cobj = cobj->next;
-    }
-  }
-}
-
 static void hide_overlays(spuhdmv_decoder_t *this, int64_t pts)
 {
   video_overlay_event_t event = {0};
   int i = 0;
 
   while (this->overlay_handles[i] >= 0) {
-    TRACE("    -> HIDE %d\n", i);
+    XINE_HDMV_TRACE("    -> HIDE %d\n", i);
 
     video_overlay_manager_t *ovl_manager = this->stream->video_out->get_overlay_manager(this->stream->video_out);
     metronom_t              *metronom    = this->stream->metronom;
@@ -756,87 +803,92 @@ static void hide_overlays(spuhdmv_decoder_t *this, int64_t pts)
   }
 }
 
-static int decode_presentation_segment(spuhdmv_decoder_t *this)
+static void update_overlays(spuhdmv_decoder_t *this)
 {
-  presentation_segment_t p     = {0};
-  segment_buffer_t      *buf   = this->buf;
-  int                    index;
+  presentation_segment_t *pseg = this->segments;
 
-  segbuf_decode_video_descriptor (this->buf);
-  segbuf_decode_composition_descriptor (this->buf, &p.comp_descr);
+  while (pseg) {
 
-  p.palette_update_flag  = !!((segbuf_get_u8(buf)) & 0x80);
-  p.palette_id_ref       = segbuf_get_u8 (buf);
-  p.object_number        = segbuf_get_u8 (buf);
+    if (!pseg->comp_descr.state) {
 
-  TRACE("  presentation_segment: object_number %d, palette %d\n",
-        p.object_number, p.palette_id_ref);
+      /* HIDE */
+      if (!pseg->shown)
+        hide_overlays (this, pseg->pts);
 
-  p.pts = this->pts; /* !! todo - use it ? */
+    } else {
 
-  for (index = 0; index < p.object_number; index++) {
-    composition_object_t *cobj = segbuf_decode_composition_object (this->buf);
-    cobj->next = p.comp_objs;
-    p.comp_objs = cobj;
+      /* SHOW */
+      composition_object_t *cobj = pseg->comp_objs;
+      int i;
+
+      for (i = 0; i < pseg->object_number; i++) {
+        if (!cobj) {
+          XINE_HDMV_ERROR("show_overlays: composition object %d missing !\n", i);
+        } else {
+          show_overlay(this, cobj, pseg->palette_id_ref, i, pseg->pts, !pseg->shown);
+          cobj = cobj->next;
+        }
+      }
+    }
+
+    pseg->shown = 1;
+
+    pseg = pseg->next;
   }
+}
 
-  if (!p.comp_descr.state) {
-    hide_overlays (this, this->pts);
-  } else {
-    show_overlays (this, &p);
-    LIST_DESTROY (p.comp_objs);
-  }
-
-  return buf->error;
+static void free_objs(spuhdmv_decoder_t *this)
+{
+  LIST_DESTROY (this->cluts,    free);
+  LIST_DESTROY (this->objects,  free_subtitle_object);
+  LIST_DESTROY (this->windows,  free);
+  LIST_DESTROY (this->segments, free_presentation_segment);
 }
 
 static void decode_segment(spuhdmv_decoder_t *this)
 {
-  TRACE("*** new segment, pts %010ld: 0x%02x (%8d bytes)",
-	 this->pts, (uint)this->buf->segment_type, (uint)this->buf->segment_len);
+  XINE_HDMV_TRACE("*** new segment, pts %010"PRId64": 0x%02x (%8d bytes)",
+                  this->pts, this->buf->segment_type, this->buf->segment_len);
 
-  switch (this->buf->segment_type) {
-  case 0x14:
-    TRACE("  segment: PALETTE\n");
+  switch (segbuf_segment_type(this->buf)) {
+  case SEGTYPE_PALETTE:
+    XINE_HDMV_TRACE("  segment: PALETTE\n");
     decode_palette(this);
     break;
-  case 0x15:
-    TRACE("  segment: OBJECT\n");
+  case SEGTYPE_OBJECT:
+    XINE_HDMV_TRACE("  segment: OBJECT\n");
     decode_object(this);
     break;
-  case 0x16:
-    TRACE("  segment: PRESENTATION SEGMENT\n");
+  case SEGTYPE_PRESENTATION_SEGMENT:
+    XINE_HDMV_TRACE("  segment: PRESENTATION SEGMENT\n");
     decode_presentation_segment(this);
     break;
-  case 0x17:
-    TRACE("  segment: WINDOW DEFINITION\n");
+  case SEGTYPE_WINDOW_DEFINITION:
+    XINE_HDMV_TRACE("  segment: WINDOW DEFINITION\n");
     decode_window_definition(this);
     break;
-  case 0x18:
-    TRACE("  segment: INTERACTIVE\n");
+  case SEGTYPE_INTERACTIVE:
+    XINE_HDMV_TRACE("  segment: INTERACTIVE\n");
     break;
-  case 0x80:
-    TRACE("  segment: END OF DISPLAY\n");
-    {
-      int64_t pts = xine_get_current_vpts(this->stream) -
-        this->stream->metronom->get_option(this->stream->metronom,
-                                           METRONOM_VPTS_OFFSET);
-      TRACE(" * current pts = %ld\n", pts);
-    }
-
+  case SEGTYPE_END_OF_DISPLAY:
+    XINE_HDMV_TRACE("  segment: END OF DISPLAY\n");
+    /* drop all cached objects */
+    free_objs(this);
     break;
   default:
-    ERROR("  segment type 0x%x unknown, skipping\n", this->buf->segment_type);
+    XINE_HDMV_ERROR("  segment type 0x%x unknown, skipping\n", segbuf_segment_type(this->buf));
     break;
   }
   if (this->buf->error) {
-    ERROR("*** DECODE ERROR ***\n");
+    XINE_HDMV_ERROR("*** DECODE ERROR ***\n");
   }
+
+  update_overlays (this);
 }
 
 static void close_osd(spuhdmv_decoder_t *this)
 {
-  video_overlay_manager_t  *ovl_manager = this->stream->video_out->get_overlay_manager (this->stream->video_out);
+  video_overlay_manager_t *ovl_manager = this->stream->video_out->get_overlay_manager (this->stream->video_out);
 
   int i = 0;
   while (this->overlay_handles[i] >= 0) {
@@ -881,9 +933,7 @@ static void spudec_reset (spu_decoder_t * this_gen)
   if (this->buf)
     segbuf_reset(this->buf);
 
-  LIST_DESTROY (this->cluts);
-  LIST_DESTROY (this->objects);
-  LIST_DESTROY (this->windows);
+  free_objs(this);
 
   close_osd(this);
 }
@@ -902,9 +952,7 @@ static void spudec_dispose (spu_decoder_t *this_gen)
   close_osd (this);
   segbuf_dispose (this->buf);
 
-  LIST_DESTROY (this->cluts);
-  LIST_DESTROY (this->objects);
-  LIST_DESTROY (this->windows);
+  free_objs(this);
 
   free (this);
 }
