@@ -38,6 +38,7 @@
 
 #include "ts2es.h"
 #include "demux_xvdr_tsdata.h"
+#include "xvdr_metronom.h"
 
 /*
  * features
@@ -145,22 +146,37 @@ static void demux_xvdr_parse_pes(demux_xvdr_t *this, buf_element_t *buf);
 static int32_t parse_video_stream(demux_xvdr_t *this, uint8_t *p, buf_element_t *buf);
 static int32_t parse_audio_stream(demux_xvdr_t *this, uint8_t *p, buf_element_t *buf);
 static int32_t parse_private_stream_1(demux_xvdr_t *this, uint8_t *p, buf_element_t *buf);
-static int32_t parse_padding_stream(demux_xvdr_t *this, uint8_t *p, buf_element_t *buf);
 
 static void pts_wrap_workaround(demux_xvdr_t *this, buf_element_t *buf, int video)
 {
+  /* PTS wrap workaround */
+  if (buf->pts > 0) {
+
 #ifdef LOG_PES_AV_DIFF
-  static int64_t vpts = 0, apts = 0;
-  if (video) vpts = buf->pts;
-  else apts = buf->pts;
-  if (vpts > 0 && apts > 0)
-    LOGMSG("pts diff [%d] %d", video, (int)(vpts - apts));
+    static int64_t vpts = 0, apts = 0;
+    if (video) vpts = buf->pts;
+    else apts = buf->pts;
+    if (vpts > 0 && apts > 0)
+      LOGMSG("pts diff [%d] %d", video, (int)(vpts - apts));
 #endif
 
-  /* PTS wrap workaround */
-  if (buf->pts >= 0) {
-    if (video)
+    if (video) {
+
+      /* VIDEO wrap in middle of GOP */
+      if (this->last_vpts < 14400 &&
+          this->last_vpts > 0     &&
+          buf->pts > (INT64_C(0x1ffffffff) - 14400) &&
+          !this->send_newpts) {
+        LOGMSG("VIDEO pts wrap in middle of GOP, ignoring video pts %" PRId64, buf->pts);
+        buf->pts = INT64_C(0);
+        return;
+      }
+
       this->last_vpts = buf->pts;
+      return;
+    }
+
+    /* VIDEO wrap before AUDIO wrap */
     else if (buf->pts        > INT64_C( 0x40400000 ) &&
              this->last_vpts < INT64_C( 0x40000000 ) &&
              this->last_vpts > INT64_C( 0 )) {
@@ -172,6 +188,18 @@ static void pts_wrap_workaround(demux_xvdr_t *this, buf_element_t *buf, int vide
 
 static void check_newpts(demux_xvdr_t *this, buf_element_t *buf, int video )
 {
+  if (buf->pts <= 0)
+    return;
+
+  if (video) {
+    int still_mode  = (int)this->stream->metronom->get_option(this->stream->metronom, XVDR_METRONOM_STILL_MODE);
+    int trick_speed = (int)this->stream->metronom->get_option(this->stream->metronom, XVDR_METRONOM_TRICK_SPEED);
+    if (still_mode > 0 || trick_speed > 0) {
+      LOGMSG("Skipping new pts %"PRId64" (still=%d trickspeed=%d)", buf->pts, still_mode, trick_speed);
+      return;
+    }
+  }
+
   pts_wrap_workaround(this, buf, video);
 
   if (buf->pts) {
@@ -179,6 +207,8 @@ static void check_newpts(demux_xvdr_t *this, buf_element_t *buf, int video )
 
     if (this->send_newpts || (this->last_pts[video] && abs(diff)>WRAP_THRESHOLD)) {
 
+      LOGVERBOSE("New PTS: %"PRId64" (%s)", buf->pts, video ? "VIDEO" : "AUDIO");
+
       if (this->buf_flag_seek) {
         _x_demux_control_newpts(this->stream, buf->pts, BUF_FLAG_SEEK);
         this->buf_flag_seek = 0;
@@ -218,7 +248,7 @@ static void post_sequence_end(fifo_buffer_t *fifo, uint32_t video_type)
     buf->content[0] = 0x00;
     buf->content[1] = 0x00;
     buf->content[2] = 0x01;
-    buf->content[3] = (video_type == BUF_VIDEO_H264) ? NAL_END_SEQ : 0xB7;
+    buf->content[3] = (video_type == BUF_VIDEO_H264) ? NAL_END_SEQ : SC_SEQUENCE_END;
     fifo->put(fifo, buf);
   }
 }
@@ -385,9 +415,8 @@ static void demux_xvdr_parse_pack (demux_xvdr_t *this)
 
   if (!buf) {
     if (errno == EINTR) {
-      /* very verbose logging ? */
-      if (iSysLogLevel >= SYSLOGLEVEL_VERBOSE)
-        LOGDBG("input->read_block() was interrupted");
+      LOGVERBOSE("input->read_block() was interrupted");
+      ts_data_flush(this->ts_data);
     } else if (errno != EAGAIN) {
       LOGDBG("DEMUX_FINISHED (input returns NULL with error)");
       this->status = DEMUX_FINISHED;
@@ -426,14 +455,22 @@ static void demux_xvdr_parse_pes (demux_xvdr_t *this, buf_element_t *buf)
   uint8_t       *p = buf->content;
   int32_t        result;
 
+  if (IS_PADDING_PACKET(p)) {
+    buf->free_buffer (buf);
+    return;
+  }
+
   this->stream_id  = p[3];
 
+  if (this->ts_data) {
+    ts_data_flush(this->ts_data);
+    ts_data_dispose(&this->ts_data);
+  }
+
   if (IS_VIDEO_PACKET(p)) {
     result = parse_video_stream(this, p, buf);
   } else if (IS_MPEG_AUDIO_PACKET(p)) {
     result = parse_audio_stream(this, p, buf);
-  } else if (IS_PADDING_PACKET(p)) {
-    result = parse_padding_stream(this, p, buf);
   } else if (IS_PS1_PACKET(p)) {
     result = parse_private_stream_1(this, p, buf);
   } else {
@@ -471,25 +508,29 @@ static void demux_xvdr_parse_ts (demux_xvdr_t *this, buf_element_t *buf)
     if (ts_pid == 0) {
       pat_data_t pat;
 
-      ts_data_flush(ts_data);
-
       if (ts_parse_pat(&pat, buf->content)) {
-        ts_data->pmt_pid        = pat.pmt_pid[0];
-        ts_data->program_number = pat.program_number[0];
-        if (iSysLogLevel >= SYSLOGLEVEL_VERBOSE)
-          LOGDBG("Got PAT, PMT pid = %d, program = %d", ts_data->pmt_pid, ts_data->program_number);
+        if (ts_data->pmt_pid        != pat.pmt_pid[0] ||
+            ts_data->program_number != pat.program_number[0]) {
+
+          LOGVERBOSE("PAT: program changed, flushing demuxer");
+
+          ts_data_flush(ts_data);
+
+          ts_data->pmt_pid        = pat.pmt_pid[0];
+          ts_data->program_number = pat.program_number[0];
+        } else {
+          LOGVERBOSE("Got PAT, PMT pid = %d, program = %d", ts_data->pmt_pid, ts_data->program_number);
+        }
       }
     }
 
     /* parse PMT */
     else if (ts_pid == ts_data->pmt_pid) {
 
-      ts_data_flush(ts_data);
-
       if (ts_parse_pmt(&ts_data->pmt, ts_data->program_number, buf->content)) {
 
         /* PMT changed, reset ts->es converters */
-        LOGMSG("PMT changed");
+        LOGMSG("PMT changed, resetting demuxer");
         ts_data_ts2es_init(&ts_data, this->stream->video_fifo, this->stream->audio_fifo);
 
         this->video_type = (ts_data->pmt.video_type == ISO_14496_PART10_VIDEO) ?
@@ -508,7 +549,6 @@ static void demux_xvdr_parse_ts (demux_xvdr_t *this, buf_element_t *buf)
       if (ts_data->video) {
         buf_element_t *vbuf     = ts2es_put(ts_data->video, buf->content, src_fifo);
         if (vbuf) {
-          this->pts = vbuf->pts;
           check_newpts( this, vbuf, PTS_VIDEO );
 
           this->stream->video_fifo->put(this->stream->video_fifo, vbuf);
@@ -524,7 +564,6 @@ static void demux_xvdr_parse_ts (demux_xvdr_t *this, buf_element_t *buf)
           if (ts_data->audio[i]) {
             buf_element_t *abuf = ts2es_put(ts_data->audio[i], buf->content, src_fifo);
             if (abuf) {
-              this->pts = abuf->pts;
               check_newpts( this, abuf, PTS_AUDIO );
               track_audio_stream_change (this, abuf);
 
@@ -560,12 +599,6 @@ static void demux_xvdr_parse_ts (demux_xvdr_t *this, buf_element_t *buf)
   buf->free_buffer(buf);
 }
 
-static int32_t parse_padding_stream(demux_xvdr_t *this, uint8_t *p, buf_element_t *buf)
-{
-  buf->free_buffer (buf);
-  return -1;
-}
-
 /* FIXME: Extension data is not parsed, and is also not skipped. */
 
 static int32_t parse_pes_for_pts(demux_xvdr_t *this, uint8_t *p, buf_element_t *buf)
@@ -943,9 +976,20 @@ static int32_t parse_video_stream(demux_xvdr_t *this, uint8_t *p, buf_element_t
   result = parse_pes_for_pts(this, p, buf);
   if (result < 0) return -1;
 
+  /* Handle marker packets after still images */
+  if (buf->size == 14 && result == 14 && this->pts == 0) {
+    /*if (this->stream->metronom->get_option(this->stream->metronom, XVDR_METRONOM_STILL_MODE))*/ {
+      LOGDBG("video fifo flush and decoder reset after still image");
+      put_control_buf(this->video_fifo, this->video_fifo, BUF_CONTROL_FLUSH_DECODER);
+      put_control_buf(this->video_fifo, this->video_fifo, BUF_CONTROL_RESET_DECODER);
+      buf->free_buffer(buf);
+      return -1;
+    }
+  }
+
   p += result;
 
-  if (this->video_type == 0) {
+  if (this->video_type == 0 && buf->size >= 4) {
     this->video_type = detect_h264(p);
   }
 
@@ -971,19 +1015,19 @@ static int32_t parse_video_stream(demux_xvdr_t *this, uint8_t *p, buf_element_t
   }
 
   /* H.264 */
-  else if (this->video_type == BUF_VIDEO_H264) {
+  else if (this->video_type == BUF_VIDEO_H264 &&
+           buf->size        >= 4) {
+
     /* Access Unit Delimiter */
     if (IS_NAL_AUD(p))
       post_frame_end (this, buf);
 
     /* Check for end of still image.
        VDR ensures that H.264 still images end with an end of sequence NAL unit */
-    if (buf->size > 4) {
-      uint8_t *end = buf->content + buf->size;
-      if (IS_NAL_END_SEQ(end-4)) {
-        LOGMSG("post_frame_h264: Still frame ? (frame ends with end of sequence NAL unit)");
-        buf->decoder_flags |= BUF_FLAG_FRAME_END;
-      }
+    uint8_t *end = buf->content + buf->size;
+    if (IS_NAL_END_SEQ(end-4)) {
+      LOGMSG("post_frame_h264: Still frame ? (frame ends with end of sequence NAL unit)");
+      buf->decoder_flags |= BUF_FLAG_FRAME_END;
     }
   }
 
@@ -1218,7 +1262,7 @@ void *demux_xvdr_init_class (xine_t *xine, void *data)
   this->demux_class.description     = N_("XVDR demux plugin");
   this->demux_class.identifier      = MRL_ID;
   this->demux_class.mimetypes       = NULL;
-  this->demux_class.extensions      = 
+  this->demux_class.extensions      =
     MRL_ID":/ "
     MRL_ID"+pipe:/ "
     MRL_ID"+tcp:/ "
