@@ -4,62 +4,44 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: xine_frontend.c,v 1.103 2009/10/08 19:52:20 phintuka Exp $
+ * $Id: xine_frontend.c,v 1.53.2.16 2010/03/31 20:50:01 phintuka Exp $
  *
  */
 
-#include "features.h"
-
 #include <inttypes.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <unistd.h>
 
-#ifdef HAVE_LIBJPEG
 # ifdef boolean
 #  define HAVE_BOOLEAN
 # endif
 # include <jpeglib.h>
 # undef boolean
-#endif
 
 #define XINE_ENGINE_INTERNAL
 #include <xine.h>
 #include <xine/xine_internal.h>
 
+#ifndef XINE_VERSION_CODE
+#  define XINE_VERSION_CODE (XINE_MAJOR_VERSION*10000 + \
+                             XINE_MINOR_VERSION*100 + \
+                             XINE_SUB_VERSION)
+#endif
+
 #define LOG_MODULENAME "[vdr-fe]    "
 #include "logdefs.h"
 
-#include "xine_frontend_internal.h"
+#include "xine_input_vdr.h"
+#include "xine_input_vdr_mrl.h"
 #include "xine/post.h"
 
-#include "xine/vo_post.h"
-#include "xine/vo_osdscaler.h"
-#include "xine/vo_osdreorder.h"
-
 #undef  MIN
 #define MIN(a,b) ( (a) < (b) ? (a) : (b))
 #undef  MAX
 #define MAX(a,b) ( (a) > (b) ? (a) : (b))
 
-
-static void intercept_video_driver(xine_video_port_t *video_port)
-{
-  vo_driver_t *osdscaler = osdscaler_init();
-  if (! wire_video_driver(video_port, osdscaler)) {
-    LOGMSG("wire_video_driver() for osdscaler failed");
-    osdscaler->dispose(osdscaler);
-  }
-
-  vo_driver_t *osdreorder = osd_reorder_init();
-  if (! wire_video_driver(video_port, osdreorder)) {
-    LOGMSG("wire_video_driver() for osdreorder failed");
-    osdreorder->dispose(osdreorder);
-  }
-}
-
-
-char *strn0cpy(char *dest, const char *src, int n) 
+static inline char *strn0cpy(char *dest, const char *src, int n) 
 {
   char *s = dest;
   for ( ; --n && (*dest = *src) != 0; dest++, src++) ;
@@ -172,9 +154,10 @@ static void *fe_control(frontend_t *this_gen, const char *cmd);
  * xine callbacks
  */
 
-static double fe_dest_pixel_aspect(const fe_t *this, double video_pixel_aspect,
+static double fe_dest_pixel_aspect(const frontend_t *this_gen, double video_pixel_aspect,
 				   int video_width, int video_height)
 {
+  const fe_t *this = (const fe_t*)this_gen;
   /*int new_cropping = 0;*/
   double result = 1.0;
 
@@ -292,7 +275,7 @@ static void fe_frame_output_cb (void *data,
   *win_x = this->xpos;
   *win_y = this->ypos;  
   
-  *dest_pixel_aspect = this->dest_pixel_aspect(this, video_pixel_aspect,
+  *dest_pixel_aspect = this->dest_pixel_aspect((frontend_t*)this, video_pixel_aspect,
 					       video_width, video_height);
 
 #if 0
@@ -338,11 +321,10 @@ static void fe_frame_output_cb (void *data,
     xine_event_send(this->stream, &event);
     this->video_width = video_width;
     this->video_height = video_height;
-#if 0
+
     /* trigger forced redraw to make cropping changes effective */
     if(this->cropping)
       xine_set_param(this->stream, XINE_PARAM_VO_ZOOM_X, 100);      
-#endif
   }
 
   if(this->aspect_controller) {
@@ -580,7 +562,7 @@ static int fe_xine_init(frontend_t *this_gen, const char *audio_driver,
   if (config_file)
     this->configfile = strdup(config_file);
   else if(asprintf(&this->configfile, "%s%s", xine_get_homedir(), 
-	      "/.xine/config_xineliboutput") < 0)
+                   "/.xine/config_xineliboutput") < 0)
     return 0;
 
   xine_config_load (this->xine, this->configfile);
@@ -643,13 +625,11 @@ static int fe_xine_init(frontend_t *this_gen, const char *audio_driver,
     return 0;
   }
 
-  intercept_video_driver(this->video_port);
-
   this->video_port_none = NULL;
   
   /* re-configure display size (DirectFB driver changes display mode in init) */
   if(this->update_display_size_cb)
-    this->update_display_size_cb(this);
+    this->update_display_size_cb(this_gen);
 
   /* create audio port */
 
@@ -1211,16 +1191,18 @@ static int fe_is_finished(frontend_t *this_gen, int slave_stream)
 {
   fe_t *this = (fe_t*)this_gen;
 
-  if(!this || this->playback_finished)
+  if (!this)
     return FE_XINE_ERROR;
-  if(this->terminate_key_pressed)
+  if (this->terminate_key_pressed)
     return FE_XINE_EXIT;
-  
-  if(slave_stream) {
-    if(!this->slave_stream || this->slave_playback_finished)
+  if (this->playback_finished)
+    return FE_XINE_ERROR;
+
+  if (slave_stream) {
+    if (!this->slave_stream || this->slave_playback_finished)
       return FE_XINE_EXIT;
   }
-  
+
   return FE_XINE_RUNNING;
 }
 
@@ -1317,7 +1299,7 @@ static int fe_send_event(frontend_t *this_gen, const char *data)
 
   if (!strcmp(data, "TOGGLE_FULLSCREEN")) {
     if(this->toggle_fullscreen_cb)
-      this->toggle_fullscreen_cb(this);
+      this->toggle_fullscreen_cb(this_gen);
 
   } else if (!strcmp(data, "QUIT")) {
     this->terminate_key_pressed = 1;
@@ -1439,7 +1421,7 @@ static void *fe_control(frontend_t *this_gen, const char *cmd)
 					    this->video_port);
       LOGMSG("  PIP %d: %dx%d @ (%d,%d)", pid & 0x0f, w, h, x, y);
       LOGMSG("create pip stream done");
-      sprintf(mrl, MRL_ID "+slave://0x%lx#nocache",
+      sprintf(mrl, MRL_ID "+slave://0x%lx#nocache;demux:mpeg_block",
 	      (unsigned long int)this);
       if(!xine_open(posts->pip_stream, mrl) ||
 	 !xine_play(posts->pip_stream, 0, 0)) {
@@ -1538,8 +1520,6 @@ static void *fe_control(frontend_t *this_gen, const char *cmd)
  *        - move to xine_input_vdr ?
  */
 
-#ifdef HAVE_LIBJPEG
-
 #define JPEGCOMPRESSMEM 500000
 
 typedef struct tJpegCompressData_s {
@@ -1654,6 +1634,9 @@ static char *frame_compress_jpeg(fe_t *this, int *size, int quality, vo_frame_t
       jpeg_set_defaults(&cinfo);
       jpeg_set_quality(&cinfo, quality, TRUE);
       cinfo.raw_data_in = TRUE;
+#if JPEG_LIB_VERSION >= 70
+      cinfo.do_fancy_downsampling = FALSE;
+#endif
 
       jpeg_set_colorspace(&cinfo, JCS_YCbCr);
       cinfo.comp_info[0].h_samp_factor =
@@ -1713,8 +1696,6 @@ static char *frame_compress_jpeg(fe_t *this, int *size, int quality, vo_frame_t
   return (char*) jcd.mem;
 }
 
-#endif /* HAVE_LIBJPEG */
-
 static vo_frame_t *yv12_to_yuy2_frame(xine_stream_t *stream, vo_frame_t *frame)
 {
   /* convert yv12 frames to yuy2 */
@@ -1806,7 +1787,6 @@ static char *fe_grab_raw_frame(fe_t *this, int *size, int jpeg, int quality, int
   if (xine_port_send_gui_data(this->stream->video_out, XINE_GUI_SEND_GRAB_FRAME, frame))
     return NULL;
 
-#ifdef HAVE_LIBJPEG
   if (jpeg) {
     /* Compress JPEG */
     struct jpeg_destination_mgr jdm;
@@ -1843,7 +1823,6 @@ static char *fe_grab_raw_frame(fe_t *this, int *size, int jpeg, int quality, int
     *size = jcd.size;
     return (char*) jcd.mem;
   }
-#endif
 
   /* convert to PNM */
 
@@ -1873,13 +1852,6 @@ static char *fe_grab(frontend_t *this_gen, int *size, int jpeg,
   fe_t *this = (fe_t*)this_gen;
   vo_frame_t *frame;
 
-#ifndef HAVE_LIBJPEG
-  if(jpeg) {
-    LOGMSG("fe_grab: JPEG grab support was not compiled in");
-    return 0;
-  }
-#endif /* HAVE_LIBJPEG */
-
   if(!find_input_plugin(this))
     return 0;
 
@@ -1889,8 +1861,8 @@ static char *fe_grab(frontend_t *this_gen, int *size, int jpeg,
   /* validate parameters */
   if ((quality < 0) || (quality > 100))
     quality = 100;
-  width  = (MAX(16, MIN(width, 1920)) + 1) & ~1; /* 16...1920, even */
-  height = (MAX(16, MIN(width, 1200)) + 1) & ~1; /* 16...1200, even */
+  width  = (MAX(16, MIN(width,  1920)) + 1) & ~1; /* 16...1920, even */
+  height = (MAX(16, MIN(height, 1200)) + 1) & ~1; /* 16...1200, even */
 
   /* get last frame */
   this->stream->xine->port_ticket->acquire(this->stream->xine->port_ticket, 0);
@@ -1922,11 +1894,7 @@ static char *fe_grab(frontend_t *this_gen, int *size, int jpeg,
   if (!jpeg)
     return frame_compress_pnm(this, size, frame);
 
-#ifdef HAVE_LIBJPEG
   return frame_compress_jpeg(this, size, quality, frame);
-#else /* HAVE_LIBJPEG */
-  return NULL;
-#endif
 }
 
 /*
@@ -1960,3 +1928,10 @@ void init_fe(fe_t *fe)
   fe->frame_output_handler = fe_frame_output_cb;
 }
 
+#ifdef FE_STANDALONE
+
+/* frontend main() */
+
+#include "xine_frontend_main.c"
+
+#endif /* #ifdef FE_STANDALONE */
