@@ -4,12 +4,10 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: xine_frontend_main.c,v 1.80 2009/08/18 12:23:09 phintuka Exp $
+ * $Id: xine_frontend_main.c,v 1.40.2.17 2009/08/21 11:50:41 phintuka Exp $
  *
  */
 
-#include "features.h"
-
 #include <stdio.h>
 #include <inttypes.h>
 #include <stdlib.h>
@@ -25,7 +23,6 @@
 
 #include <xine.h>  /* xine_get_version */
 
-#define LOG_MODULENAME "[vdr-fe]    "
 #include "logdefs.h"
 
 #include "xine_input_vdr_mrl.h"
@@ -49,6 +46,9 @@ int            gui_hotkeys = 0;
 pthread_t      kbd_thread;
 struct termios tm, saved_tm;
 
+/* include LIRC forwarding code */
+#include "xine_frontend_lirc.c"
+
 /*
  * read_key()
  *
@@ -805,6 +805,8 @@ int main(int argc, char *argv[])
 
     /* Main loop */
 
+    sleep(2);  /* give input_vdr some time to establish connection */
+
     fflush(stdout);
     fflush(stderr);
 
