@@ -4,13 +4,25 @@
  * See the main source file 'xineliboutput.c' for copyright information and
  * how to reach the author.
  *
- * $Id: xine_sxfe_frontend.c,v 1.114 2009/08/19 16:58:58 phintuka Exp $
+ * $Id: xine_sxfe_frontend.c,v 1.41.2.24 2009/10/08 12:13:11 phintuka Exp $
  *
  */
 
 /*#define HAVE_XF86VIDMODE*/
-#include "features.h"
 
+#include <errno.h>
+#include <inttypes.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <dlfcn.h>
+#include <stdlib.h>
+#include <string.h>
+#include <stdio.h>
+#include <sys/types.h>
+#include <unistd.h>
+#include <time.h>
+#include <pthread.h>
+#include <sched.h>
 #include <poll.h>
 #include <sys/ipc.h>
 #include <sys/shm.h>
@@ -21,9 +33,7 @@
 #include <X11/Xatom.h>
 #include <X11/keysym.h>
 #include <X11/Xutil.h>
-#ifdef HAVE_XSHM
-# include <X11/extensions/XShm.h>
-#endif
+#include <X11/extensions/XShm.h>
 #ifdef HAVE_XRENDER
 #  include <X11/extensions/Xrender.h>
 #endif
@@ -37,13 +47,32 @@
 #  include <X11/extensions/Xinerama.h>
 #endif
 
+#ifdef boolean
+#  define HAVE_BOOLEAN
+#endif
+#include <jpeglib.h>
+#undef boolean
+
+/* framegrab ports */
+#define XINE_ENABLE_EXPERIMENTAL_FEATURES
+
 #include <xine.h>
+#ifndef XINE_ENGINE_INTERNAL
+#  define XINE_ENGINE_INTERNAL
+#  include <xine/xine_internal.h>
+#  undef XINE_ENGINE_INTERNAL
+#else
+#  include <xine/xine_internal.h>
+#endif
+#include <xine/xineutils.h>
+#include <xine/input_plugin.h>
+#include <xine/plugin_catalog.h>
 
-#define LOG_MODULENAME "[vdr-sxfe]  "
-#include "logdefs.h"
-
+#include "xine_input_vdr.h"
 #include "xine_osd_command.h"
-#include "xine_frontend_internal.h"
+
+#include "xine_frontend.h"
+#include "xine/post.h"
 
 #ifdef HAVE_DBUS_GLIB_1
 #  include "tools/gnome_screensaver.h"
@@ -88,9 +117,26 @@ typedef struct sxfe_s {
   /* function pointers / base class */
   union {
     frontend_t fe;  /* generic frontend */
-    fe_t       x;   /* xine frontend */
   };
 
+  void   (*update_display_size_cb)(frontend_t*);
+  void   (*toggle_fullscreen_cb)  (frontend_t*);
+
+  /* from xine_frontend.c */
+  double (*dest_pixel_aspect)   (const frontend_t *,
+                                 double video_pixel_aspect,
+                                 int video_width, int video_height);
+  void   (*frame_output_handler)(void *data,
+                                 int video_width, int video_height,
+                                 double video_pixel_aspect,
+                                 int *dest_x, int *dest_y,
+                                 int *dest_width, int *dest_height,
+                                 double *dest_pixel_aspect,
+                                 int *win_x, int *win_y);
+
+  /* vdr */
+  fe_keypress_f        keypress;
+
   /* stored original handlers */
   int (*fe_xine_open)(frontend_t *this_gen, const char *mrl);
   int (*fe_xine_play)(frontend_t *this_gen);
@@ -147,6 +193,44 @@ typedef struct sxfe_s {
   uint8_t  gui_hotkeys : 1;
   uint8_t  no_x_kbd : 1;
 
+  /* xine stuff */
+  xine_t              *xine;
+  xine_stream_t       *stream;
+  xine_stream_t       *slave_stream;
+  vdr_input_plugin_if_t *input_plugin;
+  xine_video_port_t   *video_port;
+  xine_video_port_t   *video_port_none;
+  xine_audio_port_t   *audio_port;
+  xine_audio_port_t   *audio_port_none;
+  xine_event_queue_t  *event_queue;
+
+  post_plugins_t      *postplugins;
+
+  x11_visual_t         vis;
+  int                  xine_visual_type;
+
+  uint16_t             pes_buffers;
+
+  /* frontend */
+  double    display_ratio;
+  double    video_aspect;
+  char     *aspect_controller;
+  char     *video_port_name;
+  int       xpos, ypos;
+  uint16_t  video_width, video_height;
+  uint16_t  width, height;
+  uint8_t   aspect;
+  uint8_t   cropping;
+  uint8_t   scale_video;
+  uint8_t   overscan;
+  uint8_t   terminate_key_pressed;
+  uint8_t   playback_finished;
+  uint8_t   slave_playback_finished;
+  uint8_t   field_order;
+
+  /* strings */
+  char    *configfile;
+
   /* HUD stuff */
 #ifdef HAVE_XRENDER
   XImage         *hud_img;
@@ -156,17 +240,18 @@ typedef struct sxfe_s {
   uint32_t       *hud_img_mem;
   GC              gc;
   Window          hud_window;
-# ifdef HAVE_XSHM
   XShmSegmentInfo hud_shminfo;
-# endif
 
   uint16_t      osd_width;
   uint16_t      osd_height;
 #endif /* HAVE_XRENDER */
 
-} sxfe_t;
+} fe_t, sxfe_t;
 
 
+/* Common (non-X11/FB) frontend functions */
+#include "xine_frontend.c"
+
 #define DOUBLECLICK_TIME   500  // ms
 
 #define OSD_DEF_WIDTH      720
@@ -186,7 +271,7 @@ static void sxfe_dest_size_cb (void *data,
   *dest_width  = this->width;
   *dest_height = this->height;
 
-  *dest_pixel_aspect = this->dest_pixel_aspect(this, video_pixel_aspect,
+  *dest_pixel_aspect = this->dest_pixel_aspect((frontend_t*)this, video_pixel_aspect,
                                                video_width, video_height);
 }
 
@@ -215,8 +300,8 @@ static void set_fs_size_hint(sxfe_t *this)
   hint.flags  = USSize | USPosition | PPosition | PSize;
   hint.x      = this->xinerama_x;
   hint.y      = this->xinerama_y;
-  hint.width  = this->x.width;
-  hint.height = this->x.height;
+  hint.width  = this->width;
+  hint.height = this->height;
   XLockDisplay(this->display);
   XSetNormalHints(this->display, this->window[1], &hint);
   XUnlockDisplay(this->display);
@@ -265,7 +350,6 @@ static void set_fullscreen_props(sxfe_t *this)
   ev.xclient.window       = this->window[1];
   ev.xclient.format       = 32;
   ev.xclient.data.l[0]    = 1;
-  /*ev.xclient.data.l[0]    = this->atom_state_add;*/
 
   /* _NET_WM_STATE_FULLSCREEN */
   XLockDisplay(this->display);
@@ -302,7 +386,7 @@ static void update_window_title(sxfe_t *this)
 {
   XLockDisplay(this->display);
 
-  if (!this->x.keypress) { /* handler is set only in local mode */
+  if (!this->keypress) { /* handler is set only in local mode */
     char *name = NULL;
     if (XFetchName(this->display, this->window[0], &name) && name) {
       char *newname = NULL;
@@ -323,6 +407,7 @@ static void update_window_title(sxfe_t *this)
     XStoreName(this->display, this->window[0], this->stay_above ? "Local VDR (top)" : "Local VDR");
     XStoreName(this->display, this->window[1], this->stay_above ? "Local VDR (top)" : "Local VDR");
   }
+
   XUnlockDisplay(this->display);
 }
 
@@ -347,7 +432,7 @@ static void set_above(sxfe_t *this, int stay_above)
   ev.xclient.display      = this->display;
   ev.xclient.window       = this->window[0];
   ev.xclient.format       = 32;
-  ev.xclient.data.l[0]    = stay_above ? 1:0; /*this->atom_state_add : this->atom_state_del;*/
+  ev.xclient.data.l[0]    = stay_above ? 1:0;
 
   /* _NET_WM_STATE_ABOVE */
   XLockDisplay(this->display);
@@ -477,8 +562,8 @@ static void update_xinerama_info(sxfe_t *this)
       screen = 0;
     this->xinerama_x = screens[screen].x_org;
     this->xinerama_y = screens[screen].y_org;
-    this->x.width    = screens[screen].width;
-    this->x.height   = screens[screen].height;
+    this->width      = screens[screen].width;
+    this->height     = screens[screen].height;
 
     XFree(screens);
   }
@@ -493,8 +578,8 @@ static void update_xinerama_info(sxfe_t *this)
  */
 static void update_screen_size(sxfe_t *this)
 {
-  this->x.width = DisplayWidth(this->display, this->screen);
-  this->x.height = DisplayHeight(this->display, this->screen);
+  this->width = DisplayWidth(this->display, this->screen);
+  this->height = DisplayHeight(this->display, this->screen);
   update_xinerama_info(this);
 }
 
@@ -542,8 +627,8 @@ static void xrender_surf_blend(Display *dpy, Xrender_Surf *src, Xrender_Surf *ds
   xf.matrix[2][0] = 0; xf.matrix[2][1] = 0; xf.matrix[2][2] = XDoubleToFixed(1.0);
   XRenderSetPictureFilter(dpy, src->pic, smooth ? "bilinear" : "nearest", NULL, 0);
   XRenderSetPictureTransform(dpy, src->pic, &xf);
-  x = (int)ceil((double)(x>0?x-1:0) * scale_x);
-  y = (int)ceil((double)(y>0?y-1:0) * scale_y);
+  x = (int)ceil((double)(x?x-1:0) * scale_x);
+  y = (int)ceil((double)(y?y-1:0) * scale_y);
   w = (int)floor((double)(w+2) * scale_x);
   h = (int)floor((double)(h+2) * scale_y);
   XRenderComposite(dpy, PictOpSrc, src->pic, None, dst->pic, x, y, 0, 0, x, y, w, h);
@@ -667,7 +752,6 @@ static int hud_osd_command(frontend_t *this_gen, struct osd_command_s *cmd)
       LOGDBG("HUD Set RLE");
       if (!(cmd->flags & OSDFLAG_TOP_LAYER))
         break;
-#ifdef HAVE_XSHM
       if(this->completion_event != -1) {
         hud_fill_img_memory((uint32_t*)(this->hud_img->data), cmd);
         if(!cmd->scaling) {
@@ -690,13 +774,12 @@ static int hud_osd_command(frontend_t *this_gen, struct osd_command_s *cmd)
                              cmd->x + cmd->dirty_area.x1, cmd->y + cmd->dirty_area.y1,
                              cmd->dirty_area.x2 - cmd->dirty_area.x1 + 1,
                              cmd->dirty_area.y2 - cmd->dirty_area.y1 + 1,
-                             (XDouble)this->x.width / (XDouble)this->osd_width,
-                             (XDouble)this->x.height / (XDouble)this->osd_height,
+                             (XDouble)this->width / (XDouble)this->osd_width,
+                             (XDouble)this->height / (XDouble)this->osd_height,
                              (cmd->scaling & 2)); // Note: HUD_SCALING_BILINEAR=2
         }
       }
       else
-#endif
       {
         hud_fill_img_memory(this->hud_img_mem, cmd);
         if(!cmd->scaling) {
@@ -717,8 +800,8 @@ static int hud_osd_command(frontend_t *this_gen, struct osd_command_s *cmd)
                              cmd->x + cmd->dirty_area.x1, cmd->y + cmd->dirty_area.y1,
                              cmd->dirty_area.x2 - cmd->dirty_area.x1 + 1,
                              cmd->dirty_area.y2 - cmd->dirty_area.y1 + 1,
-                             (XDouble)this->x.width / (XDouble)this->osd_width,
-                             (XDouble)this->x.height / (XDouble)this->osd_height,
+                             (XDouble)this->width / (XDouble)this->osd_width,
+                             (XDouble)this->height / (XDouble)this->osd_height,
                              (cmd->scaling & 2)); // Note: HUD_SCALING_BILINEAR=2
         }
       }
@@ -742,7 +825,7 @@ static int hud_osd_command(frontend_t *this_gen, struct osd_command_s *cmd)
         break;
       XSetForeground(this->display, this->gc, 0x00000000);
       XFillRectangle(this->display, this->hud_window, this->gc,
-                     0, 0, this->x.width, this->x.height);
+                     0, 0, this->width, this->height);
       XFillRectangle(this->display, this->surf_img->draw, this->gc,
                      0, 0, this->osd_width+2, this->osd_height+2);
       XFlush(this->display);
@@ -794,8 +877,8 @@ static int hud_osd_open(sxfe_t *this)
     attributes.backing_store = Always;
 
     this->hud_window = XCreateWindow(this->display, DefaultRootWindow(this->display),
-                                     this->x.xpos, this->x.ypos,
-                                     this->x.width, this->x.height,
+                                     this->xpos, this->ypos,
+                                     this->width, this->height,
                                      0, 32, InputOutput, this->hud_vis,
                                      CWBackPixel | CWBorderPixel |
                                      CWOverrideRedirect | CWColormap,
@@ -811,7 +894,6 @@ static int hud_osd_open(sxfe_t *this)
     XStoreName(this->display, this->hud_window, "HUD");
     this->gc = XCreateGC(this->display, this->hud_window, 0, NULL);
 
-#ifdef HAVE_XSHM
     if(this->completion_event != -1) {
       this->hud_img = XShmCreateImage(this->display, this->hud_vis, 32, ZPixmap, NULL, &(this->hud_shminfo),
                                       HUD_MAX_WIDTH, HUD_MAX_HEIGHT);
@@ -823,10 +905,7 @@ static int hud_osd_open(sxfe_t *this)
       this->hud_shminfo.readOnly = True;
 
       XShmAttach(this->display, &(this->hud_shminfo));
-    }
-    else
-#endif
-    {
+    } else {
       /* Fall-back to traditional memory */
       LOGMSG("hud_osd_open: XShm not available, falling back to normal (slow) memory");
       this->hud_img_mem = malloc(4 * HUD_MAX_WIDTH * HUD_MAX_HEIGHT);
@@ -903,7 +982,6 @@ static void hud_osd_close(sxfe_t *this)
     XLockDisplay(this->display);
     LOGDBG("closing hud window...");
 
-#ifdef HAVE_XSHM
     if(this->completion_event != -1) {
       XShmDetach(this->display, &(this->hud_shminfo));
       XDestroyImage(this->hud_img);
@@ -911,7 +989,6 @@ static void hud_osd_close(sxfe_t *this)
       shmctl(this->hud_shminfo.shmid, IPC_RMID, 0);
     }
     else
-#endif
       XDestroyImage(this->hud_img);
 
     if(this->surf_img)
@@ -1020,7 +1097,7 @@ static void set_icon(sxfe_t *this)
   XUnlockDisplay(this->display);
 }
 
-/* 
+/*
  * detect_display_ratio
  *
  * Calculate display aspect ratio
@@ -1062,12 +1139,12 @@ static void create_windows(sxfe_t *this)
   /* create and display our video window */
   this->window[0] = XCreateSimpleWindow (this->display,
                                          DefaultRootWindow(this->display),
-                                         this->x.xpos, this->x.ypos,
-                                         this->x.width, this->x.height,
+                                         this->xpos, this->ypos,
+                                         this->width, this->height,
                                          1, 0, 0);
   this->window[1] = XCreateSimpleWindow(this->display, XDefaultRootWindow(this->display),
                                         this->xinerama_x, this->xinerama_y,
-                                        this->x.width, this->x.height,
+                                        this->width, this->height,
                                         0, 0, 0);
 
   /* full-screen window */
@@ -1084,7 +1161,7 @@ static void create_windows(sxfe_t *this)
   }
 
   /* Window name */
-  const char *initial_title = (!this->x.keypress) ? "Connecting to VDR ..." : "Local VDR";
+  const char *initial_title = (!this->keypress) ? "Connecting to VDR ..." : "Local VDR";
   XStoreName(this->display, this->window[0], initial_title);
   XStoreName(this->display, this->window[1], initial_title);
 
@@ -1112,8 +1189,8 @@ static int sxfe_display_open(frontend_t *this_gen,
     this->fe.fe_display_close(this_gen);
 
   if(keyfunc) {
-    this->x.keypress = keyfunc;
-    this->x.keypress("XKeySym", ""); /* triggers learning mode */
+    this->keypress = keyfunc;
+    this->keypress("XKeySym", ""); /* triggers learning mode */
   }
 
   LOGDBG("sxfe_display_open(width=%d, height=%d, fullscreen=%d, display=%s)",
@@ -1130,19 +1207,19 @@ static int sxfe_display_open(frontend_t *this_gen,
 #endif
   }
 
-  this->x.xpos        = xpos;
-  this->x.ypos        = ypos;
-  this->x.width       = width;
-  this->x.height      = height;
-  this->x.aspect      = aspect;
-/*this->x.cropping    = 0;*/
-  this->x.overscan    = 0;
-  this->x.scale_video = scale_video;
-  this->x.field_order = field_order ? 1 : 0;
-  this->x.aspect_controller = aspect_controller ? strdup(aspect_controller) : NULL;
+  this->xpos            = xpos;
+  this->ypos            = ypos;
+  this->width           = width;
+  this->height          = height;
+  this->aspect          = aspect;
+  this->cropping        = 0;
+  this->overscan        = 0;
+  this->scale_video     = scale_video;
+  this->field_order     = field_order ? 1 : 0;
+  this->aspect_controller = aspect_controller ? strdup(aspect_controller) : NULL;
 
-  this->origxpos      = 0;
-  this->origypos      = 0;
+  this->origxpos      = xpos;
+  this->origypos      = ypos;
   this->origwidth     = width>0 ? width : OSD_DEF_WIDTH;
   this->origheight    = height>0 ? height : OSD_DEF_HEIGHT;
 
@@ -1154,7 +1231,7 @@ static int sxfe_display_open(frontend_t *this_gen,
   this->fullscreen      = fullscreen;
 /*this->vmode_switch    = modeswitch;*/
   this->fullscreen_state_forced = 0;
-/*this->modeline = strdup(modeline ?: "");*/
+/*strn0cpy(this->modeline, modeline ? : "", sizeof(this->modeline));*/
   this->window_id = window_id;
 
   this->xinerama_screen = -1;
@@ -1183,11 +1260,9 @@ static int sxfe_display_open(frontend_t *this_gen,
 
   /* completion event */
   this->completion_event = -1;
-#ifdef HAVE_XSHM
   if (XShmQueryExtension (this->display) == True) {
     this->completion_event = XShmGetEventBase (this->display) + ShmCompletion;
   }
-#endif
 
   init_atoms(this);
 
@@ -1196,7 +1271,7 @@ static int sxfe_display_open(frontend_t *this_gen,
 
   /* Output to existing window ? (embedded to another app) */
   if(this->window_id > 0) {
-    LOGMSG("sxfe_display_open(): Using X11 window %d for output", this->window_id);
+    LOGMSG("sxfe_diaplay_open(): Using X11 window %d for output", this->window_id);
     this->window[0] = this->window[1] = (Window)this->window_id;
     XUnmapWindow(this->display, this->window[0]);
   } else {
@@ -1219,10 +1294,10 @@ static int sxfe_display_open(frontend_t *this_gen,
 
   /* Map current window */
   XMapRaised (this->display, this->window[this->fullscreen ? 1 : 0]);
-  XMoveWindow(this->display, this->window[0], this->x.xpos, this->x.ypos);
+  XMoveWindow(this->display, this->window[0], this->xpos, this->ypos);
 
   /* determine display aspect ratio */
-  this->x.display_ratio = detect_display_ratio(this->display, this->screen);
+  this->display_ratio = detect_display_ratio(this->display, this->screen);
 
   /* we want to get notified if user closes the window */
   XSetWMProtocols(this->display, this->window[this->fullscreen ? 1 : 0], &(this->xa_WM_DELETE_WINDOW), 1);
@@ -1243,14 +1318,13 @@ static int sxfe_display_open(frontend_t *this_gen,
   gnome_screensaver_control(0);
 #endif
 
-  /* setup xine visual type */
-  this->x.xine_visual_type         = XINE_VISUAL_TYPE_X11;
-  this->x.vis_x11.display          = this->display;
-  this->x.vis_x11.screen           = this->screen;
-  this->x.vis_x11.d                = this->window[this->fullscreen ? 1 : 0];
-  this->x.vis_x11.dest_size_cb     = sxfe_dest_size_cb;
-  this->x.vis_x11.frame_output_cb  = this->x.frame_output_handler;
-  this->x.vis_x11.user_data        = this;
+  this->xine_visual_type     = XINE_VISUAL_TYPE_X11;
+  this->vis.display          = this->display;
+  this->vis.screen           = this->screen;
+  this->vis.d                = this->window[this->fullscreen ? 1 : 0];
+  this->vis.dest_size_cb     = sxfe_dest_size_cb;
+  this->vis.frame_output_cb  = fe_frame_output_cb;
+  this->vis.user_data        = this;
 
   set_fullscreen_props(this);
 
@@ -1279,15 +1353,23 @@ static int sxfe_display_config(frontend_t *this_gen,
   if(this->fullscreen_state_forced)
     fullscreen = this->fullscreen ? 1 : 0;
 
-  if(!fullscreen && (this->x.width != width || this->x.height != height)) {
-    this->x.width      = width;
-    this->x.height     = height;
+  if (xpos < 0) xpos = this->xpos;
+  if (ypos < 0) ypos = this->ypos;
+
+  if ( !fullscreen &&
+       ( this->width != width || this->height != height ||
+         this->xpos  != xpos  || this->ypos   != ypos)) {
+    this->xpos       = xpos;
+    this->ypos       = ypos;
+    this->width      = width;
+    this->height     = height;
 
     XLockDisplay(this->display);
-    XResizeWindow(this->display, this->window[0], this->x.width, this->x.height);
+    XResizeWindow(this->display, this->window[0], this->width, this->height);
+    XMoveWindow(this->display, this->window[0], this->xpos, this->ypos);
     XUnlockDisplay(this->display);
     if(!fullscreen && !this->fullscreen)
-      xine_port_send_gui_data(this->x.video_port, XINE_GUI_SEND_DRAWABLE_CHANGED,
+      xine_port_send_gui_data(this->video_port, XINE_GUI_SEND_DRAWABLE_CHANGED,
                               (void*) this->window[0]);
   }
 
@@ -1306,46 +1388,45 @@ static int sxfe_display_config(frontend_t *this_gen,
       set_above(this, this->stay_above);
     XMapRaised(this->display, this->window[this->fullscreen ? 1 : 0]);
     if(!fullscreen) {
-      XResizeWindow(this->display, this->window[0], this->x.width, this->x.height);
-      XMoveWindow(this->display, this->window[0], this->x.xpos, this->x.ypos);
+      XResizeWindow(this->display, this->window[0], this->width, this->height);
+      XMoveWindow(this->display, this->window[0], this->xpos, this->ypos);
       LOGDBG("sxfe_display_config: XMoveWindow called with x=%d and y=%d",
-             this->x.xpos, this->x.ypos);
+             this->xpos, this->ypos);
       this->check_move = 1;
       set_above(this, this->stay_above);
     } else {
       set_fullscreen_props(this);
-      XResizeWindow(this->display, this->window[1], this->x.width, this->x.height);
+      XResizeWindow(this->display, this->window[1], this->width, this->height);
       XMoveWindow(this->display, this->window[1], this->xinerama_x, this->xinerama_y);
     }
     XSync(this->display, False);
     if(XTranslateCoordinates(this->display, this->window[this->fullscreen ? 1 : 0],
                              DefaultRootWindow(this->display),
                              0, 0, &tmp_x, &tmp_y, &tmp_win)) {
-      this->x.xpos = tmp_x;
-      this->x.ypos = tmp_y;
+      this->xpos = tmp_x;
+      this->ypos = tmp_y;
     }
     XUnlockDisplay(this->display);
-    xine_port_send_gui_data(this->x.video_port, XINE_GUI_SEND_DRAWABLE_CHANGED,
+    xine_port_send_gui_data(this->video_port, XINE_GUI_SEND_DRAWABLE_CHANGED,
                             (void*) this->window[this->fullscreen ? 1 : 0]);
   }
 
 #if 0
   if(!modeswitch && strcmp(modeline, this->modeline)) {
-    free(this->modeline);
-    this->modeline = strdup(modeline ?: "");
+    strn0cpy(this->modeline, modeline, sizeof(this->modeline));
     /* #warning TODO - switch vmode */
   }
 #endif
 
 /*this->vmode_switch = modeswitch;*/
-  this->x.aspect = aspect;
-  this->x.scale_video = scale_video;
-  this->x.field_order = field_order ? 1 : 0;
+  this->aspect = aspect;
+  this->scale_video = scale_video;
+  this->field_order = field_order ? 1 : 0;
 
   return 1;
 }
 
-static void sxfe_toggle_fullscreen(fe_t *this_gen)
+static void sxfe_toggle_fullscreen(frontend_t *this_gen)
 {
   sxfe_t *this = (sxfe_t*)this_gen;
 
@@ -1353,19 +1434,19 @@ static void sxfe_toggle_fullscreen(fe_t *this_gen)
   this->fullscreen_state_forced = 0;
 
   if(!this->fullscreen) {
-    this->origwidth  = this->x.width;
-    this->origheight = this->x.height;
-    this->origxpos = this->x.xpos;
-    this->origypos = this->x.ypos;
+    this->origwidth  = this->width;
+    this->origheight = this->height;
+    this->origxpos = this->xpos;
+    this->origypos = this->ypos;
   } else {
-    this->x.xpos = this->origxpos;
-    this->x.ypos = this->origypos;
+    this->xpos = this->origxpos;
+    this->ypos = this->origypos;
   }
 
   this->fe.fe_display_config((frontend_t*)this, -1, -1, this->origwidth, this->origheight,
                              this->fullscreen ? 0 : 1,
                              0/*this->vmode_switch*/, NULL/*this->modeline*/,
-                             this->x.aspect, this->x.scale_video, this->x.field_order);
+                             this->aspect, this->scale_video, this->field_order);
 
   this->fullscreen_state_forced = !force;
 }
@@ -1427,15 +1508,16 @@ static void XKeyEvent_handler(sxfe_t *this, XKeyEvent *kev)
           fe_event = "TOGGLE_DEINTERLACE";
         break;
       case XK_Escape:
-        if (!this->x.keypress) /* ESC exits only in remote mode */
+        if (!this->keypress) { /* ESC exits only in remote mode */
           fe_event = "QUIT";
+        }
         break;
       default:;
     }
     if (fe_event)
-      this->x.fe.send_event((frontend_t*)this, fe_event);
+      this->fe.send_event((frontend_t*)this, fe_event);
     else if (!this->no_x_kbd)
-      this->x.fe.send_input_event((frontend_t*)this, "XKeySym", XKeysymToString(ks), 0, 0);
+      this->fe.send_input_event((frontend_t*)this, "XKeySym", XKeysymToString(ks), 0, 0);
   }
 }
 
@@ -1451,21 +1533,22 @@ static void XConfigureEvent_handler(sxfe_t *this, XConfigureEvent *cev)
     hud_osd_resize(this, cev->window, cev->width, cev->height);
 #endif
 
-  if (this->x.width != cev->width || this->x.height != cev->height) {
-    char str[128];
-    snprintf(str, sizeof(str), "INFO WINDOW %dx%d", this->x.width, this->x.height);
-    this->x.fe.send_event((frontend_t*)this, str);
-  }
-
   /* update video window size */
-  this->x.width  = cev->width;
-  this->x.height = cev->height;
+  if (this->width != cev->width || this->height != cev->height) {
+    this->width  = cev->width;
+    this->height = cev->height;
+
+    /* inform VDR about new size */
+    char str[128];
+    snprintf(str, sizeof(str), "INFO WINDOW %dx%d", this->width, this->height);
+    this->fe.send_event((frontend_t*)this, str);
+  }
 
   if(this->window[0] == cev->window && this->check_move) {
     LOGDBG("ConfigureNotify reveived with x=%d, y=%d, check_move=%d",
            cev->x, cev->y, this->check_move);
     this->check_move = 0;
-    if(this->x.xpos != cev->x && this->x.ypos != cev->y) {
+    if(this->xpos != cev->x && this->ypos != cev->y) {
       XLockDisplay (this->display);
       XMoveWindow(this->display, this->window[0], cev->x, cev->y);
       XUnlockDisplay (this->display);
@@ -1480,16 +1563,16 @@ static void XConfigureEvent_handler(sxfe_t *this, XConfigureEvent *cev)
       if(XTranslateCoordinates(this->display, cev->window,
                                DefaultRootWindow(this->display),
                                0, 0, &tmp_x, &tmp_y, &tmp_win)) {
-        this->x.xpos = tmp_x;
-        this->x.ypos = tmp_y;
+        this->xpos = tmp_x;
+        this->ypos = tmp_y;
       }
       XUnlockDisplay(this->display);
     }
   } else {
     if(!this->fullscreen) {
       /* update video window position */
-      this->x.xpos = cev->x;
-      this->x.ypos = cev->y;
+      this->xpos = cev->x;
+      this->ypos = cev->y;
     }
   }
 }
@@ -1515,8 +1598,8 @@ static void XMotionEvent_handler(sxfe_t *this, XMotionEvent *mev)
                           DefaultRootWindow(this->display),
                           0, 0, &xpos, &ypos, &tmp_win);
 
-    this->x.xpos = (xpos += mev->x_root - this->dragging_x);
-    this->x.ypos = (ypos += mev->y_root - this->dragging_y);
+    this->xpos = (xpos += mev->x_root - this->dragging_x);
+    this->ypos = (ypos += mev->y_root - this->dragging_y);
     this->dragging_x = mev->x_root;
     this->dragging_y = mev->y_root;
 
@@ -1542,7 +1625,7 @@ static void XButtonEvent_handler(sxfe_t *this, XButtonEvent *bev)
       /* Double-click toggles between fullscreen and windowed mode */
       if(bev->time - this->prev_click_time < DOUBLECLICK_TIME) {
         /* Toggle fullscreen */
-        sxfe_toggle_fullscreen((fe_t*)this);
+        this->toggle_fullscreen_cb((frontend_t*)this);
         this->prev_click_time = 0; /* don't react to third click ... */
       } else {
         this->prev_click_time = bev->time;
@@ -1608,7 +1691,7 @@ static int sxfe_run(frontend_t *this_gen)
     switch (event.type) {
       case Expose:
         if (event.xexpose.count == 0)
-          xine_port_send_gui_data (this->x.video_port, XINE_GUI_SEND_EXPOSE_EVENT, &event);
+          xine_port_send_gui_data (this->video_port, XINE_GUI_SEND_EXPOSE_EVENT, &event);
         break;
 
       case ConfigureNotify:
@@ -1649,17 +1732,17 @@ static int sxfe_run(frontend_t *this_gen)
           /* we got a window deletion message from out window manager.*/
           LOGDBG("ClientMessage: WM_DELETE_WINDOW");
 
-          this->x.fe.send_event((frontend_t*)this, "QUIT");
+          this->fe.send_event((frontend_t*)this, "QUIT");
         }
         break;
       }
     }
 
     if (event.type == this->completion_event)
-      xine_port_send_gui_data (this->x.video_port, XINE_GUI_SEND_COMPLETION_EVENT, &event);
+      xine_port_send_gui_data (this->video_port, XINE_GUI_SEND_COMPLETION_EVENT, &event);
   }
 
-  return !this->x.fe.xine_is_finished((frontend_t*)this, 0);
+  return !this->fe.xine_is_finished((frontend_t*)this, 0);
 }
 
 static void sxfe_display_close(frontend_t *this_gen)
@@ -1669,7 +1752,7 @@ static void sxfe_display_close(frontend_t *this_gen)
   if(!this)
     return;
 
-  if(this->x.xine)
+  if(this->xine)
     this->fe.xine_exit(this_gen);
 
   if(this->display) {
@@ -1698,8 +1781,8 @@ static void sxfe_display_close(frontend_t *this_gen)
     this->display = NULL;
   }
 
-  free(this->x.aspect_controller);
-  this->x.aspect_controller = NULL;
+  free(this->aspect_controller);
+  this->aspect_controller = NULL;
 #if 0
   free(this->modeline);
   this->modeline = NULL;
@@ -1740,10 +1823,10 @@ static int sxfe_xine_play(frontend_t *this_gen)
   int result = this->fe_xine_play(this_gen);
 
 #ifdef HAVE_XRENDER
-  if (result && this->x.input_plugin && this->hud) {
+  if (result && this->input_plugin && this->hud) {
     LOGDBG("sxfe_xine_play: Enabling HUD OSD");
-    this->x.input_plugin->f.fe_handle     = this_gen;
-    this->x.input_plugin->f.intercept_osd = hud_osd_command;
+    this->input_plugin->f.fe_handle     = this_gen;
+    this->input_plugin->f.intercept_osd = hud_osd_command;
   }
 #endif /* HAVE_XRENDER */
 
@@ -1765,7 +1848,7 @@ static frontend_t *sxfe_get_frontend(void)
   this->fe.fe_run       = sxfe_run;
   this->fe.fe_interrupt = sxfe_interrupt;
 
-  this->x.toggle_fullscreen_cb = sxfe_toggle_fullscreen;
+  this->toggle_fullscreen_cb = sxfe_toggle_fullscreen;
 
   /* override */
 
